class
	P_IFILE

inherit
	LESS_DATA
		redefine
			default_create
		end
	S_UNISTD
	S_STDLIB
	S_STRING
	S_TERMIOS
	S_SYS_IOCTL
	S_TERMCAP
	S_STDIO
	S_CTYPE
	S_LOCALE
	S_FCNTL
	S_LIBIO
	S_SYS_STAT
	S_SIGNAL
	S_TIME
	S_BITS_ERRNO
	S_REGEX

feature {NONE} -- Initialization

	default_create
		do
			create anchor.make (<<1>>)
		end

feature {ANY} -- attributes

	anchor: CE_ARRAY [PD_IFILE]

	ifiles: INTEGER_32 assign set_ifiles

	set_ifiles (a_ifiles: INTEGER_32)
		do
			ifiles := a_ifiles
		end

feature {ANY} -- routines

	del_ifile (a_h: CE_POINTER [ANY])
		local
			l_p: CE_POINTER [PD_IFILE]
			js: JUMP_STATE
		do
			create l_p.make

			if a_h.is_void then
				js.return
			end
			if not js.return_called then
				mark_inst.unmark (a_h)
				if a_h.is_equal (main_inst.curr_ifile) then
					main_inst.curr_ifile := getoff_ifile (main_inst.curr_ifile)
				end
				l_p := create {CE_POINTER [PD_IFILE]}.make_cast (a_h)
				unlink_ifile (l_p)
				free (create {CE_POINTER [ANY]}.make_cast (l_p.item.h_filename))
				free (create {CE_POINTER [ANY]}.make_cast (l_p))
			end
		end

	find_ifile (a_filename: CE_POINTER [INTEGER_8]): CE_POINTER [PD_IFILE]
		local
			l_p: CE_POINTER [PD_IFILE]
			js: JUMP_STATE
		do
			create l_p.make

			l_p := anchor.item.h_next
			from
			until
				not (not js.loop_done and then (not l_p.is_equal (anchor)))
			loop
				if (strcmp (a_filename, l_p.item.h_filename)) = 0 then
					Result := l_p
					js.return
				end
				if not js.return_called then
					l_p := l_p.item.h_next
				end
			end
			if not js.return_called then
				Result := (create {CE_POINTER [PD_IFILE]}.make_shared (default_pointer))
			end
		end

	get_filename (a_ifile: CE_POINTER [ANY]): CE_POINTER [INTEGER_8]
		local
			js: JUMP_STATE
		do
			if a_ifile.is_void then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				Result := ((create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_filename)
			end
		end

	get_filestate (a_ifile: CE_POINTER [ANY]): CE_POINTER [ANY]
		do
			Result := ((create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_filestate)
		end

	get_ifile (a_filename: CE_POINTER [INTEGER_8]; a_prev: CE_POINTER [ANY]): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [PD_IFILE]
		do
			create l_p.make

			l_p := find_ifile (a_filename)
			if l_p.is_void then
				l_p := new_ifile (a_filename, create {CE_POINTER [PD_IFILE]}.make_cast (a_prev))
			end
			Result := (create {CE_POINTER [ANY]}.make_cast (l_p))
		end

	get_index (a_ifile: CE_POINTER [ANY]): INTEGER_32
		do
			Result := ((create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_index)
		end

	get_pos (a_ifile: CE_POINTER [ANY]; a_scrpos: CE_POINTER [PD_SCRPOS])
		do
			a_scrpos.item := ((create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_scrpos).twin
		end

	getoff_ifile (a_ifile: CE_POINTER [ANY]): CE_POINTER [ANY]
		local
			l_newifile: CE_POINTER [ANY]
			js: JUMP_STATE
		do
			create l_newifile.make

			l_newifile := prev_ifile (a_ifile)
			if not l_newifile.is_void then
				Result := l_newifile
				js.return
			end
			if not js.return_called then
				l_newifile := next_ifile (a_ifile)
				if not l_newifile.is_void then
					Result := l_newifile
					js.return
				end
			end
			if not js.return_called then
				Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
			end
		end

	held_ifile (a_ifile: CE_POINTER [ANY]): INTEGER_32
		do
			Result := ((create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_hold)
		end

	hold_ifile (a_ifile: CE_POINTER [ANY]; a_incr: INTEGER_32)
		do
			(create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_hold := (create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_hold + a_incr
		end

	incr_index (a_p: CE_POINTER [PD_IFILE]; a_incr: INTEGER_32)
		local
			l_p: CE_POINTER [PD_IFILE]
		do
			create l_p.make

			l_p := a_p
			from
			until
				l_p.is_equal (anchor)
			loop
				l_p.item.h_index := l_p.item.h_index + a_incr
				l_p := l_p.item.h_next
			end
		end

	link_ifile (a_p: CE_POINTER [PD_IFILE]; a_prev: CE_POINTER [PD_IFILE])
		local
			l_prev: CE_POINTER [PD_IFILE]
		do
			create l_prev.make

			l_prev := a_prev
			if l_prev.is_void then
				l_prev := anchor
			end
			a_p.item.h_next := l_prev.item.h_next
			a_p.item.h_prev := l_prev;
			(l_prev.item.h_next).item.h_prev := a_p
			l_prev.item.h_next := a_p
			a_p.item.h_index := l_prev.item.h_index + 1
			incr_index (a_p.item.h_next, 1)
			ifiles := ifiles + 1
		end

	new_ifile (a_filename: CE_POINTER [INTEGER_8]; a_prev: CE_POINTER [PD_IFILE]): CE_POINTER [PD_IFILE]
		local
			l_p: CE_POINTER [PD_IFILE]
		do
			create l_p.make

			l_p := create {CE_POINTER [PD_IFILE]}.make_cast ((main_inst.ecalloc (1, (create {PD_IFILE}.make).structure_size.to_natural_32)))
			l_p.item.h_filename := main_inst.save (a_filename)
			l_p.item.h_scrpos.pos := (-1).to_integer_32
			l_p.item.h_opened := (0).to_integer_8
			l_p.item.h_hold := 0
			l_p.item.h_filestate := create {CE_POINTER [ANY]}.make_shared (default_pointer)
			link_ifile (l_p, a_prev)
			Result := l_p
		end

	next_ifile (a_h: CE_POINTER [ANY]): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [PD_IFILE]
			js: JUMP_STATE
		do
			create l_p.make

			if a_h.is_void then
				l_p := anchor
			else
				l_p := create {CE_POINTER [PD_IFILE]}.make_cast (a_h)
			end
			if (l_p.item.h_next).is_equal (anchor) then
				Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				Result := (create {CE_POINTER [ANY]}.make_cast (l_p.item.h_next))
			end
		end

	nifile: INTEGER_32
		do
			Result := ifiles
		end

	opened (a_ifile: CE_POINTER [ANY]): INTEGER_32
		do
			Result := ((create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_opened).to_integer_32
		end

	prev_ifile (a_h: CE_POINTER [ANY]): CE_POINTER [ANY]
		local
			l_p: CE_POINTER [PD_IFILE]
			js: JUMP_STATE
		do
			create l_p.make

			if a_h.is_void then
				l_p := anchor
			else
				l_p := create {CE_POINTER [PD_IFILE]}.make_cast (a_h)
			end
			if (l_p.item.h_prev).is_equal (anchor) then
				Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				Result := (create {CE_POINTER [ANY]}.make_cast (l_p.item.h_prev))
			end
		end

	set_filestate (a_ifile: CE_POINTER [ANY]; a_filestate: CE_POINTER [ANY])
		do
			(create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_filestate := a_filestate
		end

	set_open (a_ifile: CE_POINTER [ANY])
		do
			(create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_opened := (1).to_integer_8
		end

	store_pos (a_ifile: CE_POINTER [ANY]; a_scrpos: CE_POINTER [PD_SCRPOS])
		do
			(create {CE_POINTER [PD_IFILE]}.make_cast (a_ifile)).item.h_scrpos := (a_scrpos.item).twin
		end

	unlink_ifile (a_p: CE_POINTER [PD_IFILE])
		do
			(a_p.item.h_next).item.h_prev := a_p.item.h_prev;
			(a_p.item.h_prev).item.h_next := a_p.item.h_next
			incr_index (a_p.item.h_next, -1)
			ifiles := ifiles - 1
		end

end
