class
	P_LINENUM

inherit
	LESS_DATA
		redefine
			default_create
		end
	S_UNISTD
	S_STDLIB
	S_STRING
	S_TERMIOS
	S_SYS_IOCTL
	S_TERMCAP
	S_STDIO
	S_CTYPE
	S_LOCALE
	S_FCNTL
	S_LIBIO
	S_SYS_STAT
	S_SIGNAL
	S_TIME
	S_BITS_ERRNO
	S_REGEX

feature {NONE} -- Initialization

	default_create
		do
			create anchor___0.make (<<1>>)
			create freelist.make
			create pool.make (<<50>>)
			create spare.make
		end

feature {ANY} -- attributes

	anchor___0: CE_ARRAY [PD_LINENUM_INFO]

	freelist: CE_POINTER [PD_LINENUM_INFO] assign set_freelist

	set_freelist (a_freelist: CE_POINTER [PD_LINENUM_INFO])
		do
			freelist := a_freelist
		end

	lnloop: INTEGER_32 assign set_lnloop

	set_lnloop (a_lnloop: INTEGER_32)
		do
			lnloop := a_lnloop
		end

	loopcount: INTEGER_32 assign set_loopcount

	set_loopcount (a_loopcount: INTEGER_32)
		do
			loopcount := a_loopcount
		end

	pool: CE_ARRAY [PD_LINENUM_INFO]

	spare: CE_POINTER [PD_LINENUM_INFO] assign set_spare

	set_spare (a_spare: CE_POINTER [PD_LINENUM_INFO])
		do
			spare := a_spare
		end

	startime: INTEGER_32 assign set_startime

	set_startime (a_startime: INTEGER_32)
		do
			startime := a_startime
		end

feature {ANY} -- routines

	add_lnum (a_linenum: INTEGER_32; a_pos: INTEGER_32)
		local
			l_p: CE_POINTER [PD_LINENUM_INFO]
			l_new: CE_POINTER [PD_LINENUM_INFO]
			l_nextp: CE_POINTER [PD_LINENUM_INFO]
			l_prevp: CE_POINTER [PD_LINENUM_INFO]
			l_mingap: INTEGER_32
			js: JUMP_STATE
		do
			create l_p.make
			create l_new.make
			create l_nextp.make
			create l_prevp.make

			l_p := anchor___0.item.next
			from
			until
				js.loop_done
			loop
				if not l_p.is_equal (anchor___0) then
					if not (l_p.item.pos < a_pos) then
						js.break
					end
				else
					js.break
				end
				if not js.loop_jumped then
					if l_p.item.line = a_linenum then
						js.return
					end
					if not js.return_called then
						l_p := l_p.item.next
					end
				end
			end
			if not js.return_called then
				l_nextp := l_p
				l_prevp := l_p.item.prev
				if not freelist.is_void then
					l_new := freelist
					freelist := freelist.item.next
				else
					l_new := spare
					spare := create {CE_POINTER [PD_LINENUM_INFO]}.make_shared (default_pointer)
				end
				l_new.item.next := l_nextp
				l_new.item.prev := l_prevp
				l_new.item.pos := a_pos
				l_new.item.line := a_linenum
				l_nextp.item.prev := l_new
				l_prevp.item.next := l_new
				calcgap (l_new)
				calcgap (l_nextp)
				calcgap (l_prevp)
				if spare.is_void then
					l_mingap := (anchor___0.item.next).item.gap
					l_p := anchor___0.item.next
					from
					until
						(l_p.item.next).is_equal (anchor___0)
					loop
						if l_p.item.gap <= l_mingap then
							spare := l_p
							l_mingap := l_p.item.gap
						end
						l_p := l_p.item.next
					end
					(spare.item.next).item.prev := spare.item.prev;
					(spare.item.prev).item.next := spare.item.next
				end
			end
		end

	calcgap (a_p: CE_POINTER [PD_LINENUM_INFO])
		local
			js: JUMP_STATE
		do
			if a_p.is_equal (anchor___0) then
				js.return
			elseif (a_p.item.next).is_equal (anchor___0) then
				js.return
			end
			if not js.return_called then
				a_p.item.gap := (a_p.item.next).item.pos - (a_p.item.prev).item.pos
			end
		end

	clr_linenum
		local
			l_p: CE_POINTER [PD_LINENUM_INFO]
		do
			create l_p.make

			l_p := pool
			from
			until
				not ((l_p.c_pointer.to_integer_32).to_natural_32 < ((pool + 48).c_pointer.to_integer_32).to_natural_32)
			loop
				l_p.item.next := l_p + 1
				l_p := l_p + 1
			end
			pool[48].next := create {CE_POINTER [PD_LINENUM_INFO]}.make_shared (default_pointer)
			freelist := pool
			spare := pool + 49
			anchor___0.item.prev := anchor___0
			anchor___0.item.next := anchor___0.item.prev
			anchor___0.item.gap := (0).to_integer_32
			anchor___0.item.pos := (0).to_integer_32
			anchor___0.item.line := (1).to_integer_32
		end

	currline (a_where: INTEGER_32): INTEGER_32
		local
			l_where: INTEGER_32
			l_pos2: INTEGER_32
			l_len: INTEGER_32
			l_linenum: INTEGER_32
			js: JUMP_STATE
		do
			l_where := a_where
			l_pos2 := position_inst.position (l_where)
			l_len := ch_inst.ch_length
			from
			until
				js.loop_done
			loop
				if l_pos2 = -1 then
					if l_where >= 0 then
						if not (l_where < screen_inst.sc_height) then
							js.break
						end
					else
						js.break
					end
				else
					js.break
				end
				if not js.loop_jumped then
					l_where := l_where + 1
					l_pos2 := position_inst.position (l_where)
				end
			end
			if l_pos2 = -1 then
				l_pos2 := l_len
			end
			l_linenum := find_linenum (l_pos2)
			if l_pos2 = l_len then
				l_linenum := l_linenum - 1
			end
			Result := l_linenum
		end

	find_linenum (a_pos: INTEGER_32): INTEGER_32
		local
			l_p: CE_POINTER [PD_LINENUM_INFO]
			l_linenum: INTEGER_32
			l_cpos: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_p.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (line_inst.linenums.item).to_boolean then
						Result := (0).to_integer_32
						js.return
					end
					if not js.return_called and then (a_pos = -1) then
						Result := (0).to_integer_32
						js.return
					end
					if not js.return_called and then (a_pos <= 0) then
						Result := (1).to_integer_32
						js.return
					end
					if not js.return_called then
						l_p := anchor___0.item.next
						from
						until
							js.loop_done
						loop
							if not l_p.is_equal (anchor___0) then
								if not (l_p.item.pos < a_pos) then
									js.break
								end
							else
								js.break
							end
							if not js.loop_jumped then
								js.continue
							end
							if js.loop_increment then
								l_p := l_p.item.next
							end
						end
						if l_p.item.pos = a_pos then
							Result := (l_p.item.line)
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						startime := os_inst.get_time
						if not (l_p.is_equal (anchor___0)) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else:
						if not (a_pos - (l_p.item.prev).item.pos < l_p.item.pos - a_pos) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if: | _L:
						l_p := l_p.item.prev
						if (ch_inst.ch_seek (l_p.item.pos)).to_boolean then
							Result := (0).to_integer_32
							js.return
						end
						if not js.return_called then
							loopcount := 0
							l_linenum := l_p.item.line
							l_cpos := l_p.item.pos
							from
							until
								not (not js.loop_done and then (l_cpos < a_pos))
							loop
								l_cpos := line_inst.forw_raw_line (l_cpos, create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer))
								if ((ch_inst.sigs).bit_and (3)).to_boolean then
									Result := (0).to_integer_32
									js.return
								elseif l_cpos = -1 then
									Result := (0).to_integer_32
									js.return
								end
								if not js.return_called then
									longish
									l_linenum := l_linenum + 1
								end
							end
						end
						if not js.return_called then
							lnloop := 0
							add_lnum (l_linenum, l_cpos)
							if l_cpos > a_pos then
								l_linenum := l_linenum - 1
							end
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else:
						if (ch_inst.ch_seek (l_p.item.pos)).to_boolean then
							Result := (0).to_integer_32
							js.return
						end
						if not js.return_called then
							loopcount := 0
							l_linenum := l_p.item.line
							l_cpos := l_p.item.pos
							from
							until
								not (not js.loop_done and then (l_cpos > a_pos))
							loop
								l_cpos := line_inst.back_raw_line (l_cpos, create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer))
								if ((ch_inst.sigs).bit_and (3)).to_boolean then
									Result := (0).to_integer_32
									js.return
								elseif l_cpos = -1 then
									Result := (0).to_integer_32
									js.return
								end
								if not js.return_called then
									longish
									l_linenum := l_linenum - 1
								end
							end
						end
						if not js.return_called then
							lnloop := 0
							add_lnum (l_linenum, l_cpos)
						end
				when 8 then -- if done | if done
						if not js.return_called then
							Result := l_linenum
						end
				when 9 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	find_pos (a_linenum: INTEGER_32): INTEGER_32
		local
			l_p: CE_POINTER [PD_LINENUM_INFO]
			l_cpos: INTEGER_32
			l_clinenum: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_p.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_linenum <= 1 then
						Result := (0).to_integer_32
						js.return
					end
					if not js.return_called then
						l_p := anchor___0.item.next
						from
						until
							js.loop_done
						loop
							if not l_p.is_equal (anchor___0) then
								if not (l_p.item.line < a_linenum) then
									js.break
								end
							else
								js.break
							end
							if not js.loop_jumped then
								js.continue
							end
							if js.loop_increment then
								l_p := l_p.item.next
							end
						end
						if l_p.item.line = a_linenum then
							Result := (l_p.item.pos)
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 8 -- skip if block
					end
				when 1 then -- if:
						if not (l_p.is_equal (anchor___0)) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 5 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 4 then -- else:
						if not (a_linenum - (l_p.item.prev).item.line < l_p.item.line - a_linenum) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if: | _L:
						l_p := l_p.item.prev
						if (ch_inst.ch_seek (l_p.item.pos)).to_boolean then
							Result := (-1).to_integer_32
							js.return
						end
						if not js.return_called then
							l_clinenum := l_p.item.line
							l_cpos := l_p.item.pos
							from
							until
								not (not js.loop_done and then (l_clinenum < a_linenum))
							loop
								l_cpos := line_inst.forw_raw_line (l_cpos, create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer))
								if ((ch_inst.sigs).bit_and (3)).to_boolean then
									Result := (-1).to_integer_32
									js.return
								elseif l_cpos = -1 then
									Result := (-1).to_integer_32
									js.return
								end
								if not js.return_called then
									l_clinenum := l_clinenum + 1
								end
							end
						end
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 7 then -- else:
						if (ch_inst.ch_seek (l_p.item.pos)).to_boolean then
							Result := (-1).to_integer_32
							js.return
						end
						if not js.return_called then
							l_clinenum := l_p.item.line
							l_cpos := l_p.item.pos
							from
							until
								not (not js.loop_done and then (l_clinenum > a_linenum))
							loop
								l_cpos := line_inst.back_raw_line (l_cpos, create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer))
								if ((ch_inst.sigs).bit_and (3)).to_boolean then
									Result := (-1).to_integer_32
									js.return
								elseif l_cpos = -1 then
									Result := (-1).to_integer_32
									js.return
								end
								if not js.return_called then
									l_clinenum := l_clinenum - 1
								end
							end
						end
				when 8 then -- if done | if done
						if not js.return_called then
							add_lnum (l_clinenum, l_cpos)
							Result := l_cpos
						end
				when 9 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	longish
		do
			if loopcount >= 0 then
				loopcount := loopcount + 1
				if loopcount > 100 then
					loopcount := 0
					if (os_inst.get_time) >= startime + 2 then
						longloopmessage
						loopcount := -1
					end
				end
			end
		end

	longloopmessage
		do
			output_inst.ierror (ce_string ("Calculating line numbers"), create {CE_POINTER [PD_PARG]}.make_shared (default_pointer))
			lnloop := 1
		end

end
