class
	P_PTIMER

inherit
	WGET_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_SYS_SELECT
	S_LIBINTL
	S_STRING
	S_STDIO
	S_STDLIB
	S_UNISTD
	S_ASSERT
	S_SYS_SOCKET
	S_NETINET_IN
	S_BITS_ERRNO
	S_LIBIO
	S_TIME
	S_SYS_STAT
	S_FNMATCH
	S_CTYPE
	S_ARPA_INET
	S_NETDB
	S_LOCALE
	S_PWD
	S_GETOPT
	S_SIGNAL
	S_WCHAR
	S_SYS_TIME
	S_SYS_MMAN
	S_UTIME
	S_FCNTL
	S_SYS_IOCTL
	S_LANGINFO
	S_STDIO_EXT
	S_TERMIOS
	S_WCTYPE

feature {NONE} -- Initialization

	default_create
		do
			create clocks.make (<<2>>)
		end

feature {ANY} -- attributes

	clocks: CE_ARRAY [PD_CLOCKS]

	init_done: BOOLEAN assign set_init_done

	set_init_done (a_init_done: BOOLEAN)
		do
			init_done := a_init_done
		end

	posix_clock_id: INTEGER_32 assign set_posix_clock_id

	set_posix_clock_id (a_posix_clock_id: INTEGER_32)
		do
			posix_clock_id := a_posix_clock_id
		end

	posix_clock_resolution: REAL_64 assign set_posix_clock_resolution

	set_posix_clock_resolution (a_posix_clock_resolution: REAL_64)
		do
			posix_clock_resolution := a_posix_clock_resolution
		end

feature {ANY} -- routines

	posix_diff (a_pst1: CE_POINTER [SD_TIMESPEC]; a_pst2: CE_POINTER [SD_TIMESPEC]): REAL_64
		do
			Result := ((a_pst1.item.tv_sec - a_pst2.item.tv_sec).to_double + (a_pst1.item.tv_nsec - a_pst2.item.tv_nsec).to_double / 1.e9)
		end

	posix_init
		local
			l_i: NATURAL_32
			l_r: CE_ARRAY [SD_TIMESPEC]
			l_tmp___2: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_r.make (<<1>>)
			create l_tmp___2.make

			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_i < ((create {PD_CLOCKS}.make).structure_size.to_natural_32 * (2).to_natural_32) // (create {PD_CLOCKS}.make).structure_size.to_natural_32))
			loop
				if clocks[l_i.to_integer_32].sysconf_name /= -1 then
					if (sysconf (clocks[l_i.to_integer_32].sysconf_name)) < 0 then
						js.continue
					end
				end
				if not js.loop_jumped then
					if (clock_getres (clocks[l_i.to_integer_32].id, l_r)) < 0 then
						js.continue
					end
				end
				if not js.loop_jumped then
					posix_clock_id := clocks[l_i.to_integer_32].id
					posix_clock_resolution := (l_r.item.tv_sec).to_double + (l_r.item.tv_nsec).to_double / 1.e9
					if posix_clock_resolution = (0).to_double then
						posix_clock_resolution := 1.e-3
					end
					js.break
				end
				if js.loop_increment then
					l_i := l_i + 1
				end
			end
			if l_i = ((create {PD_CLOCKS}.make).structure_size.to_natural_32 * (2).to_natural_32) // (create {PD_CLOCKS}.make).structure_size.to_natural_32 then
				l_tmp___2 := strerror ((errno_location).item)
				log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("Cannot get REALTIME clock frequency: %%s%N"))), l_tmp___2])
				posix_clock_id := 0
				posix_clock_resolution := 1.e-3
			end
		end

	posix_measure (a_pst: CE_POINTER [SD_TIMESPEC])
		do
			clock_gettime (posix_clock_id, a_pst).do_nothing
		end

	posix_resolution: REAL_64
		do
			Result := posix_clock_resolution
		end

	ptimer_destroy (a_pt: CE_POINTER [PD_PTIMER])
		do
			free (create {CE_POINTER [ANY]}.make_cast (a_pt))
		end

	ptimer_measure (a_pt: CE_POINTER [PD_PTIMER]): REAL_64
		local
			l_now: CE_ARRAY [SD_TIMESPEC]
			l_elapsed: REAL_64
		do
			create l_now.make (<<1>>)

			posix_measure (l_now)
			l_elapsed := a_pt.item.elapsed_pre_start + (posix_diff (l_now, create {CE_POINTER [SD_TIMESPEC]}.make_shared (a_pt.c_pointer + (create {PD_PTIMER}.make).offset_of ("start"))))
			if l_elapsed < a_pt.item.elapsed_last then
				a_pt.item.start := (l_now.item).twin
				a_pt.item.elapsed_pre_start := a_pt.item.elapsed_last
				l_elapsed := a_pt.item.elapsed_last
			end
			a_pt.item.elapsed_last := l_elapsed
			Result := l_elapsed
		end

	ptimer_new: CE_POINTER [PD_PTIMER]
		local
			l_pt: CE_POINTER [PD_PTIMER]
		do
			create l_pt.make

			l_pt := create {CE_POINTER [PD_PTIMER]}.make_cast ((xmalloc_inst.xcalloc ((1).to_natural_32, (create {PD_PTIMER}.make).structure_size.to_natural_32)))
			if not init_done then
				init_done := (1).to_boolean
				posix_init
			end
			ptimer_reset (l_pt)
			Result := l_pt
		end

	ptimer_read (a_pt: CE_POINTER [PD_PTIMER]): REAL_64
		do
			Result := (a_pt.item.elapsed_last)
		end

	ptimer_reset (a_pt: CE_POINTER [PD_PTIMER])
		do
			posix_measure (create {CE_POINTER [SD_TIMESPEC]}.make_shared (a_pt.c_pointer + (create {PD_PTIMER}.make).offset_of ("start")))
			a_pt.item.elapsed_last := (0).to_double
			a_pt.item.elapsed_pre_start := (0).to_double
		end

	ptimer_resolution: REAL_64
		do
			Result := (posix_resolution)
		end

end
