class
	P_RETR

inherit
	WGET_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_SYS_SELECT
	S_LIBINTL
	S_STRING
	S_STDIO
	S_STDLIB
	S_UNISTD
	S_ASSERT
	S_SYS_SOCKET
	S_NETINET_IN
	S_BITS_ERRNO
	S_LIBIO
	S_TIME
	S_SYS_STAT
	S_FNMATCH
	S_CTYPE
	S_ARPA_INET
	S_NETDB
	S_LOCALE
	S_PWD
	S_GETOPT
	S_SIGNAL
	S_WCHAR
	S_SYS_TIME
	S_SYS_MMAN
	S_UTIME
	S_FCNTL
	S_SYS_IOCTL
	S_LANGINFO
	S_STDIO_EXT
	S_TERMIOS
	S_WCTYPE

feature {NONE} -- Initialization

	default_create
		do
			create dlbuf.make (<<16_384>>)
			create limit_data.make (<<1>>)
			create output_stream.make
			create rate_names.make (<<4>>)
			create res.make (<<20>>)
			create rewritten_storage.make (<<1_024>>)
		end

feature {ANY} -- attributes

	dlbuf: CE_ARRAY [INTEGER_8]

	first_retrieval: BOOLEAN assign set_first_retrieval

	set_first_retrieval (a_first_retrieval: BOOLEAN)
		do
			first_retrieval := a_first_retrieval
		end

	limit_data: CE_ARRAY [PD_LIMIT_DATA]

	output_stream: CE_POINTER [SD_IO_FILE] assign set_output_stream

	set_output_stream (a_output_stream: CE_POINTER [SD_IO_FILE])
		do
			output_stream := a_output_stream
		end

	output_stream_regular: BOOLEAN assign set_output_stream_regular

	set_output_stream_regular (a_output_stream_regular: BOOLEAN)
		do
			output_stream_regular := a_output_stream_regular
		end

	rate_names: CE_ARRAY [CE_POINTER [INTEGER_8]]

	res: CE_ARRAY [INTEGER_8]

	rewritten_storage: CE_ARRAY [INTEGER_8]

	total_download_time: REAL_64 assign set_total_download_time

	set_total_download_time (a_total_download_time: REAL_64)
		do
			total_download_time := a_total_download_time
		end

	total_downloaded_bytes: INTEGER_64 assign set_total_downloaded_bytes

	set_total_downloaded_bytes (a_total_downloaded_bytes: INTEGER_64)
		do
			total_downloaded_bytes := a_total_downloaded_bytes
		end

feature {ANY} -- routines

	calc_rate (a_bytes: INTEGER_64; a_secs: REAL_64; a_units: CE_POINTER [INTEGER_32]): REAL_64
		local
			l_secs: REAL_64
			l_dlrate: REAL_64
		do
			l_secs := a_secs
			if not (l_secs >= (0).to_double) then
				assert_fail (ce_string ("secs >= 0"), ce_string ("retr.c"), {NATURAL_32} 562, ce_string ("calc_rate"))
			end
			if not (a_bytes >= {INTEGER_64} 0) then
				assert_fail (ce_string ("bytes >= 0"), ce_string ("retr.c"), {NATURAL_32} 563, ce_string ("calc_rate"))
			end
			if l_secs = (0).to_double then
				l_secs := (ptimer_inst.ptimer_resolution) / 2.0
			end
			l_dlrate := a_bytes.to_double / l_secs
			if l_dlrate < 1024.0 then
				a_units.item := 0
			elseif l_dlrate < 1024.0 * 1024.0 then
				a_units.item := 1
				l_dlrate := l_dlrate / (1024.0)
			elseif l_dlrate < (1024.0 * 1024.0) * 1024.0 then
				a_units.item := 2
				l_dlrate := l_dlrate / (1024.0 * 1024.0)
			else
				a_units.item := 3
				l_dlrate := l_dlrate / ((1024.0 * 1024.0) * 1024.0)
			end
			Result := l_dlrate
		end

	fd_read_body (a_fd: INTEGER_32; a_out: CE_POINTER [SD_IO_FILE]; a_toread: INTEGER_64; a_startpos: INTEGER_64; a_qtyread: CE_POINTER [INTEGER_64]; a_qtywritten: CE_POINTER [INTEGER_64]; a_elapsed: CE_POINTER [REAL_64]; a_flags: INTEGER_32): INTEGER_32
		local
			l_ret: INTEGER_32
			l_dlbufsize: INTEGER_32
			l_timer: CE_POINTER [PD_PTIMER]
			l_last_successful_read_tm: REAL_64
			l_progress: CE_POINTER [ANY]
			l_progress_interactive: BOOLEAN
			l_exact: BOOLEAN
			l_skip: CE_ARRAY [INTEGER_64]
			l_sum_read: INTEGER_64
			l_sum_written: CE_ARRAY [INTEGER_64]
			l_start: INTEGER_64
			l_tmp: INTEGER_64
			l_rdsize: INTEGER_32
			l_tmp___0: INTEGER_64
			l_tmp___1: INTEGER_64
			l_tmout: REAL_64
			l_waittm: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_timer.make
			create l_progress.make
			create l_skip.make (<<1>>)
			create l_sum_written.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_ret := 0
					l_dlbufsize := ({PLATFORM}.Integer_8_bytes.to_natural_32 * (16_384).to_natural_32).to_integer_32
					l_timer := create {CE_POINTER [PD_PTIMER]}.make_shared (default_pointer)
					l_last_successful_read_tm := (0).to_double
					l_progress := create {CE_POINTER [ANY]}.make_shared (default_pointer)
					l_progress_interactive := (0).to_boolean
					l_exact := (((a_flags).bit_and (1)).to_boolean)
					l_skip.item := (0).to_integer_64
					l_sum_read := (0).to_integer_64
					l_sum_written.item := (0).to_integer_64
					if ((a_flags).bit_and (2)).to_boolean then
						l_skip.item := a_startpos
					end
					if (options_inst.opt.item.verbose).to_boolean then
						if (l_skip.item).to_boolean then
							l_tmp := (0).to_integer_64
						else
							l_tmp := a_startpos
						end
						l_start := l_tmp
						l_progress := progress_inst.progress_create (l_start, l_start + a_toread)
						l_progress_interactive := progress_inst.progress_interactive_p (l_progress)
					end
					if (options_inst.opt.item.limit_rate).to_boolean then
						limit_bandwidth_reset
					end
					if (not l_progress.is_void) then
						l_timer := ptimer_inst.ptimer_new
						l_last_successful_read_tm := (0).to_double
					elseif (options_inst.opt.item.limit_rate).to_boolean then
						l_timer := ptimer_inst.ptimer_new
						l_last_successful_read_tm := (0).to_double
					elseif (not a_elapsed.is_void) then
						l_timer := ptimer_inst.ptimer_new
						l_last_successful_read_tm := (0).to_double
					end
					if (options_inst.opt.item.limit_rate).to_boolean then
						if options_inst.opt.item.limit_rate < l_dlbufsize.to_integer_64 then
							l_dlbufsize := (options_inst.opt.item.limit_rate).to_integer_32
						end
					end
				when 1 then -- loop condition:
					if js.loop_done then
						l_pos := -1 + 16 -- skip loop
					end
				when 2 then -- loop body:
						if l_exact then
							if not (l_sum_read < a_toread) then
								js.break
							end
						end
						if not js.loop_jumped then
							if l_exact then
								if a_toread - l_sum_read <= l_dlbufsize.to_integer_64 then
									l_tmp___0 := a_toread - l_sum_read
								else
									l_tmp___0 := l_dlbufsize.to_integer_64
								end
								l_tmp___1 := l_tmp___0
							else
								l_tmp___1 := l_dlbufsize.to_integer_64
							end
							l_rdsize := l_tmp___1.to_integer_32
							l_tmout := options_inst.opt.item.read_timeout
							if l_progress_interactive then
								l_tmout := 0.95
								if (options_inst.opt.item.read_timeout).truncated_to_integer.to_boolean then
									l_waittm := (ptimer_inst.ptimer_read (l_timer)) - l_last_successful_read_tm
									if l_waittm + l_tmout > options_inst.opt.item.read_timeout then
										l_tmout := options_inst.opt.item.read_timeout - l_waittm
										if l_tmout < (0).to_double then
											l_ret := -1;
											(errno_location).item := 110
											js.break
										end
									end
								end
							end
						end
						if js.loop_jumped then
							l_pos := -1 + 7 -- skip if block
						end
				when 3 then -- if:
						l_ret := connect_inst.fd_read (a_fd, dlbuf, l_rdsize, l_tmout)
						if not l_progress_interactive then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						if l_ret < 0 then
							if (errno_location).item = 110 then
								l_ret := 0
							else
								l_pos := -1 + 6 -- goto _L___0
							end
						else
							l_pos := -1 + 6 -- goto _L___0
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else: | _L___0:
						if l_ret <= 0 then
							js.break
						end
				when 7 then -- if done
						if js.loop_jumped then
							l_pos := -1 + 15 -- skip if block
						end
				when 8 then -- if:
						if l_progress.is_void then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						l_pos := -1 + 12 -- goto _L___1
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 11 then -- else:
						if not ((options_inst.opt.item.limit_rate).to_boolean) then
							l_pos := -1 + 13 -- skip if block
						end
				when 12 then -- if: | _L___1:
						ptimer_inst.ptimer_measure (l_timer).do_nothing
						if l_ret > 0 then
							l_last_successful_read_tm := ptimer_inst.ptimer_read (l_timer)
						end
				when 13 then -- if done
						if l_ret > 0 then
							l_sum_read := l_sum_read + l_ret.to_integer_64
							if not (write_data (a_out, dlbuf, l_ret, l_skip, l_sum_written)).to_boolean then
								l_ret := -2
								l_pos := -1 + 17 -- goto out
							end
						end
				when 14 then
						if (options_inst.opt.item.limit_rate).to_boolean then
							limit_bandwidth (l_ret.to_integer_64, l_timer)
						end
						if (not l_progress.is_void) then
							progress_inst.progress_update (l_progress, l_ret.to_integer_64, (ptimer_inst.ptimer_read (l_timer)))
						end
				when 15 then -- if done
						l_pos := -1 + 1 -- back to loop condition
				when 16 then -- loop done
					if l_ret < -1 then
						l_ret := -1
					end
				when 17 then -- out:
					if (not l_progress.is_void) then
						progress_inst.progress_finish (l_progress, (ptimer_inst.ptimer_read (l_timer)))
					end
					if (not a_elapsed.is_void) then
						a_elapsed.item := ptimer_inst.ptimer_read (l_timer)
					end
					if (not l_timer.is_void) then
						ptimer_inst.ptimer_destroy (l_timer)
					end
					if (not a_qtyread.is_void) then
						a_qtyread.item := a_qtyread.item + l_sum_read
					end
					if (not a_qtywritten.is_void) then
						a_qtywritten.item := a_qtywritten.item + (l_sum_written.item)
					end
					Result := l_ret
				when 18 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	fd_read_hunk (a_fd: INTEGER_32; a_terminator: CE_ROUTINE; a_sizehint: INTEGER_32; a_maxsize: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_bufsize___0: INTEGER_32
			l_hunk: CE_POINTER [INTEGER_8]
			l_tail: INTEGER_32
			l_end: CE_POINTER [INTEGER_8]
			l_pklen: INTEGER_32
			l_rdlen: INTEGER_32
			l_remain: INTEGER_32
			js: JUMP_STATE
		do
			create l_hunk.make
			create l_end.make

			l_bufsize___0 := a_sizehint
			l_hunk := create {CE_POINTER [INTEGER_8]}.make_cast ((xmalloc_inst.xmalloc (l_bufsize___0.to_natural_32)))
			l_tail := 0
			if a_maxsize.to_boolean then
				if not (a_maxsize >= l_bufsize___0) then
					assert_fail (ce_string ("!maxsize || maxsize >= bufsize"), ce_string ("retr.c"), {NATURAL_32} 410, ce_string ("fd_read_hunk"))
				end
			end
			from
			until
				js.loop_done
			loop
				l_pklen := connect_inst.fd_peek (a_fd, l_hunk + l_tail, ((l_bufsize___0 - 1) - l_tail.to_integer_32).to_integer_32, (-1).to_double)
				if l_pklen < 0 then
					free (create {CE_POINTER [ANY]}.make_cast (l_hunk))
					Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					js.return
				end
				if not js.return_called then
					l_end := create {CE_POINTER [INTEGER_8]}.make_cast (a_terminator.pointer_item ([l_hunk, (l_hunk + l_tail), l_pklen]))
					if (not l_end.is_void) then
						l_remain := l_end.distance ((l_hunk + l_tail))
						if not (l_remain >= 0) then
							assert_fail (ce_string ("remain >= 0"), ce_string ("retr.c"), {NATURAL_32} 431, ce_string ("fd_read_hunk"))
						end
						if l_remain = 0 then
							l_hunk[l_tail] := (('%/0/').code).to_integer_8
							Result := l_hunk
							js.return
						end
						if not js.return_called and then (l_bufsize___0 - 1 < (l_tail + l_remain).to_integer_32) then
							l_bufsize___0 := ((l_tail + l_remain) + 1).to_integer_32
							l_hunk := create {CE_POINTER [INTEGER_8]}.make_cast ((xmalloc_inst.xrealloc (create {CE_POINTER [ANY]}.make_cast (l_hunk), l_bufsize___0.to_natural_32)))
						end
					else
						l_remain := l_pklen
					end
				end
				if not js.return_called then
					l_rdlen := connect_inst.fd_read (a_fd, l_hunk + l_tail, l_remain, (0).to_double)
					if l_rdlen < 0 then
						if (not l_hunk.is_void) then
							free (create {CE_POINTER [ANY]}.make_cast (l_hunk))
						end
						Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
						js.return
					end
				end
				if not js.return_called then
					l_tail := l_tail + l_rdlen
					l_hunk[l_tail] := (('%/0/').code).to_integer_8
					if l_rdlen = 0 then
						if l_tail = 0 then
							free (create {CE_POINTER [ANY]}.make_cast (l_hunk));
							(errno_location).item := 0
							Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							js.return
						else
							Result := l_hunk
							js.return
						end
					end
				end
				if not js.return_called and then (not l_end.is_void) then
					if l_rdlen = l_remain then
						Result := l_hunk
						js.return
					end
				end
				if not js.return_called and then (l_tail.to_integer_32 = l_bufsize___0 - 1) then
					if a_maxsize.to_boolean then
						if l_bufsize___0 >= a_maxsize then
							free (create {CE_POINTER [ANY]}.make_cast (l_hunk));
							(errno_location).item := 12
							Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							js.return
						end
					end
					if not js.return_called then
						l_bufsize___0 := (l_bufsize___0).bit_shift_left ((1))
						if a_maxsize.to_boolean then
							if l_bufsize___0 > a_maxsize then
								l_bufsize___0 := a_maxsize
							end
						end
						l_hunk := create {CE_POINTER [INTEGER_8]}.make_cast ((xmalloc_inst.xrealloc (create {CE_POINTER [ANY]}.make_cast (l_hunk), l_bufsize___0.to_natural_32)))
					end
				end
			end
		end

	fd_read_line (a_fd: INTEGER_32): CE_POINTER [INTEGER_8]
		do
			Result := (fd_read_hunk (a_fd, line_terminator_address, 128, 4_096))
		end

	free_urlpos (a_l: CE_POINTER [PD_URLPOS])
		local
			l_l: CE_POINTER [PD_URLPOS]
			l_next: CE_POINTER [PD_URLPOS]
		do
			create l_l.make
			create l_next.make

			l_l := a_l
			from
			until
				l_l.is_void
			loop
				l_next := l_l.item.next
				if (not (l_l.item.url).is_void) then
					url_inst.url_free (l_l.item.url)
				end
				if (not (l_l.item.local_name).is_void) then
					free (create {CE_POINTER [ANY]}.make_cast (l_l.item.local_name))
				end
				free (create {CE_POINTER [ANY]}.make_cast (l_l))
				l_l := l_next
			end
		end

	getproxy (a_u: CE_POINTER [PD_URL]): CE_POINTER [INTEGER_8]
		local
			l_proxy: CE_POINTER [INTEGER_8]
			l_rewritten_url: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_proxy.make
			create l_rewritten_url.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_proxy := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					if not options_inst.opt.item.use_proxy then
						Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
						js.return
					end
					if not js.return_called then
						if (no_proxy_match (a_u.item.host, options_inst.opt.item.no_proxy)) then
							Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							js.return
						end
					end
					if js.return_called then
						l_pos := -1 + 6 -- skip if block
					end
				when 1 then -- if:
						inspect (a_u.item.scheme).to_integer_32 -- switch 1 head
						when 0 then l_pos := -1 + 2
						when 1 then l_pos := -1 + 3
						when 2 then l_pos := -1 + 4
						else l_pos := -1 + 5
						end
				when 2 then -- switch 1, case 0
						if (not (options_inst.opt.item.http_proxy).is_void) then
							l_proxy := options_inst.opt.item.http_proxy
						else
							l_proxy := (getenv (ce_string ("http_proxy")))
						end
						l_pos := -1 + 5 -- switch 1; break
				when 3 then -- switch 1, case 1
						if (not (options_inst.opt.item.ftp_proxy).is_void) then
							l_proxy := options_inst.opt.item.ftp_proxy
						else
							l_proxy := (getenv (ce_string ("ftp_proxy")))
						end
						l_pos := -1 + 5 -- switch 1; break
				when 4 then -- switch 1, case 2
						l_pos := -1 + 5 -- switch 1; break
				when 5 then -- switch 1, done:
						if l_proxy.is_void then
							Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							js.return
						elseif not (l_proxy[0]).to_boolean then
							Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
							js.return
						end
				when 6 then -- if done
					if not js.return_called then
						l_rewritten_url := url_inst.rewrite_shorthand_url (l_proxy)
						if (not l_rewritten_url.is_void) then
							strncpy (rewritten_storage, l_rewritten_url, ({PLATFORM}.Integer_8_bytes.to_natural_32 * (1_024).to_natural_32)).do_nothing
							rewritten_storage[(({PLATFORM}.Integer_8_bytes.to_natural_32 * (1_024).to_natural_32) - {NATURAL_32} 1).to_integer_32] := (('%/0/').code).to_integer_8
							l_proxy := rewritten_storage
						end
						Result := l_proxy
					end
				when 7 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	limit_bandwidth (a_bytes: INTEGER_64; a_timer: CE_POINTER [PD_PTIMER])
		local
			l_delta_t: REAL_64
			l_expected: REAL_64
			l_slp: REAL_64
			l_t0: REAL_64
			l_t1: REAL_64
			js: JUMP_STATE
		do
			l_delta_t := (ptimer_inst.ptimer_read (a_timer)) - limit_data.item.chunk_start
			limit_data.item.chunk_bytes := limit_data.item.chunk_bytes + a_bytes
			l_expected := (limit_data.item.chunk_bytes).to_double / (options_inst.opt.item.limit_rate).to_double
			if l_expected > l_delta_t then
				l_slp := (l_expected - l_delta_t) + limit_data.item.sleep_adjust
				if l_slp < 0.2 then
					from
					until
						js.loop_done
					loop
						if (builtin_expect ((options_inst.opt.item.debug2).to_integer, 0)).to_boolean then
							log_inst.debug_logprintf ([ce_string ("deferring a %%.2f ms sleep (%%s/%%.2f).%N"), l_slp * (1_000).to_double, (utils_inst.number_to_static_string (limit_data.item.chunk_bytes)), l_delta_t])
						end
						js.break
					end
					js.return
				end
				if not js.return_called then
					from
					until
						js.loop_done
					loop
						if (builtin_expect ((options_inst.opt.item.debug2).to_integer, 0)).to_boolean then
							log_inst.debug_logprintf ([ce_string ("%Nsleeping %%.2f ms for %%s bytes, adjust %%.2f ms%N"), l_slp * (1_000).to_double, (utils_inst.number_to_static_string (limit_data.item.chunk_bytes)), limit_data.item.sleep_adjust])
						end
						js.break
					end
					l_t0 := ptimer_inst.ptimer_read (a_timer)
					utils_inst.xsleep (l_slp)
					l_t1 := ptimer_inst.ptimer_measure (a_timer)
					limit_data.item.sleep_adjust := l_slp - (l_t1 - l_t0)
					if limit_data.item.sleep_adjust > 0.5 then
						limit_data.item.sleep_adjust := 0.5
					elseif limit_data.item.sleep_adjust < -0.5 then
						limit_data.item.sleep_adjust := -0.5
					end
				end
			end
			if not js.return_called then
				limit_data.item.chunk_bytes := (0).to_integer_64
				limit_data.item.chunk_start := ptimer_inst.ptimer_read (a_timer)
			end
		end

	limit_bandwidth_reset
		do
			memset (create {CE_POINTER [ANY]}.make_cast (limit_data), ('%/0/').code, (create {PD_LIMIT_DATA}.make).structure_size.to_natural_32).do_nothing
		end

	line_terminator (a_start: CE_POINTER [INTEGER_8]; a_peeked: CE_POINTER [INTEGER_8]; a_peeklen: INTEGER_32): CE_POINTER [INTEGER_8]
		local
			l_p___0: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_p___0.make

			l_p___0 := create {CE_POINTER [INTEGER_8]}.make_cast ((memchr (create {CE_POINTER [ANY]}.make_cast (a_peeked), ('%N').code, a_peeklen.to_natural_32)))
			if (not l_p___0.is_void) then
				Result := (l_p___0 + 1)
				js.return
			end
			if not js.return_called then
				Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
			end
		end

	line_terminator_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent line_terminator)
		end

	no_proxy_match (a_host: CE_POINTER [INTEGER_8]; a_no_proxy: CE_POINTER [CE_POINTER [INTEGER_8]]): BOOLEAN
		do
			if a_no_proxy.is_void then
				Result := (0).to_boolean
			else
				Result := (host_inst.sufmatch (a_no_proxy, a_host))
			end
		end

	printwhat (a_n1: INTEGER_32; a_n2: INTEGER_32)
		local
			l_tmp___1: CE_POINTER [INTEGER_8]
		do
			create l_tmp___1.make

			if a_n1 = a_n2 then
				l_tmp___1 := (gettext (ce_string ("Giving up.%N%N")))
			else
				l_tmp___1 := (gettext (ce_string ("Retrying.%N%N")))
			end
			log_inst.logputs ({PC_LOG_OPTIONS}.LOG_VERBOSE, l_tmp___1)
		end

	retr_rate (a_bytes: INTEGER_64; a_secs: REAL_64): CE_POINTER [INTEGER_8]
		local
			l_units: CE_ARRAY [INTEGER_32]
			l_dlrate: REAL_64
			l_tmp___0: INTEGER_32
			l_tmp___1: INTEGER_32
		do
			create l_units.make (<<1>>)

			l_dlrate := (calc_rate (a_bytes, a_secs, l_units))
			if l_dlrate >= 99.95 then
				l_tmp___1 := 0
			else
				if l_dlrate >= 9.995 then
					l_tmp___0 := 1
				else
					l_tmp___0 := 2
				end
				l_tmp___1 := l_tmp___0
			end
			sprintf ([res, ce_string ("%%.*f %%s"), l_tmp___1, l_dlrate, rate_names[l_units.item]]).do_nothing
			Result := res
		end

	retrieve_from_file (a_file: CE_POINTER [INTEGER_8]; a_html: BOOLEAN; a_count: CE_POINTER [INTEGER_32]): NATURAL_32
		local
			l_html: BOOLEAN
			l_status: NATURAL_32
			l_url_list: CE_POINTER [PD_URLPOS]
			l_cur_url: CE_POINTER [PD_URLPOS]
			l_iri: CE_POINTER [PD_IRI]
			l_input_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_url: CE_POINTER [INTEGER_8]
			l_dt: CE_ARRAY [INTEGER_32]
			l_url_err: CE_ARRAY [INTEGER_32]
			l_status___0: NATURAL_32
			l_url_parsed: CE_POINTER [PD_URL]
			l_error___0: CE_POINTER [INTEGER_8]
			l_filename: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_new_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_dt___0: CE_ARRAY [INTEGER_32]
			l_tmpiri: CE_POINTER [PD_IRI]
			l_parsed_url: CE_POINTER [PD_URL]
			l_old_follow_ftp: INTEGER_32
			l_tmp___4: CE_POINTER [PD_URL]
			l_tmp___5: CE_POINTER [PD_URL]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_url_list.make
			create l_cur_url.make
			create l_iri.make
			create l_input_file.make (<<1>>)
			create l_url.make
			create l_dt.make (<<1>>)
			create l_url_err.make (<<1>>)
			create l_url_parsed.make
			create l_error___0.make
			create l_filename.make (<<1>>)
			create l_new_file.make (<<1>>)
			create l_dt___0.make (<<1>>)
			create l_tmpiri.make
			create l_parsed_url.make
			create l_tmp___4.make
			create l_tmp___5.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_html := a_html
					l_iri := iri_inst.dummy_iri
					l_input_file.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					l_url := a_file
					l_status := {PC_UERR_T}.RETROK
					a_count.item := 0
					if (url_inst.url_has_scheme (l_url)) then
						l_url_parsed := (url_inst.url_parse (l_url, l_url_err, l_iri, (1).to_boolean))
						if l_url_parsed.is_void then
							l_error___0 := (url_inst.url_error (l_url, l_url_err.item))
							log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, ce_string ("%%s: %%s.%N"), l_url, l_error___0])
							free (create {CE_POINTER [ANY]}.make_cast (l_error___0))
							Result := ({PC_UERR_T}.URLERROR)
							js.return
						end
						if not js.return_called then
							if (options_inst.opt.item.base_href).is_void then
								options_inst.opt.item.base_href := xmalloc_inst.xstrdup (l_url)
							end
							l_status___0 := retrieve_url (l_url_parsed, l_url, l_input_file, create {CE_POINTER [CE_POINTER [INTEGER_8]]}.make_shared (default_pointer), create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer), l_dt, (0).to_boolean, l_iri, (1).to_boolean)
							if l_status___0 /= {NATURAL_32} 31 then
								Result := l_status___0
								js.return
							end
						end
						if not js.return_called then
							if ((l_dt.item).bit_and (1)).to_boolean then
								l_html := (1).to_boolean
							end
							l_iri.item.utf8_encode := options_inst.opt.item.enable_iri
							if (not (l_iri.item.orig_url).is_void) then
								free (create {CE_POINTER [ANY]}.make_cast (l_iri.item.orig_url))
							end
							l_iri.item.orig_url := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
						end
					else
						l_input_file.item := a_file
					end
					if js.return_called then
						l_pos := -1 + 20 -- skip if block
					end
				when 1 then -- if:
						if l_html then
							l_url_list := (html_url_inst.get_urls_html (l_input_file.item, create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer), create {CE_POINTER [BOOLEAN]}.make_shared (default_pointer), l_iri))
						else
							l_url_list := (html_url_inst.get_urls_file (l_input_file.item))
						end
						l_cur_url := l_url_list
				when 2 then -- loop condition:
						if not (not js.loop_done and then (not l_cur_url.is_void)) then
							l_pos := -1 + 19 -- skip loop
						end
				when 3 then -- loop body:
						l_filename.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
						l_new_file.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
						l_tmpiri := iri_inst.dummy_iri
						l_parsed_url := create {CE_POINTER [PD_URL]}.make_shared (default_pointer)
						if (l_cur_url.item.ignore_when_downloading).to_boolean then
							js.continue
						end
						if not js.loop_jumped and then ((options_inst.opt.item.quota).to_boolean) then
							if total_downloaded_bytes > options_inst.opt.item.quota then
								l_status := {PC_UERR_T}.QUOTEXC
								js.break
							end
						end
						if js.loop_jumped then
							l_pos := -1 + 18 -- skip if block
						end
				when 4 then -- if:
						if options_inst.opt.item.enable_iri then
							l_parsed_url := url_inst.url_parse ((l_cur_url.item.url).item.url, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer), l_tmpiri, (1).to_boolean)
						end
						if not (options_inst.opt.item.recursive) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L___1
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 17
				when 7 then -- else:
						if not (options_inst.opt.item.page_requisites) then
							l_pos := -1 + 16 -- skip if block
						end
				when 8 then -- if: | _L___1:
						if (l_cur_url.item.url).item.scheme = {NATURAL_32} 1 then
							l_pos := -1 + 11 -- skip if block
						end
				when 9 then -- if:
						l_pos := -1 + 12 -- goto _L___0
				when 10 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 11 then -- else:
						if (getproxy (l_cur_url.item.url)).is_void then
							l_pos := -1 + 14 -- skip if block
						end
				when 12 then -- if: | _L___0:
						l_old_follow_ftp := (options_inst.opt.item.follow_ftp).to_integer
						if (l_cur_url.item.url).item.scheme = {NATURAL_32} 1 then
							options_inst.opt.item.follow_ftp := (1).to_boolean
						end
						if (not l_parsed_url.is_void) then
							l_tmp___4 := l_parsed_url
						else
							l_tmp___4 := l_cur_url.item.url
						end
						l_status := recur_inst.retrieve_tree (l_tmp___4, l_tmpiri)
						options_inst.opt.item.follow_ftp := l_old_follow_ftp.to_boolean
				when 13 then -- if block done, skip else block:
						l_pos := -1 + 15
				when 14 then -- else:
						l_pos := -1 + 16 -- goto _L
				when 15 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 17
				when 16 then -- else: | _L:
						if (not l_parsed_url.is_void) then
							l_tmp___5 := l_parsed_url
						else
							l_tmp___5 := l_cur_url.item.url
						end
						l_status := retrieve_url (l_tmp___5, (l_cur_url.item.url).item.url, l_filename, l_new_file, create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer), l_dt___0, options_inst.opt.item.recursive, l_tmpiri, (1).to_boolean)
				when 17 then -- if done
						if (not l_parsed_url.is_void) then
							url_inst.url_free (l_parsed_url)
						end
						if (not (l_filename.item).is_void) then
							if options_inst.opt.item.delete_after then
								if (utils_inst.file_exists_p (l_filename.item)) then
									from
									until
										js.loop_done
									loop
										if (builtin_expect ((options_inst.opt.item.debug2).to_integer, 0)).to_boolean then
											log_inst.debug_logprintf ([ce_string ("Removing file due to --delete-after in retrieve_from_file():%N")])
										end
										js.break
									end
									log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_VERBOSE, (gettext (ce_string ("Removing %%s.%N"))), l_filename.item])
									if (unlink (l_filename.item)).to_boolean then
										log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, ce_string ("unlink: %%s%N"), (strerror ((errno_location).item))])
									end
									l_dt___0.item := (l_dt___0.item).bit_and ((-3))
								end
							end
						end
						if (not (l_new_file.item).is_void) then
							free (create {CE_POINTER [ANY]}.make_cast (l_new_file.item))
						end
						if (not (l_filename.item).is_void) then
							free (create {CE_POINTER [ANY]}.make_cast (l_filename.item))
						end
				when 18 then -- if done
						if js.loop_increment then
							l_cur_url := l_cur_url.item.next
							a_count.item := (a_count.item) + 1
						end
						l_pos := -1 + 2 -- back to loop condition
				when 19 then -- loop done
						free_urlpos (l_url_list)
						Result := l_status
				when 20 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	retrieve_url (a_orig_parsed: CE_POINTER [PD_URL]; a_origurl: CE_POINTER [INTEGER_8]; a_file: CE_POINTER [CE_POINTER [INTEGER_8]]; a_newloc: CE_POINTER [CE_POINTER [INTEGER_8]]; a_refurl: CE_POINTER [INTEGER_8]; a_dt: CE_POINTER [INTEGER_32]; a_recursive: BOOLEAN; a_iri: CE_POINTER [PD_IRI]; a_register_status: BOOLEAN): NATURAL_32
		local
			l_refurl: CE_POINTER [INTEGER_8]
			l_dt: CE_POINTER [INTEGER_32]
			l_recursive: BOOLEAN
			l_result: NATURAL_32
			l_url: CE_POINTER [INTEGER_8]
			l_location_changed: BOOLEAN
			l_iri_fallbacked: BOOLEAN
			l_dummy: CE_ARRAY [INTEGER_32]
			l_mynewloc: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_proxy: CE_POINTER [INTEGER_8]
			l_u: CE_POINTER [PD_URL]
			l_proxy_url: CE_POINTER [PD_URL]
			l_up_error_code: CE_ARRAY [INTEGER_32]
			l_local_file: CE_ARRAY [CE_POINTER [INTEGER_8]]
			l_redirection_count: INTEGER_32
			l_post_data_suspended: BOOLEAN
			l_saved_post_data: CE_POINTER [INTEGER_8]
			l_saved_post_file_name: CE_POINTER [INTEGER_8]
			l_pi: CE_POINTER [PD_IRI]
			l_error___0: CE_POINTER [INTEGER_8]
			l_oldrec: BOOLEAN
			l_glob: BOOLEAN
			l_construced_newloc: CE_POINTER [INTEGER_8]
			l_newloc_parsed: CE_POINTER [PD_URL]
			l_error___1: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_refurl.make
			create l_dt.make
			create l_url.make
			create l_dummy.make (<<1>>)
			create l_mynewloc.make (<<1>>)
			create l_proxy.make
			create l_u.make
			create l_proxy_url.make
			create l_up_error_code.make (<<1>>)
			create l_local_file.make (<<1>>)
			create l_saved_post_data.make
			create l_saved_post_file_name.make
			create l_pi.make
			create l_error___0.make
			create l_construced_newloc.make
			create l_newloc_parsed.make
			create l_error___1.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_refurl := a_refurl
					l_dt := a_dt
					l_recursive := a_recursive
					l_iri_fallbacked := (0).to_boolean
					l_u := a_orig_parsed
					l_redirection_count := 0
					l_post_data_suspended := (0).to_boolean
					l_saved_post_data := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					l_saved_post_file_name := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					if l_dt.is_void then
						l_dt := l_dummy
						l_dummy.item := 0
					end
					l_url := xmalloc_inst.xstrdup (a_origurl)
					if (not a_newloc.is_void) then
						a_newloc.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					end
					if (not a_file.is_void) then
						a_file.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					end
					if l_refurl.is_void then
						l_refurl := options_inst.opt.item.referer
					end
				when 1 then -- redirected:
					l_result := {PC_UERR_T}.NOCONERROR
					l_mynewloc.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					l_local_file.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
					l_proxy_url := create {CE_POINTER [PD_URL]}.make_shared (default_pointer)
					l_proxy := getproxy (l_u)
					if l_proxy.is_void then
						l_pos := -1 + 4 -- skip if block
					end
				when 2 then -- if:
						l_pi := iri_inst.dummy_iri
						l_pi.item.utf8_encode := (0).to_boolean
						l_proxy_url := url_inst.url_parse (l_proxy, l_up_error_code, create {CE_POINTER [PD_IRI]}.make_shared (default_pointer), (1).to_boolean)
						if l_proxy_url.is_void then
							l_error___0 := (url_inst.url_error (l_proxy, l_up_error_code.item))
							log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("Error parsing proxy URL %%s: %%s.%N"))), l_proxy, l_error___0])
							free (create {CE_POINTER [ANY]}.make_cast (l_url))
							free (create {CE_POINTER [ANY]}.make_cast (l_error___0))
							from
							until
								js.loop_done
							loop
								if l_post_data_suspended then
									options_inst.opt.item.post_data := l_saved_post_data
									options_inst.opt.item.post_file_name := l_saved_post_file_name
									l_post_data_suspended := (0).to_boolean
								end
								js.break
							end
							l_result := {PC_UERR_T}.PROXERR
							l_pos := -1 + 24 -- goto bail
						end
				when 3 then
						if l_proxy_url.item.scheme /= {NATURAL_32} 0 then
							if l_proxy_url.item.scheme /= l_u.item.scheme then
								log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("Error in proxy URL %%s: Must be HTTP.%N"))), l_proxy])
								url_inst.url_free (l_proxy_url)
								free (create {CE_POINTER [ANY]}.make_cast (l_url))
								from
								until
									js.loop_done
								loop
									if l_post_data_suspended then
										options_inst.opt.item.post_data := l_saved_post_data
										options_inst.opt.item.post_file_name := l_saved_post_file_name
										l_post_data_suspended := (0).to_boolean
									end
									js.break
								end
								l_result := {PC_UERR_T}.PROXERR
								l_pos := -1 + 24 -- goto bail
							end
						end
				when 4 then
					if l_u.item.scheme /= {NATURAL_32} 0 then
						l_pos := -1 + 7 -- skip if block
					end
				when 5 then -- if:
						l_result := http_inst.http_loop (l_u, l_mynewloc, l_local_file, l_refurl, l_dt, l_proxy_url, a_iri)
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 7 then -- else:
						if l_proxy_url.is_void then
							l_pos := -1 + 10 -- skip if block
						end
				when 8 then -- if:
						if l_proxy_url.item.scheme = {NATURAL_32} 0 then
							l_result := http_inst.http_loop (l_u, l_mynewloc, l_local_file, l_refurl, l_dt, l_proxy_url, a_iri)
						else
							l_pos := -1 + 10 -- goto _L
						end
				when 9 then -- if block done, skip else block:
						l_pos := -1 + 11
				when 10 then -- else: | _L:
						if l_u.item.scheme = {NATURAL_32} 1 then
							l_oldrec := l_recursive
							l_glob := options_inst.opt.item.ftp_glob
							if l_redirection_count.to_boolean then
								l_glob := (0).to_boolean
								l_oldrec := l_glob
							end
							l_result := ftp_inst.ftp_loop (l_u, l_dt, l_proxy_url, l_recursive, l_glob)
							l_recursive := l_oldrec
							if l_redirection_count.to_boolean then
								if (not (l_local_file.item).is_void) then
									if l_u.item.scheme = {NATURAL_32} 1 then
										if (utils_inst.has_html_suffix_p (l_local_file.item)) then
											l_dt.item := (l_dt.item).bit_or ((1))
										end
									end
								end
							end
						end
				when 11 then -- if done
						if (not l_proxy_url.is_void) then
							url_inst.url_free (l_proxy_url)
							l_proxy_url := create {CE_POINTER [PD_URL]}.make_shared (default_pointer)
						end
						l_location_changed := (l_result = {NATURAL_32} 6)
						if not l_location_changed then
							l_pos := -1 + 15 -- skip if block
						end
				when 12 then -- if:
							if (l_mynewloc.item).is_void then
								assert_fail (ce_string ("mynewloc != ((void *)0)"), ce_string ("retr.c"), {NATURAL_32} 729, ce_string ("retrieve_url"))
							end
							if (not (l_local_file.item).is_void) then
								free (create {CE_POINTER [ANY]}.make_cast (l_local_file.item))
							end
							l_construced_newloc := url_inst.uri_merge (l_url, l_mynewloc.item)
							free (create {CE_POINTER [ANY]}.make_cast (l_mynewloc.item))
							l_mynewloc.item := l_construced_newloc
							a_iri.item.utf8_encode := options_inst.opt.item.enable_iri
							if (not (a_iri.item.orig_url).is_void) then
								free (create {CE_POINTER [ANY]}.make_cast (a_iri.item.orig_url))
							end
							l_newloc_parsed := url_inst.url_parse (l_mynewloc.item, l_up_error_code, a_iri, (1).to_boolean)
							if l_newloc_parsed.is_void then
								l_error___1 := (url_inst.url_error (l_mynewloc.item, l_up_error_code.item))
								log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, ce_string ("%%s: %%s.%N"), (log_inst.escnonprint_uri (l_mynewloc.item)), l_error___1])
								if not a_orig_parsed.is_equal (l_u) then
									url_inst.url_free (l_u)
								end
								free (create {CE_POINTER [ANY]}.make_cast (l_url))
								free (create {CE_POINTER [ANY]}.make_cast (l_mynewloc.item))
								free (create {CE_POINTER [ANY]}.make_cast (l_error___1))
								from
								until
									js.loop_done
								loop
									if l_post_data_suspended then
										options_inst.opt.item.post_data := l_saved_post_data
										options_inst.opt.item.post_file_name := l_saved_post_file_name
										l_post_data_suspended := (0).to_boolean
									end
									js.break
								end
								l_pos := -1 + 24 -- goto bail
							end
				when 13 then
							free (create {CE_POINTER [ANY]}.make_cast (l_mynewloc.item))
							l_mynewloc.item := xmalloc_inst.xstrdup (l_newloc_parsed.item.url)
							l_redirection_count := l_redirection_count + 1
							if l_redirection_count > options_inst.opt.item.max_redirect then
								log_inst.logprintf ([{PC_LOG_OPTIONS}.LOG_NOTQUIET, (gettext (ce_string ("%%d redirections exceeded.%N"))), options_inst.opt.item.max_redirect])
								url_inst.url_free (l_newloc_parsed)
								if not a_orig_parsed.is_equal (l_u) then
									url_inst.url_free (l_u)
								end
								free (create {CE_POINTER [ANY]}.make_cast (l_url))
								free (create {CE_POINTER [ANY]}.make_cast (l_mynewloc.item))
								from
								until
									js.loop_done
								loop
									if l_post_data_suspended then
										options_inst.opt.item.post_data := l_saved_post_data
										options_inst.opt.item.post_file_name := l_saved_post_file_name
										l_post_data_suspended := (0).to_boolean
									end
									js.break
								end
								l_result := {PC_UERR_T}.WRONGCODE
								l_pos := -1 + 24 -- goto bail
							end
				when 14 then
							free (create {CE_POINTER [ANY]}.make_cast (l_url))
							l_url := l_mynewloc.item
							if not a_orig_parsed.is_equal (l_u) then
								url_inst.url_free (l_u)
							end
							l_u := l_newloc_parsed
							if not l_post_data_suspended then
								from
								until
									js.loop_done
								loop
									l_post_data_suspended := (1).to_boolean
									l_saved_post_data := options_inst.opt.item.post_data
									l_saved_post_file_name := options_inst.opt.item.post_file_name
									options_inst.opt.item.post_data := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
									options_inst.opt.item.post_file_name := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
									js.break
								end
							end
							l_pos := -1 + 1 -- goto redirected
				when 15 then
						if not ((l_dt.item).bit_and (2)).to_boolean then
							if a_iri.item.utf8_encode then
								a_iri.item.utf8_encode := (0).to_boolean
								if not a_orig_parsed.is_equal (l_u) then
									url_inst.url_free (l_u)
								end
								l_u := url_inst.url_parse (a_origurl, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer), a_iri, (1).to_boolean)
								if (not l_u.is_void) then
									from
									until
										js.loop_done
									loop
										if (builtin_expect ((options_inst.opt.item.debug2).to_integer, 0)).to_boolean then
											log_inst.debug_logprintf ([ce_string ("[IRI fallbacking to non-utf8 for %%s%N"), (quote_inst.quote (l_url))])
										end
										js.break
									end
									l_url := xmalloc_inst.xstrdup (l_u.item.url)
									l_iri_fallbacked := (1).to_boolean
									l_pos := -1 + 1 -- goto redirected
								else
									from
									until
										js.loop_done
									loop
										if (builtin_expect ((options_inst.opt.item.debug2).to_integer, 0)).to_boolean then
											log_inst.debug_logprintf ([ce_string ("[Couldn%'t fallback to non-utf8 for %%s%N"), (quote_inst.quote (l_url))])
										end
										js.break
									end
								end
							end
						end
				when 16 then
						if (not (l_local_file.item).is_void) then
							if ((l_dt.item).bit_and (2)).to_boolean then
								convert_inst.register_download (l_u.item.url, l_local_file.item)
								if l_redirection_count.to_boolean then
									if 0 /= (strcmp (a_origurl, l_u.item.url)) then
										convert_inst.register_redirection (a_origurl, l_u.item.url)
									end
								end
								if ((l_dt.item).bit_and (1)).to_boolean then
									convert_inst.register_html (l_u.item.url, l_local_file.item)
								end
								if ((l_dt.item).bit_and (2)).to_boolean then
									convert_inst.register_download (l_u.item.url, l_local_file.item)
									if l_redirection_count.to_boolean then
										if 0 /= (strcmp (a_origurl, l_u.item.url)) then
											convert_inst.register_redirection (a_origurl, l_u.item.url)
										end
									end
									if ((l_dt.item).bit_and (1)).to_boolean then
										convert_inst.register_html (l_u.item.url, l_local_file.item)
									end
									if ((l_dt.item).bit_and (64)).to_boolean then
										convert_inst.register_css (l_u.item.url, l_local_file.item)
									end
								end
							end
						end
						if (not a_file.is_void) then
							if (not (l_local_file.item).is_void) then
								a_file.item := l_local_file.item
							else
								a_file.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
							end
						elseif (not (l_local_file.item).is_void) then
							free (create {CE_POINTER [ANY]}.make_cast (l_local_file.item))
						end
						if not a_orig_parsed.is_equal (l_u) then
							url_inst.url_free (l_u)
						end
						if not (l_redirection_count.to_boolean) then
							l_pos := -1 + 19 -- skip if block
						end
				when 17 then -- if:
							l_pos := -1 + 20 -- goto _L___0
				when 18 then -- if block done, skip else block:
							l_pos := -1 + 23
				when 19 then -- else:
							if not l_iri_fallbacked then
								l_pos := -1 + 22 -- skip if block
							end
				when 20 then -- if: | _L___0:
							if (not a_newloc.is_void) then
								a_newloc.item := l_url
							else
								free (create {CE_POINTER [ANY]}.make_cast (l_url))
							end
				when 21 then -- if block done, skip else block:
							l_pos := -1 + 23
				when 22 then -- else:
							if (not a_newloc.is_void) then
								a_newloc.item := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
							end
							free (create {CE_POINTER [ANY]}.make_cast (l_url))
				when 23 then -- if done
							from
							until
								js.loop_done
							loop
								if l_post_data_suspended then
									options_inst.opt.item.post_data := l_saved_post_data
									options_inst.opt.item.post_file_name := l_saved_post_file_name
									l_post_data_suspended := (0).to_boolean
								end
								js.break
							end
				when 24 then -- bail:
							if a_register_status then
								exits_inst.inform_exit_status (l_result)
							end
							Result := l_result
				when 25 then
							l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
				end

	rotate_backups (a_fname: CE_POINTER [INTEGER_8])
		local
			l_maxlen: INTEGER_32
			l_tmp: NATURAL_32
			l_from: CE_POINTER [INTEGER_8]
			l_to: CE_POINTER [INTEGER_8]
			l_sb: CE_ARRAY [SD_STAT]
			l_i: INTEGER_32
			js: JUMP_STATE
			l_aaf: CE_ALLOCA_FACTORY
		do
			create l_from.make
			create l_to.make
			create l_sb.make (<<1>>)

			l_tmp := strlen (a_fname)
			l_maxlen := (((l_tmp + {NATURAL_32} 1) + (utils_inst.numdigit ((options_inst.opt.item.backups).to_integer.to_integer_64)).to_natural_32) + {NATURAL_32} 1).to_integer_32
			l_from := create {CE_POINTER [INTEGER_8]}.make_cast ((l_aaf.new (l_maxlen.to_natural_32)))
			l_to := create {CE_POINTER [INTEGER_8]}.make_cast ((l_aaf.new (l_maxlen.to_natural_32)))
			if (stat (a_fname, l_sb)) = 0 then
				if ((l_sb.item.st_mode).bit_and ({NATURAL_32} 61_440)) /= {NATURAL_32} 32_768 then
					js.return
				end
			end
			if not js.return_called then
				l_i := (options_inst.opt.item.backups).to_integer
				from
				until
					not (l_i > 1)
				loop
					sprintf ([l_from, ce_string ("%%s.%%d"), a_fname, l_i - 1]).do_nothing
					sprintf ([l_to, ce_string ("%%s.%%d"), a_fname, l_i]).do_nothing
					rename2 (l_from, l_to).do_nothing
					l_i := l_i - 1
				end
				sprintf ([l_to, ce_string ("%%s.%%d"), a_fname, 1]).do_nothing
				rename2 (a_fname, l_to).do_nothing
			end
		end

	set_local_file (a_file: CE_POINTER [CE_POINTER [INTEGER_8]]; a_default_file: CE_POINTER [INTEGER_8])
		do
			if (not (options_inst.opt.item.output_document).is_void) then
				if output_stream_regular then
					a_file.item := options_inst.opt.item.output_document
				end
			else
				a_file.item := a_default_file
			end
		end

	sleep_between_retrievals (a_count: INTEGER_32)
		local
			l_waitsecs: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if first_retrieval then
						first_retrieval := (0).to_boolean
						js.return
					end
					if js.return_called then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						if not ((options_inst.opt.item.waitretry).truncated_to_integer.to_boolean) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						if a_count > 1 then
							if a_count.to_double <= options_inst.opt.item.waitretry then
								utils_inst.xsleep ((a_count - 1).to_double)
							else
								utils_inst.xsleep (options_inst.opt.item.waitretry)
							end
						else
							l_pos := -1 + 4 -- goto _L
						end
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 4 then -- else: | _L:
						if (options_inst.opt.item.wait).truncated_to_integer.to_boolean then
							if not options_inst.opt.item.random_wait then
								utils_inst.xsleep (options_inst.opt.item.wait)
							elseif a_count > 1 then
								utils_inst.xsleep (options_inst.opt.item.wait)
							else
								l_waitsecs := (0.5 + (utils_inst.random_float)) * options_inst.opt.item.wait
								from
								until
									js.loop_done
								loop
									if (builtin_expect ((options_inst.opt.item.debug2).to_integer, 0)).to_boolean then
										log_inst.debug_logprintf ([ce_string ("sleep_between_retrievals: avg=%%f,sleep=%%f%N"), options_inst.opt.item.wait, l_waitsecs])
									end
									js.break
								end
								utils_inst.xsleep (l_waitsecs)
							end
						end
				when 5 then -- if done
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	url_uses_proxy (a_u: CE_POINTER [PD_URL]): BOOLEAN
		local
			l_ret: BOOLEAN
			js: JUMP_STATE
		do
			if a_u.is_void then
				Result := (0).to_boolean
				js.return
			end
			if not js.return_called then
				l_ret := (not (getproxy (a_u)).is_void)
				Result := l_ret
			end
		end

	write_data (a_out: CE_POINTER [SD_IO_FILE]; a_buf___6: CE_POINTER [INTEGER_8]; a_bufsize___0: INTEGER_32; a_skip: CE_POINTER [INTEGER_64]; a_written: CE_POINTER [INTEGER_64]): INTEGER_32
		local
			l_buf___6: CE_POINTER [INTEGER_8]
			l_bufsize___0: INTEGER_32
			l_tmp___0: INTEGER_32
			js: JUMP_STATE
		do
			create l_buf___6.make

			l_buf___6 := a_buf___6
			l_bufsize___0 := a_bufsize___0
			if a_out.is_void then
				Result := 1
				js.return
			end
			if not js.return_called and then (a_skip.item > l_bufsize___0.to_integer_64) then
				a_skip.item := a_skip.item - l_bufsize___0.to_integer_64
				Result := 1
				js.return
			end
			if not js.return_called and then ((a_skip.item).to_boolean) then
				l_buf___6 := l_buf___6 + (a_skip.item).to_integer_32
				l_bufsize___0 := (l_bufsize___0.to_integer_64 - a_skip.item).to_integer_32
				a_skip.item := (0).to_integer_64
				if l_bufsize___0 = 0 then
					Result := 1
					js.return
				end
			end
			if not js.return_called then
				fwrite (create {CE_POINTER [ANY]}.make_cast (l_buf___6), (1).to_natural_32, l_bufsize___0.to_natural_32, a_out).do_nothing
				a_written.item := a_written.item + l_bufsize___0.to_integer_64
				fflush (a_out).do_nothing
				if (ferror (a_out)).to_boolean then
					l_tmp___0 := 0
				else
					l_tmp___0 := 1
				end
				Result := l_tmp___0
			end
		end

end
