class
	P_SELECT

inherit
	LINKS_DATA
		redefine
			default_create
		end
	S_SYS_SELECT
	S_STDLIB
	S_UNISTD
	S_STRING
	S_BITS_ERRNO
	S_SYS_SOCKET
	S_STDIO
	S_FCNTL
	S_NETINET_IN
	S_SYS_STAT
	S_SYS_UTSNAME
	S_NETDB
	S_SIGNAL
	S_PWD
	S_GRP
	S_TIME
	S_DIRENT
	S_TERMIOS
	S_SYS_IOCTL
	S_SYS_TIME
	S_SYS_WAIT
	S_UTIME

feature {NONE} -- Initialization

	default_create
		do
			create bottom_halves.make (<<1>>)
			create sa_zero.make (<<1>>)
			create signal_handlers.make (<<32>>)
			create signal_mask.make (<<32>>)
			create signal_pipe.make (<<2>>)
			create threads.make (<<1_024>>)
			create timers.make (<<1>>)
			create w_error.make (<<1>>)
			create w_read.make (<<1>>)
			create w_write.make (<<1>>)
			create x_error.make (<<1>>)
			create x_read.make (<<1>>)
			create x_write.make (<<1>>)
		end

feature {ANY} -- attributes

	bottom_halves: CE_ARRAY [PD_LIST_HEAD]

	last_time: INTEGER_32 assign set_last_time

	set_last_time (a_last_time: INTEGER_32)
		do
			last_time := a_last_time
		end

	sa_zero: CE_ARRAY [SD_SIGACTION]

	signal_handlers: CE_ARRAY [PD_SIGNAL_HANDLER]

	signal_mask: CE_ARRAY [INTEGER_32]

	signal_pipe: CE_ARRAY [INTEGER_32]

	terminate: INTEGER_32 assign set_terminate

	set_terminate (a_terminate: INTEGER_32)
		do
			terminate := a_terminate
		end

	threads: CE_ARRAY [PD_THREAD]

	timer_id: INTEGER_32 assign set_timer_id

	set_timer_id (a_timer_id: INTEGER_32)
		do
			timer_id := a_timer_id
		end

	timers: CE_ARRAY [PD_LIST_HEAD]

	w_error: CE_ARRAY [SD_FD_SET]

	w_max: INTEGER_32 assign set_w_max

	set_w_max (a_w_max: INTEGER_32)
		do
			w_max := a_w_max
		end

	w_read: CE_ARRAY [SD_FD_SET]

	w_write: CE_ARRAY [SD_FD_SET]

	x_error: CE_ARRAY [SD_FD_SET]

	x_read: CE_ARRAY [SD_FD_SET]

	x_write: CE_ARRAY [SD_FD_SET]

feature {ANY} -- routines

	check_bottom_halves
		local
			l_bh: CE_POINTER [PD_BOTTOM_HALF]
			l_fn: CE_ROUTINE
			l_data: CE_POINTER [ANY]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_bh.make
			create l_fn.make
			create l_data.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start: | rep:
					if (bottom_halves.item.next).is_equal (bottom_halves) then
						js.return
					end
					if not js.return_called then
						l_bh := create {CE_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item.prev)
						l_fn := l_bh.item.fn
						l_data := l_bh.item.data
						error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_bh));
						(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_bh.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_bh.item.prev);
						(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_bh.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_bh.item.next)
						error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_bh))
						error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c")), 91, create {CE_POINTER [ANY]}.make_cast (l_bh))
						l_fn.call ([l_data])
						l_pos := -1 + 0 -- goto rep
					end
				when 1 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	check_signals: INTEGER_32
		local
			l_i: INTEGER_32
			l_r: INTEGER_32
		do
			l_r := 0
			l_i := 0
			from
			until
				not (l_i < 32)
			loop
				if (signal_mask[l_i]).to_boolean then
					signal_mask[l_i] := 0
					if (not (signal_handlers[l_i].fn).is_void) then
						(signal_handlers[l_i].fn).call ([signal_handlers[l_i].data])
					end
					if not ((bottom_halves.item.next).is_equal (bottom_halves)) then
						check_bottom_halves
					end
					l_r := 1
				end
				l_i := l_i + 1
			end
			Result := l_r
		end

	check_timers
		local
			l_interval: INTEGER_32
			l_t: CE_POINTER [PD_TIMER]
			l_tt: CE_POINTER [PD_TIMER]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_t.make
			create l_tt.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_interval := (get_time) - last_time
					l_t := create {CE_POINTER [PD_TIMER]}.make_cast (timers.item.next)
					from
					until
						l_t.is_equal (timers)
					loop
						l_t.item.interval := l_t.item.interval - l_interval
						l_t := l_t.item.next
					end
				when 1 then -- ch:
					l_t := create {CE_POINTER [PD_TIMER]}.make_cast (timers.item.next)
				when 2 then -- loop condition:
					if not (not js.loop_done and then (not l_t.is_equal (timers))) then
						l_pos := -1 + 5 -- skip loop
					end
				when 3 then -- loop body:
						if l_t.item.interval <= 0 then
							l_tt := l_t
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_tt));
							(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_tt.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_tt.item.prev);
							(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_tt.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_tt.item.next)
							error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_tt));
							(l_tt.item.func).call ([l_tt.item.data])
							error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c")), 110, create {CE_POINTER [ANY]}.make_cast (l_tt))
							if not ((bottom_halves.item.next).is_equal (bottom_halves)) then
								check_bottom_halves
							end
							l_pos := -1 + 1 -- goto ch
						else
							js.break
						end
				when 4 then
						if not js.loop_jumped then
							l_t := l_t.item.next
						end
						l_pos := -1 + 2 -- back to loop condition
				when 5 then -- loop done
					last_time := last_time + l_interval
				when 6 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	get_handler (a_fd: INTEGER_32; a_tp: INTEGER_32): CE_POINTER [ANY]
		local
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_fd < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_fd >= 1_024) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
							links_inst.errline := 148
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("get_handler: handle %%d >= FD_SETSIZE %%d")), a_fd, 1_024])
						end
						Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
						js.return
				when 5 then -- if done
						if js.return_called then
							l_pos := -1 + 11 -- skip if block
						end
				when 6 then -- if:
							inspect a_tp -- switch 1 head
							when 0 then l_pos := -1 + 7
							when 1 then l_pos := -1 + 8
							when 2 then l_pos := -1 + 9
							when 3 then l_pos := -1 + 10
							else l_pos := -1 + 11
							end
				when 7 then -- switch 1, case 0
							Result := (create {CE_POINTER [ANY]}.make_cast (threads[a_fd].read_func))
							js.return
				when 8 then -- switch 1, case 1
							if not js.return_called then
								Result := (create {CE_POINTER [ANY]}.make_cast (threads[a_fd].write_func))
								js.return
							end
				when 9 then -- switch 1, case 2
							if not js.return_called then
								Result := (create {CE_POINTER [ANY]}.make_cast (threads[a_fd].error_func))
								js.return
							end
				when 10 then -- switch 1, case 3
							if not js.return_called then
								Result := (threads[a_fd].data)
								js.return
							end
				when 11 then -- switch 1, done: | if done
							if not js.return_called then
								from
								until
									False
								loop
									links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
									links_inst.errline := 157
									error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("get_handler: bad type %%d")), a_tp])
								end
								Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
							end
				when 12 then
							l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
				end

	get_time: INTEGER_32
		local
			l_tv: CE_ARRAY [SD_TIMEVAL]
		do
			create l_tv.make (<<1>>)

			gettimeofday (l_tv, create {CE_POINTER [SD_TIMEZONE]}.make_shared (default_pointer)).do_nothing
			Result := (l_tv.item.tv_sec * 1_000 + l_tv.item.tv_usec // 1_000)
		end

	got_signal (a_sig: INTEGER_32)
		local
			l_sv_errno: INTEGER_32
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sv_errno := (errno_location).item
					if a_sig >= 32 then
						l_pos := -1 + 4 -- goto ret
					elseif a_sig < 0 then
						l_pos := -1 + 4 -- goto ret
					end
				when 1 then
					if (signal_handlers[a_sig].fn).is_void then
						l_pos := -1 + 4 -- goto ret
					end
				when 2 then
					if (signal_handlers[a_sig].critical).to_boolean then
						(signal_handlers[a_sig].fn).call ([signal_handlers[a_sig].data])
						l_pos := -1 + 4 -- goto ret
					end
				when 3 then
					signal_mask[a_sig] := 1
				when 4 then -- ret:
					if (links_inst.can_write (signal_pipe[1])).to_boolean then
						write (signal_pipe[1], create {CE_POINTER [ANY]}.make_cast (ce_string ("x")), (1).to_natural_32).do_nothing
					end
					(errno_location).item := l_sv_errno
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	got_signal_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent got_signal)
		end

	install_signal_handler (a_sig: INTEGER_32; a_fn: CE_ROUTINE; a_data: CE_POINTER [ANY]; a_critical: INTEGER_32)
		local
			l_sa: CE_ARRAY [SD_SIGACTION]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_sa.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sa.item := (sa_zero.item).twin
					if not (a_sig >= 32) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_sig < 0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
							links_inst.errline := 240
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("bad signal number: %%d")), a_sig])
						end
						js.return
				when 5 then -- if done
						if not js.return_called then
							if a_fn.is_void then
								l_sa.item.sigaction_handler.sa_handler := create {CE_ROUTINE}.make_shared (default_pointer + 1)
							else
								l_sa.item.sigaction_handler.sa_handler := got_signal_address
							end
							sigfillset (create {CE_POINTER [SD_SIGSET_T]}.make_shared (l_sa.item.c_pointer + (create {SD_SIGACTION}.make).offset_of ("sa_mask"))).do_nothing
							l_sa.item.sa_flags := 268_435_456
							if a_fn.is_void then
								sigaction (a_sig, l_sa, create {CE_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
							end
							signal_handlers[a_sig].fn := a_fn
							signal_handlers[a_sig].data := a_data
							signal_handlers[a_sig].critical := a_critical
							if (not a_fn.is_void) then
								sigaction (a_sig, l_sa, create {CE_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
							end
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	install_timer (a_t: INTEGER_32; a_func: CE_ROUTINE; a_data: CE_POINTER [ANY]): INTEGER_32
		local
			l_tm: CE_POINTER [PD_TIMER]
			l_tt: CE_POINTER [PD_TIMER]
			l_tmp___0: INTEGER_32
			js: JUMP_STATE
		do
			create l_tm.make
			create l_tt.make

			l_tm := create {CE_POINTER [PD_TIMER]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c")), 120, (create {PD_TIMER}.make).structure_size.to_natural_32)))
			if l_tm.is_void then
				Result := (-1)
				js.return
			end
			if not js.return_called then
				l_tm.item.interval := a_t
				l_tm.item.func := a_func
				l_tm.item.data := a_data
				l_tmp___0 := timer_id
				timer_id := timer_id + 1
				l_tm.item.id := l_tmp___0
				l_tt := create {CE_POINTER [PD_TIMER]}.make_cast (timers.item.next)
				from
				until
					not (not js.loop_done and then (not l_tt.is_equal (timers)))
				loop
					if l_tt.item.interval >= a_t then
						js.break
					end
					if not js.loop_jumped then
						l_tt := l_tt.item.next
					end
				end
				from
				until
					js.loop_done
				loop
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_tt.item.prev))
					l_tm.item.next := (l_tt.item.prev).item.next
					l_tm.item.prev := l_tt.item.prev;
					(l_tt.item.prev).item.next := l_tm;
					(l_tm.item.next).item.prev := l_tm
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_tt.item.prev))
					js.break
				end
				Result := (l_tm.item.id)
			end
		end

	interruptible_signal (a_sig: INTEGER_32; a_in: INTEGER_32)
		local
			l_sa: CE_ARRAY [SD_SIGACTION]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_sa.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_sa.item := (sa_zero.item).twin
					if not (a_sig >= 32) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_sig < 0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
							links_inst.errline := 258
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("bad signal number: %%d")), a_sig])
						end
						js.return
				when 5 then -- if done
						if not js.return_called and then ((signal_handlers[a_sig].fn).is_void) then
							js.return
						end
						if not js.return_called then
							l_sa.item.sigaction_handler.sa_handler := got_signal_address
							sigfillset (create {CE_POINTER [SD_SIGSET_T]}.make_shared (l_sa.item.c_pointer + (create {SD_SIGACTION}.make).offset_of ("sa_mask"))).do_nothing
							if not a_in.to_boolean then
								l_sa.item.sa_flags := 268_435_456
							end
							sigaction (a_sig, l_sa, create {CE_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	kill_timer (a_id: INTEGER_32)
		local
			l_tm: CE_POINTER [PD_TIMER]
			l_k: INTEGER_32
			l_tt: CE_POINTER [PD_TIMER]
		do
			create l_tm.make
			create l_tt.make

			l_k := 0
			l_tm := create {CE_POINTER [PD_TIMER]}.make_cast (timers.item.next)
			from
			until
				l_tm.is_equal (timers)
			loop
				if l_tm.item.id = a_id then
					l_tt := l_tm
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_tm));
					(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_tm.item.next)).item.prev := create {CE_POINTER [ANY]}.make_cast (l_tm.item.prev);
					(create {CE_POINTER [PD_LIST_HEAD]}.make_cast (l_tm.item.prev)).item.next := create {CE_POINTER [ANY]}.make_cast (l_tm.item.next)
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (l_tm))
					l_tm := l_tm.item.prev
					error_inst.debug_mem_free (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c")), 138, create {CE_POINTER [ANY]}.make_cast (l_tt))
					l_k := l_k + 1
				end
				l_tm := l_tm.item.next
			end
			if not l_k.to_boolean then
				from
				until
					False
				loop
					links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
					links_inst.errline := 141
					error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("trying to kill nonexisting timer"))])
				end
			end
			if l_k >= 2 then
				from
				until
					False
				loop
					links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
					links_inst.errline := 142
					error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("more timers with same id"))])
				end
			end
		end

	register_bottom_half (a_fn: CE_ROUTINE; a_data: CE_POINTER [ANY]): INTEGER_32
		local
			l_bh: CE_POINTER [PD_BOTTOM_HALF]
			js: JUMP_STATE
		do
			create l_bh.make

			l_bh := create {CE_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves.item.next)
			from
			until
				not (not js.loop_done and then (not l_bh.is_equal (bottom_halves)))
			loop
				if (l_bh.item.fn).is_equal (a_fn) then
					if (l_bh.item.data).is_equal (a_data) then
						Result := 0
						js.return
					end
				end
				if not js.return_called then
					l_bh := l_bh.item.next
				end
			end
			if not js.return_called then
				l_bh := create {CE_POINTER [PD_BOTTOM_HALF]}.make_cast ((error_inst.debug_mem_alloc (create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c")), 73, (create {PD_BOTTOM_HALF}.make).structure_size.to_natural_32)))
				if l_bh.is_void then
					Result := (-1)
					js.return
				end
			end
			if not js.return_called then
				l_bh.item.fn := a_fn
				l_bh.item.data := a_data
				from
				until
					js.loop_done
				loop
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (bottom_halves))
					l_bh.item.next := (create {CE_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves)).item.next
					l_bh.item.prev := create {CE_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves);
					(create {CE_POINTER [PD_BOTTOM_HALF]}.make_cast (bottom_halves)).item.next := l_bh;
					(l_bh.item.next).item.prev := l_bh
					error_inst.do_not_optimize_here (create {CE_POINTER [ANY]}.make_cast (bottom_halves))
					js.break
				end
				Result := 0
			end
		end

	select_info (a_type: INTEGER_32): INTEGER_32
		local
			l_i: INTEGER_32
			l_j: INTEGER_32
			l_ce: CE_POINTER [PD_CACHE_ENTRY]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_ce.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_i := 0
					inspect a_type -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 5 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2
					l_j := 0
					from
					until
						not (l_j < 1_024)
					loop
						if (not (threads[l_j].read_func).is_void) then
							l_i := l_i + 1
						elseif (not (threads[l_j].write_func).is_void) then
							l_i := l_i + 1
						elseif (not (threads[l_j].error_func).is_void) then
							l_i := l_i + 1
						end
						l_j := l_j + 1
					end
					Result := l_i.to_integer_32
					js.return
				when 2 then -- switch 1, case 5
					if not js.return_called then
						l_ce := create {CE_POINTER [PD_CACHE_ENTRY]}.make_cast (timers.item.next)
						from
						until
							l_ce.is_equal (timers)
						loop
							l_i := l_i + 1
							l_ce := l_ce.item.next
						end
						Result := l_i.to_integer_32
						js.return
					end
				when 3 then -- switch 1, default:
					if not js.return_called then
						from
						until
							False
						loop
							links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
							links_inst.errline := 55
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("cache_info: bad request"))])
						end
					end
				when 4 then -- switch 1, done:
					if not js.return_called then
						Result := 0
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	select_loop (a_init___1: CE_ROUTINE)
		local
			l_st: CE_ARRAY [SD_STAT]
			l___d0: CE_ARRAY [INTEGER_32]
			l___d1: CE_ARRAY [INTEGER_32]
			l___d0___0: CE_ARRAY [INTEGER_32]
			l___d1___0: CE_ARRAY [INTEGER_32]
			l___d0___1: CE_ARRAY [INTEGER_32]
			l___d1___1: CE_ARRAY [INTEGER_32]
			l_n: INTEGER_32
			l_i: CE_ARRAY [INTEGER_32]
			l_tv: CE_ARRAY [SD_TIMEVAL]
			l_tm: CE_POINTER [SD_TIMEVAL]
			l_tt: INTEGER_32
			l_k: INTEGER_32
			l___result: CE_ARRAY [INTEGER_8]
			l___result___0: CE_ARRAY [INTEGER_8]
			l___result___1: CE_ARRAY [INTEGER_8]
			js: JUMP_STATE
		do
			create l_st.make (<<1>>)
			create l___d0.make (<<1>>)
			create l___d1.make (<<1>>)
			create l___d0___0.make (<<1>>)
			create l___d1___0.make (<<1>>)
			create l___d0___1.make (<<1>>)
			create l___d1___1.make (<<1>>)
			create l_i.make (<<1>>)
			create l_tv.make (<<1>>)
			create l_tm.make
			create l___result.make (<<1>>)
			create l___result___0.make (<<1>>)
			create l___result___1.make (<<1>>)

			if (stat (ce_string ("."), l_st)).to_boolean then
				if (not (getenv (ce_string ("HOME"))).is_void) then
					chdir ((getenv (ce_string ("HOME")))).do_nothing
				end
			end
			memset (create {CE_POINTER [ANY]}.make_cast (sa_zero), 0, (create {SD_SIGACTION}.make).structure_size.to_natural_32).do_nothing
			memset (create {CE_POINTER [ANY]}.make_cast (signal_mask), 0, ({PLATFORM}.Integer_32_bytes.to_natural_32 * (32).to_natural_32)).do_nothing
			memset (create {CE_POINTER [ANY]}.make_cast (signal_handlers), 0, ((create {PD_SIGNAL_HANDLER}.make).structure_size.to_natural_32 * (32).to_natural_32)).do_nothing
			from
			until
				js.loop_done
			loop
				asm18 (l___d0, l___d1, w_read)
				js.break
			end
			from
			until
				js.loop_done
			loop
				asm19 (l___d0___0, l___d1___0, w_write)
				js.break
			end
			from
			until
				js.loop_done
			loop
				asm20 (l___d0___1, l___d1___1, w_error)
				js.break
			end
			w_max := 0
			last_time := get_time
			signal (13, create {CE_ROUTINE}.make_shared (default_pointer + 1)).do_nothing
			if (os_dep_inst.c2_pipe (signal_pipe)).to_boolean then
				error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: can%'t create pipe for signal handling"))])
				kbd_inst.fatal_tty_exit
				exit (4)
			end
			fcntl ([signal_pipe[0], 4, 2_048]).do_nothing
			fcntl ([signal_pipe[1], 4, 2_048]).do_nothing
			set_handlers (signal_pipe[0], signal_break_address, create {CE_ROUTINE}.make_shared (default_pointer), create {CE_ROUTINE}.make_shared (default_pointer), create {CE_POINTER [ANY]}.make_shared (default_pointer))
			a_init___1.call ([])
			if not ((bottom_halves.item.next).is_equal (bottom_halves)) then
				check_bottom_halves
			end
			from
			until
				not (not js.loop_done and then (not terminate.to_boolean))
			loop
				l_tm := create {CE_POINTER [SD_TIMEVAL]}.make_shared (default_pointer)
				check_signals.do_nothing
				check_timers
				terminal_inst.redraw_all_terminals
				if not ((timers.item.next).is_equal (timers)) then
					l_tt := (create {CE_POINTER [PD_TIMER]}.make_cast (timers.item.next)).item.interval + 1
					if l_tt < 0 then
						l_tt := (0).to_integer_32
					end
					l_tv.item.tv_sec := l_tt // 1_000
					l_tv.item.tv_usec := (l_tt \\ 1_000) * 1_000
					l_tm := l_tv
				end
				memcpy (create {CE_POINTER [ANY]}.make_cast (x_read), create {CE_POINTER [ANY]}.make_cast (w_read), (create {SD_FD_SET}.make).structure_size.to_natural_32).do_nothing
				memcpy (create {CE_POINTER [ANY]}.make_cast (x_write), create {CE_POINTER [ANY]}.make_cast (w_write), (create {SD_FD_SET}.make).structure_size.to_natural_32).do_nothing
				memcpy (create {CE_POINTER [ANY]}.make_cast (x_error), create {CE_POINTER [ANY]}.make_cast (w_error), (create {SD_FD_SET}.make).structure_size.to_natural_32).do_nothing
				if terminate.to_boolean then
					js.break
				end
				if not js.loop_jumped and then (not w_max.to_boolean) then
					if (timers.item.next).is_equal (timers) then
						js.break
					end
				end
				if not js.loop_jumped then
					if (check_signals).to_boolean then
						js.continue
					end
				end
				if not js.loop_jumped then
					l_n := select2 (w_max, x_read, x_write, x_error, l_tm)
					if l_n < 0 then
						if (errno_location).item /= 4 then
							error_inst.error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("ERROR: select failed: %%d")), (errno_location).item])
						end
						js.continue
					end
				end
				if not js.loop_jumped then
					check_signals.do_nothing
					check_timers
					l_i.item := -1
					from
					until
						js.loop_done
					loop
						if l_n > 0 then
							l_i.item := l_i.item + 1
							if not (l_i.item < w_max) then
								js.break
							end
						else
							js.break
						end
						if not js.loop_jumped then
							l_k := 0
							asm21 (l___result, l_i, x_read)
							if (l___result.item).to_boolean then
								if (not (threads[l_i.item].read_func).is_void) then
									(threads[l_i.item].read_func).call ([threads[l_i.item].data])
									if not ((bottom_halves.item.next).is_equal (bottom_halves)) then
										check_bottom_halves
									end
								end
								l_k := 1
							end
							asm22 (l___result___0, l_i, x_write)
							if (l___result___0.item).to_boolean then
								if (not (threads[l_i.item].write_func).is_void) then
									(threads[l_i.item].write_func).call ([threads[l_i.item].data])
									if not ((bottom_halves.item.next).is_equal (bottom_halves)) then
										check_bottom_halves
									end
								end
								l_k := 1
							end
							asm23 (l___result___1, l_i, x_error)
							if (l___result___1.item).to_boolean then
								if (not (threads[l_i.item].error_func).is_void) then
									(threads[l_i.item].error_func).call ([threads[l_i.item].data])
									if not ((bottom_halves.item.next).is_equal (bottom_halves)) then
										check_bottom_halves
									end
								end
								l_k := 1
							end
							l_n := l_n - l_k
						end
					end
				end
			end
		end

	set_handlers (a_fd: INTEGER_32; a_read_func: CE_ROUTINE; a_write_func: CE_ROUTINE; a_error_func: CE_ROUTINE; a_data: CE_POINTER [ANY])
		local
			l_fd: CE_ARRAY [INTEGER_32]
			l_i: CE_ARRAY [INTEGER_32]
			l___result: CE_ARRAY [INTEGER_8]
			l___result___0: CE_ARRAY [INTEGER_8]
			l___result___1: CE_ARRAY [INTEGER_8]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_fd.make (<<1>>)
			create l_i.make (<<1>>)
			create l___result.make (<<1>>)
			create l___result___0.make (<<1>>)
			create l___result___1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_fd.item := a_fd
					if not (l_fd.item < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (l_fd.item >= 1_024) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							False
						loop
							links_inst.errfile := create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("select.c"))
							links_inst.errline := 164
							error_inst.int_error ([create {CE_POINTER [NATURAL_8]}.make_cast (ce_string ("set_handlers: handle %%d >= FD_SETSIZE %%d")), l_fd.item, 1_024])
						end
						js.return
				when 5 then -- if done
						if js.return_called then
							l_pos := -1 + 16 -- skip if block
						end
				when 6 then -- if:
							threads[l_fd.item].read_func := a_read_func
							threads[l_fd.item].write_func := a_write_func
							threads[l_fd.item].error_func := a_error_func
							threads[l_fd.item].data := a_data
							if (not a_read_func.is_void) then
								asm6 (w_read, l_fd)
							else
								asm7 (w_read, l_fd)
								asm8 (x_read, l_fd)
							end
							if (not a_write_func.is_void) then
								asm9 (w_write, l_fd)
							else
								asm10 (w_write, l_fd)
								asm11 (x_write, l_fd)
							end
							if (not a_error_func.is_void) then
								asm12 (w_error, l_fd)
							else
								asm13 (w_error, l_fd)
								asm14 (x_error, l_fd)
							end
							if a_read_func.is_void then
								l_pos := -1 + 9 -- skip if block
							end
				when 7 then -- if:
							l_pos := -1 + 13 -- goto _L___0
				when 8 then -- if block done, skip else block:
							l_pos := -1 + 16
				when 9 then -- else:
							if a_write_func.is_void then
								l_pos := -1 + 12 -- skip if block
							end
				when 10 then -- if:
							l_pos := -1 + 13 -- goto _L___0
				when 11 then -- if block done, skip else block:
							l_pos := -1 + 16
				when 12 then -- else:
							if a_error_func.is_void then
								l_pos := -1 + 15 -- skip if block
							end
				when 13 then -- if: | _L___0:
							if l_fd.item >= w_max then
								w_max := l_fd.item + 1
							end
				when 14 then -- if block done, skip else block:
							l_pos := -1 + 16
				when 15 then -- else:
							if l_fd.item = w_max - 1 then
								l_i.item := l_fd.item - 1
								from
								until
									not (not js.loop_done and then (l_i.item >= 0))
								loop
									asm15 (l___result, l_i, w_read)
									if (l___result.item).to_boolean then
										js.break
									else
										asm16 (l___result___0, l_i, w_write)
										if (l___result___0.item).to_boolean then
											js.break
										else
											asm17 (l___result___1, l_i, w_error)
											if (l___result___1.item).to_boolean then
												js.break
											end
										end
									end
									if not js.loop_jumped then
										l_i.item := l_i.item - 1
									end
								end
								w_max := l_i.item + 1
							end
				when 16 then -- if done | if done | if done
							l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
				end

	set_sigcld
		do
			install_signal_handler (17, sigchld_address, create {CE_POINTER [ANY]}.make_shared (default_pointer), 1)
		end

	sigchld (a_p: CE_POINTER [ANY])
		local
			js: JUMP_STATE
		do
			from
			until
				js.loop_done
			loop
				if not ((waitpid (-1, create {CE_POINTER [INTEGER_32]}.make_shared (default_pointer), 1)) > 0) then
					js.break
				end
			end
		end

	sigchld_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent sigchld)
		end

	signal_break (a_data: CE_POINTER [ANY])
		local
			l_c: CE_ARRAY [INTEGER_8]
			js: JUMP_STATE
		do
			create l_c.make (<<1>>)

			from
			until
				js.loop_done
			loop
				if not (links_inst.can_read (signal_pipe[0])).to_boolean then
					js.break
				end
				if not js.loop_jumped then
					read (signal_pipe[0], create {CE_POINTER [ANY]}.make_cast (l_c), (1).to_natural_32).do_nothing
				end
			end
		end

	signal_break_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent signal_break)
		end

end
