class
	CE_ARRAY [G]

inherit
	CE_POINTER [G]
		rename
			make as make_from_pointer,
			make_shared as make_shared_from_pointer,
			plus as plus_from_pointer,
			minus as minus_from_pointer
		redefine
			make_shared_from_pointer,
			copy,
			is_equal,
			i_th
		end
	DISPOSABLE
		rename
			is_equal as is_equal_from_any
		undefine
			is_equal_from_any
		redefine
			copy
		end

create
	make,
	make_shared,
	make_shared_from_pointer

feature {CE_POINTER, CE_CLASS}

	make (a_dimensions: ARRAY [INTEGER_32])
		require
			a_dimensions /= Void
			not a_dimensions.is_empty
			across a_dimensions as c all c.item >= 0 end
		do
			make_shared (default_pointer, a_dimensions)
			c_pointer := default_pointer.memory_calloc (total_count, element_size)
			is_allocated_memory := True
		end

	make_shared (a_c_pointer: POINTER; a_dimensions: ARRAY [INTEGER_32])
		require
			a_dimensions /= Void
			not a_dimensions.is_empty
			across a_dimensions as c all c.item >= 0 end
		local
			i: INTEGER_32
		do
			set_element_info
			byte_count := a_dimensions[1] * element_size
			c_pointer := a_c_pointer

			from
				create subarray_dimensions.make_filled (0, 1, a_dimensions.count - 1)
				total_count := a_dimensions[1]
				i := 2
			until
				i > a_dimensions.count
			loop
				subarray_dimensions[i - 1] := a_dimensions[i]
				total_count := total_count * a_dimensions[i]
				i := i + 1
			end

			if not subarray_dimensions.is_empty then
				create subarray_cache.make_filled (({G}).default, 0, a_dimensions[1] - 1)
			end
		end

	make_shared_from_pointer (a_c_pointer: POINTER)
		do
			check
				not_multi_dimension: not attached {TYPE [CE_ARRAY [ANY]]} generating_type.generic_parameter_type (1)
			end
			make_shared (a_c_pointer, <<{INTEGER_32}.Max_value>>)
		end

feature {NONE}

	subarray_cache: ARRAY [G]

feature {ANY}

	subarray_dimensions: ARRAY [INTEGER_32]

	total_count: INTEGER_32
			-- the number of all elements in the array (all dimensions)

	i_th alias "[]" (a_index: INTEGER_32): G assign set_i_th
			-- Current[0]
		local
			l_old_state: BOOLEAN
			l_tmp_array: CE_ARRAY [ANY]
		do
			if not subarray_dimensions.is_empty then
				if subarray_cache[a_index] = Void then
					l_old_state := {ISE_RUNTIME}.check_assert (False)
					l_tmp_array ?= new_instance_of (generating_type.generic_parameter_type (1).type_id)
					l_tmp_array.make_shared (c_pointer + a_index * (total_count // count) * element_size, subarray_dimensions)
					l_old_state := {ISE_RUNTIME}.check_assert (l_old_state)
					subarray_cache[a_index] := ({G} #? l_tmp_array)
				end
				Result := subarray_cache[a_index]
			else
				Result := Precursor (a_index)
			end
		end

feature {ANY} -- pointer arithmetics

	plus alias "+" (i: INTEGER_32): CE_POINTER [G]
		do
			if subarray_dimensions.count > 0 then
				create Result.make_shared (c_pointer + (i * element_size * subarray_dimensions[1]))
			else
				Result := plus_from_pointer (i)
			end
		end

	minus alias "-" (i: INTEGER_32): CE_POINTER [G]
		do
			Result := plus (-i)
		end

feature {ANY}

	copy (a_other: like Current)
		local
			l_dimensions: ARRAY [INTEGER_32]
		do
			create l_dimensions.make_filled (0, 1, a_other.subarray_dimensions.count + 1)
			l_dimensions[1] := a_other.count
			l_dimensions.subcopy (a_other.subarray_dimensions, 1, a_other.subarray_dimensions.count, 2)
			make (l_dimensions)
			c_pointer.memory_copy (a_other.c_pointer, total_count * element_size)
		end

	is_equal (a_other: CE_POINTER [ANY]): BOOLEAN
		do
			Result :=
				Precursor (a_other) or else
				(
					attached {CE_ARRAY [G]} a_other as tmp and then
					total_count = tmp.total_count and then
					c_pointer.memory_compare (a_other.c_pointer, total_count * element_size)
				)
		end

feature {NONE}

	is_allocated_memory: BOOLEAN

	dispose
		do
			if is_allocated_memory then
				c_pointer.memory_free
			end
		end

invariant
	c_pointer /= default_pointer
	subarray_dimensions /= Void

end
