class
	PD_DIGESTDATA

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			create nonce.make
			create cnonce.make
			create realm.make
			create opaque.make
			create qop.make
			create algorithm.make
		end

feature {ANY} -- fields

	nonce: CE_POINTER [INTEGER_8] assign set_nonce

	cnonce: CE_POINTER [INTEGER_8] assign set_cnonce

	realm: CE_POINTER [INTEGER_8] assign set_realm

	algo: INTEGER_32 assign set_algo

	stale: BOOLEAN assign set_stale

	opaque: CE_POINTER [INTEGER_8] assign set_opaque

	qop: CE_POINTER [INTEGER_8] assign set_qop

	algorithm: CE_POINTER [INTEGER_8] assign set_algorithm

	nc: INTEGER_32 assign set_nc

feature {ANY} -- setters

	set_nonce (a_nonce: CE_POINTER [INTEGER_8])
		do
			nonce := a_nonce
			update_memory_field ("nonce")
		end

	set_cnonce (a_cnonce: CE_POINTER [INTEGER_8])
		do
			cnonce := a_cnonce
			update_memory_field ("cnonce")
		end

	set_realm (a_realm: CE_POINTER [INTEGER_8])
		do
			realm := a_realm
			update_memory_field ("realm")
		end

	set_algo (a_algo: INTEGER_32)
		do
			algo := a_algo
			update_memory_field ("algo")
		end

	set_stale (a_stale: BOOLEAN)
		do
			stale := a_stale
			update_memory_field ("stale")
		end

	set_opaque (a_opaque: CE_POINTER [INTEGER_8])
		do
			opaque := a_opaque
			update_memory_field ("opaque")
		end

	set_qop (a_qop: CE_POINTER [INTEGER_8])
		do
			qop := a_qop
			update_memory_field ("qop")
		end

	set_algorithm (a_algorithm: CE_POINTER [INTEGER_8])
		do
			algorithm := a_algorithm
			update_memory_field ("algorithm")
		end

	set_nc (a_nc: INTEGER_32)
		do
			nc := a_nc
			update_memory_field ("nc")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			nonce := a_other.nonce
			cnonce := a_other.cnonce
			realm := a_other.realm
			algo := a_other.algo
			stale := a_other.stale
			opaque := a_other.opaque
			qop := a_other.qop
			algorithm := a_other.algorithm
			nc := a_other.nc
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				nonce.is_equal (a_other.nonce) and
				cnonce.is_equal (a_other.cnonce) and
				realm.is_equal (a_other.realm) and
				algo = a_other.algo and
				stale = a_other.stale and
				opaque.is_equal (a_other.opaque) and
				qop.is_equal (a_other.qop) and
				algorithm.is_equal (a_other.algorithm) and
				nc = a_other.nc
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("nonce"),
					create {CE_STRUCTURE_FIELD}.make ("cnonce"),
					create {CE_STRUCTURE_FIELD}.make ("realm"),
					create {CE_STRUCTURE_FIELD}.make ("algo"),
					create {CE_STRUCTURE_FIELD}.make ("stale"),
					create {CE_STRUCTURE_FIELD}.make ("opaque"),
					create {CE_STRUCTURE_FIELD}.make ("qop"),
					create {CE_STRUCTURE_FIELD}.make ("algorithm"),
					create {CE_STRUCTURE_FIELD}.make ("nc")
				>>)
			init_structure_info
		end

end
