class
	PD_PROTO2

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			make_memory -- union; back with memory for correct semantics
		end

feature {ANY} -- fields

	http: CE_POINTER [PD_HTTP] assign set_http

	https: CE_POINTER [PD_HTTP] assign set_https

	rtsp: CE_POINTER [PD_RTSP] assign set_rtsp

	ftp: CE_POINTER [PD_FTP] assign set_ftp

	file: CE_POINTER [PD_FILEPROTO] assign set_file

	telnet: CE_POINTER [ANY] assign set_telnet

	generic: CE_POINTER [ANY] assign set_generic

	ssh: CE_POINTER [PD_SSHPROTO] assign set_ssh

	imap: CE_POINTER [PD_FTP] assign set_imap

	pop3: CE_POINTER [PD_FTP] assign set_pop3

	smtp: CE_POINTER [PD_FTP] assign set_smtp

feature {ANY} -- setters

	set_http (a_http: CE_POINTER [PD_HTTP])
		do
			http := a_http
			update_memory_field ("http")
		end

	set_https (a_https: CE_POINTER [PD_HTTP])
		do
			https := a_https
			update_memory_field ("https")
		end

	set_rtsp (a_rtsp: CE_POINTER [PD_RTSP])
		do
			rtsp := a_rtsp
			update_memory_field ("rtsp")
		end

	set_ftp (a_ftp: CE_POINTER [PD_FTP])
		do
			ftp := a_ftp
			update_memory_field ("ftp")
		end

	set_file (a_file: CE_POINTER [PD_FILEPROTO])
		do
			file := a_file
			update_memory_field ("file")
		end

	set_telnet (a_telnet: CE_POINTER [ANY])
		do
			telnet := a_telnet
			update_memory_field ("telnet")
		end

	set_generic (a_generic: CE_POINTER [ANY])
		do
			generic := a_generic
			update_memory_field ("generic")
		end

	set_ssh (a_ssh: CE_POINTER [PD_SSHPROTO])
		do
			ssh := a_ssh
			update_memory_field ("ssh")
		end

	set_imap (a_imap: CE_POINTER [PD_FTP])
		do
			imap := a_imap
			update_memory_field ("imap")
		end

	set_pop3 (a_pop3: CE_POINTER [PD_FTP])
		do
			pop3 := a_pop3
			update_memory_field ("pop3")
		end

	set_smtp (a_smtp: CE_POINTER [PD_FTP])
		do
			smtp := a_smtp
			update_memory_field ("smtp")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			http := a_other.http
			https := a_other.https
			rtsp := a_other.rtsp
			ftp := a_other.ftp
			file := a_other.file
			telnet := a_other.telnet
			generic := a_other.generic
			ssh := a_other.ssh
			imap := a_other.imap
			pop3 := a_other.pop3
			smtp := a_other.smtp
			create_memory_copy -- union; back with memory for correct semantics
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				http.is_equal (a_other.http) and
				https.is_equal (a_other.https) and
				rtsp.is_equal (a_other.rtsp) and
				ftp.is_equal (a_other.ftp) and
				file.is_equal (a_other.file) and
				telnet.is_equal (a_other.telnet) and
				generic.is_equal (a_other.generic) and
				ssh.is_equal (a_other.ssh) and
				imap.is_equal (a_other.imap) and
				pop3.is_equal (a_other.pop3) and
				smtp.is_equal (a_other.smtp)
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_union (<<
					create {CE_STRUCTURE_FIELD}.make ("http"),
					create {CE_STRUCTURE_FIELD}.make ("https"),
					create {CE_STRUCTURE_FIELD}.make ("rtsp"),
					create {CE_STRUCTURE_FIELD}.make ("ftp"),
					create {CE_STRUCTURE_FIELD}.make ("file"),
					create {CE_STRUCTURE_FIELD}.make ("telnet"),
					create {CE_STRUCTURE_FIELD}.make ("generic"),
					create {CE_STRUCTURE_FIELD}.make ("ssh"),
					create {CE_STRUCTURE_FIELD}.make ("imap"),
					create {CE_STRUCTURE_FIELD}.make ("pop3"),
					create {CE_STRUCTURE_FIELD}.make ("smtp")
				>>)
			init_structure_info
		end

end
