class
	PD_SSL_CONFIG_DATA

inherit
	CE_CLASS
		redefine
			make,
			copy,
			is_equal
		end

create
	make,
	make_shared

feature {NONE}

	make
		do
			create CApath.make
			create CAfile.make
			create CRLfile.make
			create issuercert.make
			create random_file.make
			create egdsocket.make
			create cipher_list.make
			create fsslctx.make
			create fsslctxp.make
		end

feature {ANY} -- fields

	version: INTEGER_32 assign set_version

	certverifyresult: INTEGER_32 assign set_certverifyresult

	verifypeer: INTEGER_32 assign set_verifypeer

	verifyhost: INTEGER_32 assign set_verifyhost

	CApath: CE_POINTER [INTEGER_8] assign set_CApath

	CAfile: CE_POINTER [INTEGER_8] assign set_CAfile

	CRLfile: CE_POINTER [INTEGER_8] assign set_CRLfile

	issuercert: CE_POINTER [INTEGER_8] assign set_issuercert

	random_file: CE_POINTER [INTEGER_8] assign set_random_file

	egdsocket: CE_POINTER [INTEGER_8] assign set_egdsocket

	cipher_list: CE_POINTER [INTEGER_8] assign set_cipher_list

	numsessions: INTEGER_32 assign set_numsessions

	fsslctx: CE_ROUTINE assign set_fsslctx

	fsslctxp: CE_POINTER [ANY] assign set_fsslctxp

	sessionid: BOOLEAN assign set_sessionid

	certinfo: BOOLEAN assign set_certinfo

feature {ANY} -- setters

	set_version (a_version: INTEGER_32)
		do
			version := a_version
			update_memory_field ("version")
		end

	set_certverifyresult (a_certverifyresult: INTEGER_32)
		do
			certverifyresult := a_certverifyresult
			update_memory_field ("certverifyresult")
		end

	set_verifypeer (a_verifypeer: INTEGER_32)
		do
			verifypeer := a_verifypeer
			update_memory_field ("verifypeer")
		end

	set_verifyhost (a_verifyhost: INTEGER_32)
		do
			verifyhost := a_verifyhost
			update_memory_field ("verifyhost")
		end

	set_CApath (a_CApath: CE_POINTER [INTEGER_8])
		do
			CApath := a_CApath
			update_memory_field ("CApath")
		end

	set_CAfile (a_CAfile: CE_POINTER [INTEGER_8])
		do
			CAfile := a_CAfile
			update_memory_field ("CAfile")
		end

	set_CRLfile (a_CRLfile: CE_POINTER [INTEGER_8])
		do
			CRLfile := a_CRLfile
			update_memory_field ("CRLfile")
		end

	set_issuercert (a_issuercert: CE_POINTER [INTEGER_8])
		do
			issuercert := a_issuercert
			update_memory_field ("issuercert")
		end

	set_random_file (a_random_file: CE_POINTER [INTEGER_8])
		do
			random_file := a_random_file
			update_memory_field ("random_file")
		end

	set_egdsocket (a_egdsocket: CE_POINTER [INTEGER_8])
		do
			egdsocket := a_egdsocket
			update_memory_field ("egdsocket")
		end

	set_cipher_list (a_cipher_list: CE_POINTER [INTEGER_8])
		do
			cipher_list := a_cipher_list
			update_memory_field ("cipher_list")
		end

	set_numsessions (a_numsessions: INTEGER_32)
		do
			numsessions := a_numsessions
			update_memory_field ("numsessions")
		end

	set_fsslctx (a_fsslctx: CE_ROUTINE)
		do
			fsslctx := a_fsslctx
			update_memory_field ("fsslctx")
		end

	set_fsslctxp (a_fsslctxp: CE_POINTER [ANY])
		do
			fsslctxp := a_fsslctxp
			update_memory_field ("fsslctxp")
		end

	set_sessionid (a_sessionid: BOOLEAN)
		do
			sessionid := a_sessionid
			update_memory_field ("sessionid")
		end

	set_certinfo (a_certinfo: BOOLEAN)
		do
			certinfo := a_certinfo
			update_memory_field ("certinfo")
		end

feature {ANY}

	copy (a_other: like Current)
		do
			version := a_other.version
			certverifyresult := a_other.certverifyresult
			verifypeer := a_other.verifypeer
			verifyhost := a_other.verifyhost
			CApath := a_other.CApath
			CAfile := a_other.CAfile
			CRLfile := a_other.CRLfile
			issuercert := a_other.issuercert
			random_file := a_other.random_file
			egdsocket := a_other.egdsocket
			cipher_list := a_other.cipher_list
			numsessions := a_other.numsessions
			fsslctx := a_other.fsslctx
			fsslctxp := a_other.fsslctxp
			sessionid := a_other.sessionid
			certinfo := a_other.certinfo
		end

	is_equal (a_other: like Current): BOOLEAN
		do
			Result :=
				version = a_other.version and
				certverifyresult = a_other.certverifyresult and
				verifypeer = a_other.verifypeer and
				verifyhost = a_other.verifyhost and
				CApath.is_equal (a_other.CApath) and
				CAfile.is_equal (a_other.CAfile) and
				CRLfile.is_equal (a_other.CRLfile) and
				issuercert.is_equal (a_other.issuercert) and
				random_file.is_equal (a_other.random_file) and
				egdsocket.is_equal (a_other.egdsocket) and
				cipher_list.is_equal (a_other.cipher_list) and
				numsessions = a_other.numsessions and
				fsslctx.is_equal (a_other.fsslctx) and
				fsslctxp.is_equal (a_other.fsslctxp) and
				sessionid = a_other.sessionid and
				certinfo = a_other.certinfo
		end

feature {NONE}

	structure_info: CE_STRUCTURE
		once ("PROCESS")
			create Result.make_struct (<<
					create {CE_STRUCTURE_FIELD}.make ("version"),
					create {CE_STRUCTURE_FIELD}.make ("certverifyresult"),
					create {CE_STRUCTURE_FIELD}.make ("verifypeer"),
					create {CE_STRUCTURE_FIELD}.make ("verifyhost"),
					create {CE_STRUCTURE_FIELD}.make ("CApath"),
					create {CE_STRUCTURE_FIELD}.make ("CAfile"),
					create {CE_STRUCTURE_FIELD}.make ("CRLfile"),
					create {CE_STRUCTURE_FIELD}.make ("issuercert"),
					create {CE_STRUCTURE_FIELD}.make ("random_file"),
					create {CE_STRUCTURE_FIELD}.make ("egdsocket"),
					create {CE_STRUCTURE_FIELD}.make ("cipher_list"),
					create {CE_STRUCTURE_FIELD}.make ("numsessions"),
					create {CE_STRUCTURE_FIELD}.make ("fsslctx"),
					create {CE_STRUCTURE_FIELD}.make ("fsslctxp"),
					create {CE_STRUCTURE_FIELD}.make ("sessionid"),
					create {CE_STRUCTURE_FIELD}.make ("certinfo")
				>>)
			init_structure_info
		end

end
