class
	P_GOPHER

inherit
	LIBCURL_DATA
		redefine
			default_create
		end
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {NONE} -- Initialization

	default_create
		do
			create Curl_handler_gopher.make (<<1>>)
		end

feature {ANY} -- attributes

	Curl_handler_gopher: CE_ARRAY [PD_CURL_HANDLER]

feature {ANY} -- routines

	gopher_do (a_conn: CE_POINTER [PD_CONNECTDATA]; a_done: CE_POINTER [BOOLEAN]): NATURAL_32
		local
			l_result: NATURAL_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_sockfd: INTEGER_32
			l_bytecount: CE_POINTER [INTEGER_64]
			l_path: CE_POINTER [INTEGER_8]
			l_sel: CE_POINTER [INTEGER_8]
			l_sel_org: CE_POINTER [INTEGER_8]
			l_amount: CE_ARRAY [INTEGER_32]
			l_k: INTEGER_32
			l_newp: CE_POINTER [INTEGER_8]
			l_j: NATURAL_32
			l_i: NATURAL_32
			l_len: CE_ARRAY [INTEGER_32]
			js: JUMP_STATE
		do
			create l_data.make
			create l_bytecount.make
			create l_path.make
			create l_sel.make
			create l_sel_org.make
			create l_amount.make (<<1>>)
			create l_newp.make
			create l_len.make (<<1>>)

			l_result := {PC_CURLCODE}.CURLE_OK
			l_data := a_conn.item.data
			l_sockfd := a_conn.item.sock[0]
			l_bytecount := create {CE_POINTER [INTEGER_64]}.make_shared (l_data.c_pointer + (create {PD_SESSIONHANDLE}.make).offset_of ("req") + (create {PD_SINGLEREQUEST}.make).offset_of ("bytecount"))
			l_path := l_data.item.state.path
			l_sel_org := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			a_done.item := (1).to_boolean
			if (strlen (l_path)) <= {NATURAL_32} 2 then
				l_sel := ce_string ("")
			else
				l_newp := l_path
				l_newp := l_newp + (2)
				l_j := strlen (l_newp)
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_j)
				loop
					if (l_newp[l_i.to_integer_32]).to_integer_32 = 63 then
						l_newp[l_i.to_integer_32] := (('%T').code).to_integer_8
					end
					l_i := l_i + 1
				end
				l_sel := escape_inst.curl_easy_unescape (create {CE_POINTER [ANY]}.make_cast (l_data), l_newp, 0, l_len)
				if l_sel.is_void then
					Result := ({PC_CURLCODE}.CURLE_OUT_OF_MEMORY)
					js.return
				end
				if not js.return_called then
					l_sel_org := l_sel
				end
			end
			if not js.return_called then
				l_k := (strlen (l_sel)).to_integer_32
				from
				until
					js.loop_done
				loop
					l_result := sendf_inst.Curl_write (a_conn, l_sockfd, create {CE_POINTER [ANY]}.make_cast (l_sel), l_k.to_natural_32, l_amount)
					if {NATURAL_32} 0 = l_result then
						l_result := sendf_inst.Curl_client_write (a_conn, (1).bit_shift_left (1), l_sel, (l_amount.item).to_natural_32)
						if l_result.to_boolean then
							url_inst.Curl_safefree (create {CE_POINTER [ANY]}.make_cast (l_sel_org))
							Result := l_result
							js.return
						end
						if not js.return_called then
							l_k := l_k - (l_amount.item)
							l_sel := l_sel + (l_amount.item)
							if l_k < 1 then
								js.break
							end
						end
					else
						sendf_inst.Curl_failf ([l_data, ce_string ("Failed sending Gopher request")])
						url_inst.Curl_safefree (create {CE_POINTER [ANY]}.make_cast (l_sel_org))
						Result := l_result
						js.return
					end
					if not js.loop_jumped then
						select_inst.Curl_socket_ready (-1, l_sockfd, 100).do_nothing
					end
				end
			end
			if not js.return_called then
				url_inst.Curl_safefree (create {CE_POINTER [ANY]}.make_cast (l_sel_org))
				l_result := sendf_inst.Curl_sendf ([l_sockfd, a_conn, ce_string ("%R%N")])
				if l_result /= {NATURAL_32} 0 then
					sendf_inst.Curl_failf ([l_data, ce_string ("Failed sending Gopher request")])
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				l_result := sendf_inst.Curl_client_write (a_conn, (1).bit_shift_left (1), ce_string ("%R%N"), (2).to_natural_32)
				if l_result.to_boolean then
					Result := l_result
					js.return
				end
			end
			if not js.return_called then
				transfer_inst.Curl_setup_transfer (a_conn, 0, (-1).to_integer_64, (0).to_boolean, l_bytecount, -1, create {CE_POINTER [INTEGER_64]}.make_shared (default_pointer))
				Result := ({PC_CURLCODE}.CURLE_OK)
			end
		end

	gopher_do_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent gopher_do)
		end

end
