class
	P_HOSTIP

inherit
	LIBCURL_DATA
		redefine
			default_create
		end
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {NONE} -- Initialization

	default_create
		do
			create hostname_cache.make (<<1>>)
		end

feature {ANY} -- attributes

	curl_jmpenv: CE_EXCEPTION assign set_curl_jmpenv

	set_curl_jmpenv (a_curl_jmpenv: CE_EXCEPTION)
		do
			curl_jmpenv := a_curl_jmpenv
		end

	host_cache_initialized: INTEGER_32 assign set_host_cache_initialized

	set_host_cache_initialized (a_host_cache_initialized: INTEGER_32)
		do
			host_cache_initialized := a_host_cache_initialized
		end

	hostname_cache: CE_ARRAY [PD_CURL_HASH]

feature {ANY} -- routines

	Curl_cache_addr (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_addr: CE_POINTER [PD_CURL_ADDRINFO]; a_hostname: CE_POINTER [INTEGER_8]; a_port: INTEGER_32): CE_POINTER [PD_CURL_DNS_ENTRY]
		local
			l_entry_id: CE_POINTER [INTEGER_8]
			l_entry_len: NATURAL_32
			l_dns: CE_POINTER [PD_CURL_DNS_ENTRY]
			l_dns2: CE_POINTER [PD_CURL_DNS_ENTRY]
			js: JUMP_STATE
		do
			create l_entry_id.make
			create l_dns.make
			create l_dns2.make

			l_entry_id := create_hostcache_id (a_hostname, a_port)
			if l_entry_id.is_void then
				Result := (create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_entry_len := strlen (l_entry_id)
				l_dns := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_ccalloc.pointer_item ([(1).to_natural_32, (create {PD_CURL_DNS_ENTRY}.make).structure_size.to_natural_32]))))
				if l_dns.is_void then
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_entry_id)])
					Result := (create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				l_dns.item.inuse := 0
				l_dns.item.addr := a_addr
				time (create {CE_POINTER [INTEGER_32]}.make_shared (l_dns.c_pointer + (create {PD_CURL_DNS_ENTRY}.make).offset_of ("timestamp"))).do_nothing
				if l_dns.item.timestamp = 0 then
					l_dns.item.timestamp := (1).to_integer_32
				end
				l_dns2 := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_cast ((hash_inst.Curl_hash_add (a_data.item.dns.hostcache, create {CE_POINTER [ANY]}.make_cast (l_entry_id), l_entry_len + {NATURAL_32} 1, create {CE_POINTER [ANY]}.make_cast (l_dns))))
				if l_dns2.is_void then
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_dns)])
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_entry_id)])
					Result := (create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				l_dns := l_dns2
				l_dns.item.inuse := (l_dns.item.inuse) + 1
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_entry_id)])
				Result := l_dns
			end
		end

	Curl_global_host_cache_dtor
		do
			if host_cache_initialized.to_boolean then
				hash_inst.Curl_hash_clean (hostname_cache)
				host_cache_initialized := 0
			end
		end

	Curl_global_host_cache_init: CE_POINTER [PD_CURL_HASH]
		local
			l_rc: INTEGER_32
			l_tmp: CE_POINTER [PD_CURL_HASH]
		do
			create l_tmp.make

			l_rc := 0
			if not host_cache_initialized.to_boolean then
				l_rc := hash_inst.Curl_hash_init (hostname_cache, 7, hash_inst.Curl_hash_str_address, hash_inst.Curl_str_key_compare_address, freednsentry_address)
				if not l_rc.to_boolean then
					host_cache_initialized := 1
				end
			end
			if l_rc.to_boolean then
				l_tmp := create {CE_POINTER [PD_CURL_HASH]}.make_shared (default_pointer)
			else
				l_tmp := hostname_cache
			end
			Result := l_tmp
		end

	Curl_hostcache_prune (a_data: CE_POINTER [PD_SESSIONHANDLE])
		local
			l_now: CE_ARRAY [INTEGER_32]
			js: JUMP_STATE
		do
			create l_now.make (<<1>>)

			if a_data.item.set.dns_cache_timeout = -1 then
				js.return
			elseif (a_data.item.dns.hostcache).is_void then
				js.return
			end
			if not js.return_called then
				if (not (a_data.item.share).is_void) then
					share_inst.Curl_share_lock (a_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS, {PC_CURL_LOCK_ACCESS}.CURL_LOCK_ACCESS_SINGLE).do_nothing
				end
				time (l_now).do_nothing
				hostcache_prune (a_data.item.dns.hostcache, a_data.item.set.dns_cache_timeout, l_now.item)
				if (not (a_data.item.share).is_void) then
					share_inst.Curl_share_unlock (a_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS).do_nothing
				end
			end
		end

	Curl_mk_dnscache: CE_POINTER [PD_CURL_HASH]
		do
			Result := (hash_inst.Curl_hash_alloc (7, hash_inst.Curl_hash_str_address, hash_inst.Curl_str_key_compare_address, freednsentry_address))
		end

	Curl_num_addresses (a_addr: CE_POINTER [PD_CURL_ADDRINFO]): INTEGER_32
		local
			l_addr: CE_POINTER [PD_CURL_ADDRINFO]
			l_i: INTEGER_32
		do
			create l_addr.make

			l_addr := a_addr
			l_i := 0
			from
			until
				l_addr.is_void
			loop
				l_addr := l_addr.item.ai_next
				l_i := l_i + 1
			end
			Result := l_i
		end

	Curl_printable_address (a_ai: CE_POINTER [PD_CURL_ADDRINFO]; a_buf: CE_POINTER [INTEGER_8]; a_bufsize: NATURAL_32): CE_POINTER [INTEGER_8]
		local
			l_sa4: CE_POINTER [SD_SOCKADDR_IN]
			l_ipaddr4: CE_POINTER [SD_IN_ADDR]
			l_sa6: CE_POINTER [SD_SOCKADDR_IN6]
			l_ipaddr6: CE_POINTER [SD_IN6_ADDR]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_sa4.make
			create l_ipaddr4.make
			create l_sa6.make
			create l_ipaddr6.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					inspect a_ai.item.ai_family -- switch 1 head
					when 2 then l_pos := -1 + 1
					when 10 then l_pos := -1 + 2
					else l_pos := -1 + 3
					end
				when 1 then -- switch 1, case 2
					l_sa4 := create {CE_POINTER [SD_SOCKADDR_IN]}.make_cast (a_ai.item.ai_addr)
					l_ipaddr4 := create {CE_POINTER [SD_IN_ADDR]}.make_shared (l_sa4.c_pointer + (create {SD_SOCKADDR_IN}.make).offset_of ("sin_addr"))
					Result := (inet_ntop (a_ai.item.ai_family, create {CE_POINTER [ANY]}.make_cast (l_ipaddr4), a_buf, a_bufsize))
					js.return
				when 2 then -- switch 1, case 10
					if not js.return_called then
						l_sa6 := create {CE_POINTER [SD_SOCKADDR_IN6]}.make_cast (a_ai.item.ai_addr)
						l_ipaddr6 := create {CE_POINTER [SD_IN6_ADDR]}.make_shared (l_sa6.c_pointer + (create {SD_SOCKADDR_IN6}.make).offset_of ("sin6_addr"))
						Result := (inet_ntop (a_ai.item.ai_family, create {CE_POINTER [ANY]}.make_cast (l_ipaddr6), a_buf, a_bufsize))
						js.return
					end
				when 3 then -- switch 1, default:
					l_pos := -1 + 4 -- switch 1; break
				when 4 then -- switch 1, done:
					if not js.return_called then
						Result := (create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer))
					end
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	Curl_resolv (a_conn: CE_POINTER [PD_CONNECTDATA]; a_hostname: CE_POINTER [INTEGER_8]; a_port: INTEGER_32; a_entry: CE_POINTER [CE_POINTER [PD_CURL_DNS_ENTRY]]): INTEGER_32
		local
			l_entry_id: CE_POINTER [INTEGER_8]
			l_dns: CE_ARRAY [CE_POINTER [PD_CURL_DNS_ENTRY]]
			l_entry_len: NATURAL_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_result: NATURAL_32
			l_rc: INTEGER_32
			l_addr: CE_POINTER [PD_CURL_ADDRINFO]
			l_respwait: CE_ARRAY [INTEGER_32]
			js: JUMP_STATE
		do
			create l_entry_id.make
			create l_dns.make (<<1>>)
			create l_data.make
			create l_addr.make
			create l_respwait.make (<<1>>)

			l_entry_id := create {CE_POINTER [INTEGER_8]}.make_shared (default_pointer)
			l_dns.item := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer)
			l_data := a_conn.item.data
			l_rc := -1
			a_entry.item := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer)
			l_entry_id := create_hostcache_id (a_hostname, a_port)
			if l_entry_id.is_void then
				Result := l_rc
				js.return
			end
			if not js.return_called then
				l_entry_len := strlen (l_entry_id)
				if (not (l_data.item.share).is_void) then
					share_inst.Curl_share_lock (l_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS, {PC_CURL_LOCK_ACCESS}.CURL_LOCK_ACCESS_SINGLE).do_nothing
				end
				l_dns.item := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_cast ((hash_inst.Curl_hash_pick (l_data.item.dns.hostcache, create {CE_POINTER [ANY]}.make_cast (l_entry_id), l_entry_len + {NATURAL_32} 1)))
				if (remove_entry_if_stale (l_data, l_dns.item)).to_boolean then
					l_dns.item := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer)
				end
				if (not (l_dns.item).is_void) then
					l_dns.item.item.inuse := (l_dns.item.item.inuse) + 1
					l_rc := 0
				end
				if (not (l_data.item.share).is_void) then
					share_inst.Curl_share_unlock (l_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS).do_nothing
				end
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_entry_id)])
				if (l_dns.item).is_void then
					if not (hostip6_inst.Curl_ipvalid (l_data)) then
						Result := (-1)
						js.return
					end
					if not js.return_called then
						l_addr := hostip6_inst.Curl_getaddrinfo (a_conn, a_hostname, a_port, l_respwait)
						if l_addr.is_void then
							if (l_respwait.item).to_boolean then
								l_result := hostsyn_inst.Curl_is_resolved (a_conn, l_dns)
								if l_result.to_boolean then
									Result := (-1)
									js.return
								end
								if not js.return_called then
									if (not (l_dns.item).is_void) then
										l_rc := 0
									else
										l_rc := 1
									end
								end
							end
						else
							if (not (l_data.item.share).is_void) then
								share_inst.Curl_share_lock (l_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS, {PC_CURL_LOCK_ACCESS}.CURL_LOCK_ACCESS_SINGLE).do_nothing
							end
							l_dns.item := Curl_cache_addr (l_data, l_addr, a_hostname, a_port)
							if (not (l_data.item.share).is_void) then
								share_inst.Curl_share_unlock (l_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS).do_nothing
							end
							if (l_dns.item).is_void then
								curl_addrinfo_inst.Curl_freeaddrinfo (l_addr)
							else
								l_rc := 0
							end
						end
					end
				end
			end
			if not js.return_called then
				a_entry.item := l_dns.item
				Result := l_rc
			end
		end

	Curl_resolv_timeout (a_conn: CE_POINTER [PD_CONNECTDATA]; a_hostname: CE_POINTER [INTEGER_8]; a_port: INTEGER_32; a_entry: CE_POINTER [CE_POINTER [PD_CURL_DNS_ENTRY]]; a_timeoutms: INTEGER_32): INTEGER_32
		local
			l_keep_sigact: CE_ARRAY [SD_SIGACTION]
			l_keep_copysig: BOOLEAN
			l_sigact: CE_ARRAY [SD_SIGACTION]
			l_timeout: INTEGER_32
			l_prev_alarm: NATURAL_32
			l_data: CE_POINTER [PD_SESSIONHANDLE]
			l_rc: INTEGER_32
			l_tmp___1: INTEGER_32
			l_elapsed_ms: NATURAL_32
			l_alarm_set: NATURAL_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_keep_sigact.make (<<1>>)
			create l_sigact.make (<<1>>)
			create l_data.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_keep_copysig := (0).to_boolean
					l_prev_alarm := (0).to_natural_32
					l_data := a_conn.item.data
					a_entry.item := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_shared (default_pointer)
					if l_data.item.set.no_signal then
						l_timeout := (0).to_integer_32
					else
						l_timeout := a_timeoutms
					end
					if not l_timeout.to_boolean then
						Result := (Curl_resolv (a_conn, a_hostname, a_port, a_entry))
						js.return
					end
					if not js.return_called and then (l_timeout < (1_000).to_integer_32) then
						Result := (-2)
						js.return
					end
					if js.return_called then
						l_pos := -1 + 5 -- skip if block
					end
				when 1 then -- if:
						sigaction (14, create {CE_POINTER [SD_SIGACTION]}.make_shared (default_pointer), l_sigact).do_nothing
						l_keep_sigact.item := (l_sigact.item).twin
						l_keep_copysig := (1).to_boolean
						l_sigact.item.sigaction_handler.sa_handler := alarmfunc_address
						l_sigact.item.sa_flags := (l_sigact.item.sa_flags).bit_and ((-268_435_457))
						sigaction (14, l_sigact, create {CE_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
						l_prev_alarm := (alarm ((l_timeout // 1_000).to_natural_32))
						curl_jmpenv := create {CE_EXCEPTION}.make ($Curl_resolv_timeout, l_pos, $l_tmp___1)
				when 2 then -- continue here on raising curl_jmpenv:
						if l_tmp___1.to_boolean then
							sendf_inst.Curl_failf ([l_data, ce_string ("name lookup timed out")])
							l_rc := -1
							l_pos := -1 + 4 -- goto clean_up
						end
				when 3 then
						l_rc := Curl_resolv (a_conn, a_hostname, a_port, a_entry)
				when 4 then -- clean_up:
						if not l_prev_alarm.to_boolean then
							alarm ({NATURAL_32} 0).do_nothing
						end
						if l_keep_copysig then
							sigaction (14, l_keep_sigact, create {CE_POINTER [SD_SIGACTION]}.make_shared (default_pointer)).do_nothing
						end
						if l_prev_alarm.to_boolean then
							l_elapsed_ms := (timeval_inst.curlx_tvdiff ((timeval_inst.curlx_tvnow).twin, (a_conn.item.created).twin)).to_natural_32
							l_alarm_set := l_prev_alarm.to_natural_32 - l_elapsed_ms // {NATURAL_32} 1_000
							if not l_alarm_set.to_boolean then
								alarm ({NATURAL_32} 1).do_nothing
								l_rc := -2
								sendf_inst.Curl_failf ([l_data, ce_string ("Previous alarm fired off!")])
							elseif l_alarm_set >= {NATURAL_32} 2_147_483_648 then
								if l_prev_alarm < {NATURAL_32} 2_147_483_648 then
									alarm ({NATURAL_32} 1).do_nothing
									l_rc := -2
									sendf_inst.Curl_failf ([l_data, ce_string ("Previous alarm fired off!")])
								else
									alarm (l_alarm_set.to_natural_32).do_nothing
								end
							else
								alarm (l_alarm_set.to_natural_32).do_nothing
							end
						end
						Result := l_rc
				when 5 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		rescue
			if attached {CE_EXCEPTION} (create {EXCEPTION_MANAGER}).last_exception as ex then
				if ex.routine = $Curl_resolv_timeout then
					l_pos := ex.position + 1
					retry
				end
				ex.re_raise
			end
		end

	Curl_resolv_unlock (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_dns: CE_POINTER [PD_CURL_DNS_ENTRY])
		do
			if (not (a_data.item.share).is_void) then
				share_inst.Curl_share_lock (a_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS, {PC_CURL_LOCK_ACCESS}.CURL_LOCK_ACCESS_SINGLE).do_nothing
			end
			a_dns.item.inuse := (a_dns.item.inuse) - 1
			if a_dns.item.inuse = 0 then
				if a_dns.item.timestamp = 0 then
					curl_addrinfo_inst.Curl_freeaddrinfo (a_dns.item.addr)
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (a_dns)])
				end
			end
			if (not (a_data.item.share).is_void) then
				share_inst.Curl_share_unlock (a_data, {PC_CURL_LOCK_DATA}.CURL_LOCK_DATA_DNS).do_nothing
			end
		end

	alarmfunc (a_sig: INTEGER_32)
		do
			curl_jmpenv.raise (1)
		end

	alarmfunc_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent alarmfunc)
		end

	create_hostcache_id (a_server: CE_POINTER [INTEGER_8]; a_port: INTEGER_32): CE_POINTER [INTEGER_8]
		do
			Result := (mprintf_inst.curl_maprintf ([ce_string ("%%s:%%d"), a_server, a_port]))
		end

	freednsentry (a_freethis: CE_POINTER [ANY])
		local
			l_p: CE_POINTER [PD_CURL_DNS_ENTRY]
		do
			create l_p.make

			l_p := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_cast (a_freethis)
			l_p.item.timestamp := (0).to_integer_32
			if l_p.item.inuse = 0 then
				curl_addrinfo_inst.Curl_freeaddrinfo (l_p.item.addr)
				easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_p)])
			end
		end

	freednsentry_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent freednsentry)
		end

	hostcache_prune (a_hostcache: CE_POINTER [PD_CURL_HASH]; a_cache_timeout: INTEGER_32; a_now: INTEGER_32)
		local
			l_user: CE_ARRAY [PD_HOSTCACHE_PRUNE_DATA]
		do
			create l_user.make (<<1>>)

			l_user.item.cache_timeout := a_cache_timeout
			l_user.item.now := a_now
			hash_inst.Curl_hash_clean_with_criterium (a_hostcache, create {CE_POINTER [ANY]}.make_cast (l_user), hostcache_timestamp_remove_address)
		end

	hostcache_timestamp_remove (a_datap: CE_POINTER [ANY]; a_hc: CE_POINTER [ANY]): INTEGER_32
		local
			l_data: CE_POINTER [PD_HOSTCACHE_PRUNE_DATA]
			l_c: CE_POINTER [PD_CURL_DNS_ENTRY]
		do
			create l_data.make
			create l_c.make

			l_data := create {CE_POINTER [PD_HOSTCACHE_PRUNE_DATA]}.make_cast (a_datap)
			l_c := create {CE_POINTER [PD_CURL_DNS_ENTRY]}.make_cast (a_hc)
			Result := (l_data.item.now - l_c.item.timestamp >= l_data.item.cache_timeout).to_integer
		end

	hostcache_timestamp_remove_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent hostcache_timestamp_remove)
		end

	remove_entry_if_stale (a_data: CE_POINTER [PD_SESSIONHANDLE]; a_dns: CE_POINTER [PD_CURL_DNS_ENTRY]): INTEGER_32
		local
			l_user: CE_ARRAY [PD_HOSTCACHE_PRUNE_DATA]
			js: JUMP_STATE
		do
			create l_user.make (<<1>>)

			if a_dns.is_void then
				Result := 0
				js.return
			elseif a_data.item.set.dns_cache_timeout = -1 then
				Result := 0
				js.return
			elseif (a_data.item.dns.hostcache).is_void then
				Result := 0
				js.return
			end
			if not js.return_called then
				time (create {CE_POINTER [INTEGER_32]}.make_shared (l_user.item.c_pointer + (create {PD_HOSTCACHE_PRUNE_DATA}.make).offset_of ("now"))).do_nothing
				l_user.item.cache_timeout := a_data.item.set.dns_cache_timeout
				if not (hostcache_timestamp_remove (create {CE_POINTER [ANY]}.make_cast (l_user), create {CE_POINTER [ANY]}.make_cast (a_dns))).to_boolean then
					Result := 0
					js.return
				end
			end
			if not js.return_called then
				hash_inst.Curl_hash_clean_with_criterium (a_data.item.dns.hostcache, create {CE_POINTER [ANY]}.make_cast (l_user), hostcache_timestamp_remove_address)
				Result := 1
			end
		end

end
