class
	P_SLIST

inherit
	LIBCURL_DATA
	S_STDIO
	S_STDLIB
	S_STRING
	S_CTYPE
	S_SYS_STAT
	S_TIME
	S_UNISTD
	S_FCNTL
	S_SYS_TIME
	S_ARPA_INET
	S_SIGNAL
	S_LIBGEN
	S_SYS_SOCKET
	S_BITS_ERRNO
	S_NETINET_IN
	S_IFADDRS
	S_PWD
	S_SYS_POLL
	S_NETDB

feature {ANY} -- routines

	Curl_slist_duplicate (a_inlist: CE_POINTER [PD_CURL_SLIST]): CE_POINTER [PD_CURL_SLIST]
		local
			l_inlist: CE_POINTER [PD_CURL_SLIST]
			l_outlist: CE_POINTER [PD_CURL_SLIST]
			l_tmp: CE_POINTER [PD_CURL_SLIST]
			js: JUMP_STATE
		do
			create l_inlist.make
			create l_outlist.make
			create l_tmp.make

			l_inlist := a_inlist
			l_outlist := create {CE_POINTER [PD_CURL_SLIST]}.make_shared (default_pointer)
			from
			until
				not (not js.loop_done and then (not l_inlist.is_void))
			loop
				l_tmp := curl_slist_append (l_outlist, l_inlist.item.data)
				if l_tmp.is_void then
					curl_slist_free_all (l_outlist)
					Result := (create {CE_POINTER [PD_CURL_SLIST]}.make_shared (default_pointer))
					js.return
				end
				if not js.return_called then
					l_outlist := l_tmp
					l_inlist := l_inlist.item.next
				end
			end
			if not js.return_called then
				Result := l_outlist
			end
		end

	curl_slist_append (a_list: CE_POINTER [PD_CURL_SLIST]; a_data: CE_POINTER [INTEGER_8]): CE_POINTER [PD_CURL_SLIST]
		local
			l_last: CE_POINTER [PD_CURL_SLIST]
			l_new_item: CE_POINTER [PD_CURL_SLIST]
			l_dupdata: CE_POINTER [INTEGER_8]
			js: JUMP_STATE
		do
			create l_last.make
			create l_new_item.make
			create l_dupdata.make

			l_new_item := create {CE_POINTER [PD_CURL_SLIST]}.make_cast ((create {CE_POINTER [ANY]}.make_cast (easy_inst.Curl_cmalloc.pointer_item ([(create {PD_CURL_SLIST}.make).structure_size.to_natural_32]))))
			if (not l_new_item.is_void) then
				l_dupdata := (create {CE_POINTER [INTEGER_8]}.make_cast (easy_inst.Curl_cstrdup.pointer_item ([a_data])))
				if (not l_dupdata.is_void) then
					l_new_item.item.next := create {CE_POINTER [PD_CURL_SLIST]}.make_shared (default_pointer)
					l_new_item.item.data := l_dupdata
				else
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_new_item)])
					Result := (create {CE_POINTER [PD_CURL_SLIST]}.make_shared (default_pointer))
					js.return
				end
			else
				Result := (create {CE_POINTER [PD_CURL_SLIST]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called and then (not a_list.is_void) then
				l_last := slist_get_last (a_list)
				l_last.item.next := l_new_item
				Result := a_list
				js.return
			end
			if not js.return_called then
				Result := l_new_item
			end
		end

	curl_slist_free_all (a_list: CE_POINTER [PD_CURL_SLIST])
		local
			l_next: CE_POINTER [PD_CURL_SLIST]
			l_item: CE_POINTER [PD_CURL_SLIST]
			js: JUMP_STATE
		do
			create l_next.make
			create l_item.make

			if a_list.is_void then
				js.return
			end
			if not js.return_called then
				l_item := a_list
				from
				until
					js.loop_done
				loop
					l_next := l_item.item.next
					if (not (l_item.item.data).is_void) then
						easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_item.item.data)])
					end
					easy_inst.Curl_cfree.call ([create {CE_POINTER [ANY]}.make_cast (l_item)])
					l_item := l_next
					if l_next.is_void then
						js.break
					end
				end
			end
		end

	slist_get_last (a_list: CE_POINTER [PD_CURL_SLIST]): CE_POINTER [PD_CURL_SLIST]
		local
			l_item: CE_POINTER [PD_CURL_SLIST]
			js: JUMP_STATE
		do
			create l_item.make

			if a_list.is_void then
				Result := (create {CE_POINTER [PD_CURL_SLIST]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_item := a_list
				from
				until
					(l_item.item.next).is_void
				loop
					l_item := l_item.item.next
				end
				Result := l_item
			end
		end

end
