class
	P_MEMORY

inherit
	LIBGMP_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_STDIO
	S_STDLIB
	S_SIGNAL
	S_STRING
	S_CTYPE
	S_LANGINFO
	S_LIBIO
	S_OBSTACK

feature {NONE} -- Initialization

	default_create
		do
			create gmp_allocate_func.make
			create gmp_free_func.make
			create gmp_reallocate_func.make
		end

feature {ANY} -- attributes

	gmp_allocate_func: CE_ROUTINE assign set_gmp_allocate_func

	set_gmp_allocate_func (a_gmp_allocate_func: CE_ROUTINE)
		do
			gmp_allocate_func := a_gmp_allocate_func
		end

	gmp_free_func: CE_ROUTINE assign set_gmp_free_func

	set_gmp_free_func (a_gmp_free_func: CE_ROUTINE)
		do
			gmp_free_func := a_gmp_free_func
		end

	gmp_reallocate_func: CE_ROUTINE assign set_gmp_reallocate_func

	set_gmp_reallocate_func (a_gmp_reallocate_func: CE_ROUTINE)
		do
			gmp_reallocate_func := a_gmp_reallocate_func
		end

feature {ANY} -- routines

	gmp_default_allocate (a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_ret: CE_POINTER [ANY]
		do
			create l_ret.make

			l_ret := malloc (a_size)
			if l_ret.is_void then
				fprintf ([(create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("GNU MP: Cannot allocate memory (size=%%lu)%N"), a_size.to_integer_32]).do_nothing
				abort
			end
			Result := l_ret
		end

	gmp_default_allocate_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent gmp_default_allocate)
		end

	gmp_default_free (a_blk_ptr: CE_POINTER [ANY]; a_blk_size: NATURAL_32)
		do
			free (a_blk_ptr)
		end

	gmp_default_free_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent gmp_default_free)
		end

	gmp_default_reallocate (a_oldptr: CE_POINTER [ANY]; a_old_size: NATURAL_32; a_new_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_ret: CE_POINTER [ANY]
		do
			create l_ret.make

			l_ret := realloc (a_oldptr, a_new_size)
			if l_ret.is_void then
				fprintf ([(create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.error.file_pointer)), ce_string ("GNU MP: Cannot reallocate memory (old_size=%%lu new_size=%%lu)%N"), a_old_size.to_integer_32, a_new_size.to_integer_32]).do_nothing
				abort
			end
			Result := l_ret
		end

	gmp_default_reallocate_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent gmp_default_reallocate)
		end

end
