class
	CE_POINTER [G]

inherit
	ANY -- to make its features accessible to clients
		rename
			is_equal as is_equal_from_any
		redefine
			is_equal_from_any
		end
	INTERNAL
		rename
			is_equal as is_equal_from_any
		export
			{NONE}all
		redefine
			is_equal_from_any
		end

create
	make,
	make_shared,
	make_shared_with_count,
	make_cast

feature {CE_POINTER, CE_CLASS}

	make
		do
			set_element_info
			byte_count := {INTEGER_32}.Max_value
		end

	make_shared (a_c_pointer: POINTER)
		do
			set_element_info
			byte_count := {INTEGER_32}.Max_value
			c_pointer := a_c_pointer
		end

	make_shared_with_count (a_c_pointer: POINTER; a_count: INTEGER_32)
		do
			set_element_info
			byte_count := a_count * element_size
			c_pointer := a_c_pointer
		end

	make_cast (a_other: CE_POINTER [ANY])
		require
			a_other /= Void
		do
			set_element_info
			if
				a_other.byte_count /= {INTEGER_32}.Max_value and then
				attached {CE_ARRAY [ANY]} a_other as tmp
			then
				byte_count := tmp.total_count * a_other.element_size
			else
				byte_count := a_other.byte_count
			end
			c_pointer := a_other.c_pointer
		end

feature {CE_UTILITIES, CE_CLASS}

	set_element_info
		local
			l_is_array: BOOLEAN
			l_element_type: TYPE [ANY]
			l_old_state: BOOLEAN
		do
			l_is_array := attached {CE_ARRAY [ANY]} Current

			-- element_type
			l_element_type := generating_type.generic_parameter_type (1)
			if l_is_array then
				from until
					not attached {TYPE [CE_ARRAY [ANY]]} l_element_type
				loop
					l_element_type := l_element_type.generic_parameter_type (1)
				end
			end
			element_eif_type := l_element_type.type_id

			-- element_id
			element_ce_type := (create {CE_UTILITIES}).ce_type (l_element_type)
			check
				valid_array_element_id: not (l_is_array and (element_ce_type = {CE_UTILITIES}.Ce_type_void or element_ce_type = {CE_UTILITIES}.Ce_type_array))
			end

			-- element_size
			inspect
				element_ce_type
			when {CE_UTILITIES}.Ce_type_boolean    then element_size := {PLATFORM}.Boolean_bytes
			when {CE_UTILITIES}.Ce_type_integer_8  then element_size := {PLATFORM}.Integer_8_bytes
			when {CE_UTILITIES}.Ce_type_integer_16 then element_size := {PLATFORM}.Integer_16_bytes
			when {CE_UTILITIES}.Ce_type_integer_32 then element_size := {PLATFORM}.Integer_32_bytes
			when {CE_UTILITIES}.Ce_type_integer_64 then element_size := {PLATFORM}.Integer_64_bytes
			when {CE_UTILITIES}.Ce_type_natural_8  then element_size := {PLATFORM}.Natural_8_bytes
			when {CE_UTILITIES}.Ce_type_natural_16 then element_size := {PLATFORM}.Natural_16_bytes
			when {CE_UTILITIES}.Ce_type_natural_32 then element_size := {PLATFORM}.Natural_32_bytes
			when {CE_UTILITIES}.Ce_type_natural_64 then element_size := {PLATFORM}.Natural_64_bytes
			when {CE_UTILITIES}.Ce_type_real_32    then element_size := {PLATFORM}.Real_32_bytes
			when {CE_UTILITIES}.Ce_type_real_64    then element_size := {PLATFORM}.Real_64_bytes
			when {CE_UTILITIES}.Ce_type_real_96    then element_size := {CE_UTILITIES}.Real_96_bytes
			when {CE_UTILITIES}.Ce_type_void       then element_size := 1
			when {CE_UTILITIES}.Ce_type_va_list    then element_size := {PLATFORM}.Pointer_bytes
			when {CE_UTILITIES}.Ce_type_pointer    then element_size := {PLATFORM}.Pointer_bytes
			when {CE_UTILITIES}.Ce_type_array then
				-- TODO: That's actually wrong. Should be sizeof(array).
				element_size := {PLATFORM}.Pointer_bytes
			when {CE_UTILITIES}.Ce_type_class then
				l_old_state := {ISE_RUNTIME}.check_assert (False)
				element_size := ({CE_CLASS} #? new_instance_of (element_eif_type)).structure_size
				l_old_state := {ISE_RUNTIME}.check_assert (l_old_state)
			end
		end

feature {CE_POINTER, CE_UTILITIES, CE_CLASS}

	element_eif_type: INTEGER_32

	element_ce_type: INTEGER_32

	element_size: INTEGER_32

	byte_count: INTEGER_32

feature {ANY}

	c_pointer: POINTER

	is_void: BOOLEAN
		do
			Result := c_pointer = default_pointer
		end

	is_equal (a_other: CE_POINTER [ANY]): BOOLEAN
		do
			Result := c_pointer = a_other.c_pointer
		end

	is_equal_from_any (a_other: like Current): BOOLEAN
		do
			Result := is_equal (a_other)
		end

feature {ANY}

	count: INTEGER_32
		do
			-- Don't store the count itself; store the byte count. Otherwise:
			--    int[1] ia; (int[])(long long[])ia
			-- would result in having a count of 0.
			Result := byte_count // element_size
		end

	item: G assign set_item
			-- *Current
		do
			Result := i_th (0)
		end

	set_item (a_item: G)
		do
			set_i_th (a_item, 0)
		end

	i_th alias "[]" (a_index: INTEGER_32): G assign set_i_th
			-- Current[0]
		require
			non_void_target: c_pointer /= default_pointer
			in_bounds: not {CE_SETTINGS}.Disable_bounds_checking implies a_index < count
		local
			p: POINTER
			l_old_state: BOOLEAN
			l_tmp_pointer: CE_POINTER [ANY]
			l_tmp_class: CE_CLASS
		do
			inspect
				element_ce_type
			when
				{CE_UTILITIES}.Ce_type_boolean,
				{CE_UTILITIES}.Ce_type_integer_8, {CE_UTILITIES}.Ce_type_integer_16, {CE_UTILITIES}.Ce_type_integer_32, {CE_UTILITIES}.Ce_type_integer_64,
				{CE_UTILITIES}.Ce_type_natural_8, {CE_UTILITIES}.Ce_type_natural_16, {CE_UTILITIES}.Ce_type_natural_32, {CE_UTILITIES}.Ce_type_natural_64,
				{CE_UTILITIES}.Ce_type_real_32, {CE_UTILITIES}.Ce_type_real_64
			then
				($Result).memory_copy (c_pointer + (a_index * element_size), element_size)
			when {CE_UTILITIES}.Ce_type_real_96 then
				({REAL_96} #? Result).c_pointer.memory_copy (c_pointer + (a_index * element_size), element_size)
			when {CE_UTILITIES}.Ce_type_pointer then
				($p).memory_copy (c_pointer + (a_index * element_size), element_size)

				l_tmp_pointer ?= new_instance_of (element_eif_type)
				l_tmp_pointer.make_shared (p)

				Result := ({G} #? l_tmp_pointer)
			when {CE_UTILITIES}.Ce_type_class then
				p := c_pointer + (a_index * element_size)

				l_old_state := {ISE_RUNTIME}.check_assert (False)
				l_tmp_class ?= new_instance_of (element_eif_type)
				l_tmp_class.make_shared (p)
				l_old_state := {ISE_RUNTIME}.check_assert (l_old_state)

				Result := ({G} #? l_tmp_class)
			when {CE_UTILITIES}.Ce_type_va_list then
				Result := ({G} #? create {CE_VA_LIST}.make_shared_indirect (c_pointer + (a_index * element_size)))
			end
		end

	set_i_th (a_item: G; a_index: INTEGER_32)
			-- Current[0] := foo
		require
			non_void_target: c_pointer /= default_pointer
			in_bounds: not {CE_SETTINGS}.Disable_bounds_checking implies a_index < count
			not_subarray_set: not (attached {CE_ARRAY [ANY]} Current as tmp and then tmp.subarray_dimensions.count > 0)
		local
			p: POINTER
		do
			inspect
				element_ce_type
			when
				{CE_UTILITIES}.Ce_type_boolean,
				{CE_UTILITIES}.Ce_type_integer_8, {CE_UTILITIES}.Ce_type_integer_16, {CE_UTILITIES}.Ce_type_integer_32, {CE_UTILITIES}.Ce_type_integer_64,
				{CE_UTILITIES}.Ce_type_natural_8, {CE_UTILITIES}.Ce_type_natural_16, {CE_UTILITIES}.Ce_type_natural_32, {CE_UTILITIES}.Ce_type_natural_64,
				{CE_UTILITIES}.Ce_type_real_32, {CE_UTILITIES}.Ce_type_real_64
			then
				(c_pointer + (a_index * element_size)).memory_copy ($a_item, element_size)
			when {CE_UTILITIES}.Ce_type_real_96 then
				(c_pointer + (a_index * element_size)).memory_copy (({REAL_96} #? a_item).c_pointer, element_size)
			when {CE_UTILITIES}.Ce_type_pointer then
				p := ({CE_POINTER [ANY]} #? a_item).c_pointer;
				(c_pointer + (a_index * element_size)).memory_copy ($p, element_size)
			when {CE_UTILITIES}.Ce_type_class then
				p := ({CE_CLASS} #? a_item).c_pointer;
				(c_pointer + (a_index * element_size)).memory_copy (p, element_size)
			when {CE_UTILITIES}.Ce_type_va_list then
				p := ({CE_VA_LIST} #? a_item).c_pointer;
				(c_pointer + (a_index * element_size)).memory_copy ($p, element_size)
			end
		end

feature {ANY} -- pointer arithmetics

	plus alias "+" (i: INTEGER_32): CE_POINTER [G]
		local
			l_count: INTEGER_32
		do
			if byte_count = {INTEGER_32}.Max_value then
				l_count := count
			else
				l_count := count - i
			end
			create Result.make_shared_with_count (c_pointer + (i * element_size), l_count)
		end

	minus alias "-" (i: INTEGER_32): CE_POINTER [G]
		do
			Result := plus (-i)
		end

	distance (other: CE_POINTER [G]): INTEGER_32
		do
			Result := (c_pointer.to_integer_32 - other.c_pointer.to_integer_32) // element_size
		end

end
