class
	P_BESSEL_JNU

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_sf_bessel_Jnu (a_nu: REAL_64; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Jnu_e (a_nu, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Jnu_e(nu, x, &result)"), ce_string ("bessel_Jnu.c"), 184, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Jnu_e (a_nu: REAL_64; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_N: INTEGER_32
			l_mu: REAL_64
			l_Jnup1_Jnu: CE_ARRAY [REAL_64]
			l_sgn_Jnu: CE_ARRAY [REAL_64]
			l_stat_CF1: INTEGER_32
			l_Y_mu: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Y_mup1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_mu: INTEGER_32
			l_Ynm1: REAL_64
			l_Yn: REAL_64
			l_Ynp1: REAL_64
			l_n2: INTEGER_32
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
			l_Jmu: REAL_64
			l_Jmup1_Jmu: REAL_64
			l_sgn_Jmu: REAL_64
			l_Jmuprime_Jmu: REAL_64
			l_P___2: CE_ARRAY [REAL_64]
			l_Q___1: CE_ARRAY [REAL_64]
			l_stat_CF2: INTEGER_32
			l_gamma___0: REAL_64
			l_Jnp1: REAL_64
			l_Jn: REAL_64
			l_Jnm1: REAL_64
			l_n___0: INTEGER_32
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_Jnup1_Jnu.make (<<1>>)
			create l_sgn_Jnu.make (<<1>>)
			create l_Y_mu.make (<<1>>)
			create l_Y_mup1.make (<<1>>)
			create l_P___2.make (<<1>>)
			create l_Q___1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x < 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_nu < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Jnu.c"), 82, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_x = 0.0 then
							if a_nu = 0.0 then
								a_result.item.val := 1.0
								a_result.item.err := 0.0
							else
								a_result.item.val := 0.0
								a_result.item.err := 0.0
							end
							Result := 0
							js.return
						elseif a_x * a_x < 10.0 * (a_nu + 1.0) then
							Result := (bessel_inst.gsl_sf_bessel_IJ_taylor_e (a_nu, a_x, -1, 100, 2.2204460492503131e-16, a_result))
							js.return
						elseif a_nu > 50.0 then
							Result := (bessel_olver_inst.gsl_sf_bessel_Jnu_asymp_Olver_e (a_nu, a_x, a_result))
							js.return
						elseif a_x > 1000.0 then
							Result := (bessel_inst.gsl_sf_bessel_Jnu_asympx_e (a_nu, a_x, a_result))
							js.return
						else
							l_N := (a_nu + 0.5).truncated_to_integer
							l_mu := a_nu - l_N.to_double
							l_stat_CF1 := (bessel_inst.gsl_sf_bessel_J_CF1 (a_nu, a_x, l_Jnup1_Jnu, l_sgn_Jnu))
							if a_x < 2.0 then
								l_stat_mu := (bessel_temme_inst.gsl_sf_bessel_Y_temme (l_mu, a_x, l_Y_mu, l_Y_mup1))
								l_Ynm1 := l_Y_mu.item.val
								l_Yn := l_Y_mup1.item.val
								l_Ynp1 := 0.0
								l_n2 := 1
								from
								until
									not (l_n2 < l_N)
								loop
									l_Ynp1 := ((2.0 * (l_mu + l_n2.to_double)) / a_x) * l_Yn - l_Ynm1
									l_Ynm1 := l_Yn
									l_Yn := l_Ynp1
									l_n2 := l_n2 + 1
								end
								a_result.item.val := (2.0 / (3.14159265358979323846 * a_x)) / (l_Jnup1_Jnu.item * l_Yn - l_Ynp1)
								a_result.item.err := (2.2204460492503131e-16 * (l_N.to_double + 2.0)) * (fabs (a_result.item.val))
								if l_stat_mu /= 0 then
									l_tmp___8 := l_stat_mu
								else
									if l_stat_CF1 /= 0 then
										l_tmp___7 := l_stat_CF1
									else
										l_tmp___7 := 0
									end
									l_tmp___8 := l_tmp___7
								end
								Result := l_tmp___8
								js.return
							else
								l_stat_CF2 := (bessel_inst.gsl_sf_bessel_JY_steed_CF2 (l_mu, a_x, l_P___2, l_Q___1))
								l_Jnp1 := (l_sgn_Jnu.item * 1.4916681462400413e-154) * l_Jnup1_Jnu.item
								l_Jn := l_sgn_Jnu.item * 1.4916681462400413e-154
								l_n___0 := l_N
								from
								until
									not (l_n___0 > 0)
								loop
									l_Jnm1 := ((2.0 * (l_mu + l_n___0.to_double)) / a_x) * l_Jn - l_Jnp1
									l_Jnp1 := l_Jn
									l_Jn := l_Jnm1
									l_n___0 := l_n___0 - 1
								end
								l_Jmup1_Jmu := l_Jnp1 / l_Jn
								if l_Jn >= 0.0 then
									l_sgn_Jmu := (1).to_double
								else
									l_sgn_Jmu := (-1).to_double
								end
								l_Jmuprime_Jmu := l_mu / a_x - l_Jmup1_Jmu
								l_gamma___0 := (l_P___2.item - l_Jmuprime_Jmu) / l_Q___1.item
								l_Jmu := l_sgn_Jmu * (sqrt ((2.0 / (3.14159265358979323846 * a_x)) / (l_Q___1.item + l_gamma___0 * (l_P___2.item - l_Jmuprime_Jmu))))
								a_result.item.val := (l_Jmu * (l_sgn_Jnu.item * 1.4916681462400413e-154)) / l_Jn
								a_result.item.err := ((2.0 * 2.2204460492503131e-16) * (l_N.to_double + 2.0)) * (fabs (a_result.item.val))
								if l_stat_CF2 /= 0 then
									l_tmp___13 := l_stat_CF2
								else
									if l_stat_CF1 /= 0 then
										l_tmp___12 := l_stat_CF1
									else
										l_tmp___12 := 0
									end
									l_tmp___13 := l_tmp___12
								end
								Result := l_tmp___13
								js.return
							end
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
