class
	P_BESSEL_K0

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create ak02_cs.make (<<1>>)
			create ak02_data.make (<<14>>)
			create ak0_cs.make (<<1>>)
			create ak0_data.make (<<17>>)
			create bk0_cs.make (<<1>>)
			create bk0_data.make (<<11>>)
		end

feature {ANY} -- attributes

	ak02_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	ak02_data: CE_ARRAY [REAL_64]

	ak0_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	ak0_data: CE_ARRAY [REAL_64]

	bk0_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	bk0_data: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gsl_sf_bessel_K0 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_K0_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_K0_e(x, &result)"), ce_string ("bessel_K0.c"), 212, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_K0_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lx: REAL_64
			l_stat_I0: INTEGER_32
			l_I0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_c___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_K0_scaled: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_K0: INTEGER_32
			l_stat_e: INTEGER_32
			l_tmp___7: INTEGER_32
			l_tmp___8: INTEGER_32
			js: JUMP_STATE
		do
			create l_I0.make (<<1>>)
			create l_c___1.make (<<1>>)
			create l_K0_scaled.make (<<1>>)

			if a_x <= 0.0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_K0.c"), 176, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif a_x <= 2.0 then
				l_lx := (log (a_x))
				cheb_eval_inst.cheb_eval_e (bk0_cs, (0.5 * a_x) * a_x - 1.0, l_c___1).do_nothing
				l_stat_I0 := bessel_i0_inst.gsl_sf_bessel_I0_e (a_x, l_I0)
				a_result.item.val := ((-l_lx + 0.69314718055994530942) * l_I0.item.val - 0.25) + l_c___1.item.val
				a_result.item.err := ((fabs (l_lx)) + 0.69314718055994530942) * l_I0.item.err + l_c___1.item.err
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := l_stat_I0
				js.return
			else
				l_stat_K0 := (gsl_sf_bessel_K0_scaled_e2 (a_x, l_K0_scaled))
				l_stat_e := (exp_inst.gsl_sf_exp_mult_err_e (-a_x, 2.2204460492503131e-16 * (fabs (a_x)), l_K0_scaled.item.val, l_K0_scaled.item.err, a_result))
				if l_stat_e /= 0 then
					l_tmp___8 := l_stat_e
				else
					if l_stat_K0 /= 0 then
						l_tmp___7 := l_stat_K0
					else
						l_tmp___7 := 0
					end
					l_tmp___8 := l_tmp___7
				end
				Result := l_tmp___8
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_sf_bessel_K0_scaled2 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_K0_scaled_e2 (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_K0_scaled_e(x, &result)"), ce_string ("bessel_K0.c"), 207, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_K0_scaled_e2 (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_lx: REAL_64
			l_ex: REAL_64
			l_stat_I0: INTEGER_32
			l_I0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_c___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_sx: REAL_64
			l_c___2: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_sx___0: REAL_64
			l_c___3: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			js: JUMP_STATE
		do
			create l_I0.make (<<1>>)
			create l_c___1.make (<<1>>)
			create l_c___2.make (<<1>>)
			create l_c___3.make (<<1>>)

			if a_x <= 0.0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_K0.c"), 135, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif a_x <= 2.0 then
				l_lx := (log (a_x))
				l_ex := (exp (a_x))
				cheb_eval_inst.cheb_eval_e (bk0_cs, (0.5 * a_x) * a_x - 1.0, l_c___1).do_nothing
				l_stat_I0 := bessel_i0_inst.gsl_sf_bessel_I0_e (a_x, l_I0)
				a_result.item.val := l_ex * (((-l_lx + 0.69314718055994530942) * l_I0.item.val - 0.25) + l_c___1.item.val)
				a_result.item.err := l_ex * ((0.69314718055994530942 + (fabs (l_lx))) * l_I0.item.err + l_c___1.item.err)
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := l_stat_I0
				js.return
			elseif a_x <= 8.0 then
				l_sx := (sqrt (a_x))
				cheb_eval_inst.cheb_eval_e (ak0_cs, (16.0 / a_x - 5.0) / 3.0, l_c___2).do_nothing
				a_result.item.val := (1.25 + l_c___2.item.val) / l_sx
				a_result.item.err := l_c___2.item.err / l_sx
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			else
				l_sx___0 := (sqrt (a_x))
				cheb_eval_inst.cheb_eval_e (ak02_cs, 16.0 / a_x - 1.0, l_c___3).do_nothing
				a_result.item.val := (1.25 + l_c___3.item.val) / l_sx___0
				a_result.item.err := (l_c___3.item.err + 2.2204460492503131e-16) / l_sx___0
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

end
