class
	P_BESSEL_Y1

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create by1_cs.make (<<1>>)
			create by1_data.make (<<14>>)
		end

feature {ANY} -- attributes

	by1_cs: CE_ARRAY [PD_CHEB_SERIES_STRUCT]

	by1_data: CE_ARRAY [REAL_64]

feature {ANY} -- routines

	gsl_sf_bessel_Y1 (a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Y1_e (a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Y1_e(x, &result)"), ce_string ("bessel_Y1.c"), 136, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Y1_e (a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_two_over_pi: REAL_64
			l_xmin: REAL_64
			l_x_small: REAL_64
			l_xmax: REAL_64
			l_lnterm: REAL_64
			l_J1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_c___1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			l_tmp___5: REAL_64
			l_lnterm___0: REAL_64
			l_status___0: INTEGER_32
			l_J1___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_c___2: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___8: REAL_64
			l_z: REAL_64
			l_ca: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_ct: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_cp: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_ca: INTEGER_32
			l_stat_ct: INTEGER_32
			l_stat_cp: INTEGER_32
			l_sqrtx: REAL_64
			l_ampl: REAL_64
			l_tmp___14: REAL_64
			l_tmp___17: INTEGER_32
			l_tmp___18: INTEGER_32
			l_tmp___19: INTEGER_32
			js: JUMP_STATE
		do
			create l_J1.make (<<1>>)
			create l_c___1.make (<<1>>)
			create l_J1___0.make (<<1>>)
			create l_c___2.make (<<1>>)
			create l_ca.make (<<1>>)
			create l_ct.make (<<1>>)
			create l_cp.make (<<1>>)

			l_two_over_pi := 2.0 / 3.14159265358979323846
			l_xmin := 1.571 * 2.2250738585072014e-308
			l_x_small := 2.0 * 1.4901161193847656e-08
			l_xmax := 1.0 / 2.2204460492503131e-16
			if a_x <= 0.0 then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_nanf (ce_string (""))).to_double
					a_result.item.err := (builtin_nanf (ce_string (""))).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Y1.c"), 83, 1)
						Result := 1
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif a_x < l_xmin then
				from
				until
					js.loop_done
				loop
					a_result.item.val := (builtin_inff).to_double
					a_result.item.err := (builtin_inff).to_double
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("overflow"), ce_string ("bessel_Y1.c"), 86, 16)
						Result := 16
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			elseif a_x < l_x_small then
				l_lnterm := (log (0.5 * a_x))
				l_status := (bessel_j1_inst.gsl_sf_bessel_J1_e (a_x, l_J1))
				cheb_eval_inst.cheb_eval_e (by1_cs, -1.0, l_c___1).do_nothing
				a_result.item.val := (l_two_over_pi * l_lnterm) * l_J1.item.val + (0.5 + l_c___1.item.val) / a_x
				l_tmp___5 := fabs (l_lnterm)
				a_result.item.err := l_tmp___5 * ((fabs (2.2204460492503131e-16 * l_J1.item.val)) + l_J1.item.err) + l_c___1.item.err / a_x
				Result := l_status
				js.return
			elseif a_x < 4.0 then
				l_lnterm___0 := (log (0.5 * a_x))
				cheb_eval_inst.cheb_eval_e (by1_cs, (0.125 * a_x) * a_x - 1.0, l_c___2).do_nothing
				l_status___0 := bessel_j1_inst.gsl_sf_bessel_J1_e (a_x, l_J1___0)
				a_result.item.val := (l_two_over_pi * l_lnterm___0) * l_J1___0.item.val + (0.5 + l_c___2.item.val) / a_x
				l_tmp___8 := fabs (l_lnterm___0)
				a_result.item.err := l_tmp___8 * ((fabs (2.2204460492503131e-16 * l_J1___0.item.val)) + l_J1___0.item.err) + l_c___2.item.err / a_x
				Result := l_status___0
				js.return
			elseif a_x < l_xmax then
				l_z := 32.0 / (a_x * a_x) - 1.0
				l_stat_ca := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bm1_cs, l_z, l_ca))
				l_stat_ct := (cheb_eval_inst.cheb_eval_e (bessel_amp_phase_inst.gsl_sf_bessel_amp_phase_bth1_cs, l_z, l_ct))
				l_stat_cp := (bessel_inst.gsl_sf_bessel_cos_pi4_e (a_x, l_ct.item.val / a_x, l_cp))
				l_sqrtx := (sqrt (a_x))
				l_ampl := (0.75 + l_ca.item.val) / l_sqrtx
				a_result.item.val := -l_ampl * l_cp.item.val
				l_tmp___14 := fabs (l_cp.item.val)
				a_result.item.err := (l_tmp___14 * l_ca.item.err) / l_sqrtx + (fabs (l_ampl)) * l_cp.item.err
				a_result.item.err := a_result.item.err + (2.2204460492503131e-16 * (fabs (a_result.item.val)))
				if l_stat_ca /= 0 then
					l_tmp___19 := l_stat_ca
				else
					if l_stat_ct /= 0 then
						l_tmp___18 := l_stat_ct
					else
						if l_stat_cp /= 0 then
							l_tmp___17 := l_stat_cp
						else
							l_tmp___17 := 0
						end
						l_tmp___18 := l_tmp___17
					end
					l_tmp___19 := l_tmp___18
				end
				Result := l_tmp___19
				js.return
			else
				from
				until
					js.loop_done
				loop
					a_result.item.val := 0.0
					a_result.item.err := 2.2250738585072014e-308
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("underflow"), ce_string ("bessel_Y1.c"), 125, 15)
						Result := 15
						js.return
						if not js.return_called then
							js.break
						end
					end
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

end
