class
	P_BESSEL_YN

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	bessel_Yn_small_x (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_k: INTEGER_32
			l_y: REAL_64
			l_ln_x_2: REAL_64
			l_ln_nm1_fact: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_k_term: REAL_64
			l_term1: REAL_64
			l_sum1: REAL_64
			l_ln_pre1: REAL_64
			l_term2: REAL_64
			l_sum2: REAL_64
			l_pre2: REAL_64
			l_KMAX: INTEGER_32
			l_psi_n: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_npk_fact: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_yk: REAL_64
			l_k_fact: REAL_64
			l_psi_kp1: REAL_64
			l_psi_npkp1: REAL_64
			l_tmp___3: REAL_64
			l_tmp___4: REAL_64
			js: JUMP_STATE
		do
			create l_ln_nm1_fact.make (<<1>>)
			create l_psi_n.make (<<1>>)
			create l_npk_fact.make (<<1>>)

			l_y := (0.25 * a_x) * a_x
			l_ln_x_2 := (log (0.5 * a_x))
			gamma_inst.gsl_sf_lnfact_e ((a_n - 1).to_natural_32, l_ln_nm1_fact).do_nothing
			l_ln_pre1 := (-a_n).to_double * l_ln_x_2 + l_ln_nm1_fact.item.val
			if l_ln_pre1 > 7.0978271289338397e+02 - 3.0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_Yn.c"), 51, 16)
					Result := 16
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_sum1 := 1.0
				l_k_term := 1.0
				l_k := 1
				from
				until
					not (l_k <= a_n - 1)
				loop
					l_k_term := l_k_term * (l_y / (l_k * (a_n - l_k)).to_double)
					l_sum1 := l_sum1 + l_k_term
					l_k := l_k + 1
				end
				l_term1 := (-(exp (l_ln_pre1)) * l_sum1) / 3.14159265358979323846
				l_pre2 := -(exp (a_n.to_double * l_ln_x_2)) / 3.14159265358979323846
				if (fabs (l_pre2)) > 0.0 then
					l_KMAX := 20
					l_yk := 1.0
					l_k_fact := 1.0
					l_psi_kp1 := -0.57721566490153286060651209008
					psi_inst.gsl_sf_psi_int_e (a_n, l_psi_n).do_nothing
					gamma_inst.gsl_sf_fact_e (a_n.to_natural_32, l_npk_fact).do_nothing
					l_psi_npkp1 := l_psi_n.item.val + 1.0 / a_n.to_double
					l_sum2 := ((l_psi_kp1 + l_psi_npkp1) - 2.0 * l_ln_x_2) / l_npk_fact.item.val
					l_k := 1
					from
					until
						not (l_k < l_KMAX)
					loop
						l_psi_kp1 := l_psi_kp1 + (1. / l_k.to_double)
						l_psi_npkp1 := l_psi_npkp1 + (1. / (a_n + l_k).to_double)
						l_k_fact := l_k_fact * l_k.to_double
						l_npk_fact.item.val := l_npk_fact.item.val * (a_n + l_k).to_double
						l_yk := l_yk * (-l_y)
						l_k_term := (l_yk * ((l_psi_kp1 + l_psi_npkp1) - 2.0 * l_ln_x_2)) / (l_k_fact * l_npk_fact.item.val)
						l_sum2 := l_sum2 + l_k_term
						l_k := l_k + 1
					end
					l_term2 := l_pre2 * l_sum2
				else
					l_term2 := 0.0
				end
				a_result.item.val := l_term1 + l_term2
				l_tmp___3 := fabs (l_ln_pre1)
				l_tmp___4 := fabs (l_term1)
				a_result.item.err := 2.2204460492503131e-16 * (l_tmp___3 * l_tmp___4 + (fabs (l_term2)))
				a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
				Result := 0
			end
		end

	gsl_sf_bessel_Yn (a_n: INTEGER_32; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Yn_e (a_n, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Yn_e(n, x, &result)"), ce_string ("bessel_Yn.c"), 215, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Yn_array (a_nmin: INTEGER_32; a_nmax: INTEGER_32; a_x: REAL_64; a_result_array: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_j: INTEGER_32
			l_r_Ynm1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_r_Yn: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_nm1: INTEGER_32
			l_stat_n: INTEGER_32
			l_Ynp1: REAL_64
			l_Yn: REAL_64
			l_Ynm1: REAL_64
			l_n: INTEGER_32
			l_stat: INTEGER_32
			l_tmp___1: INTEGER_32
			l_tmp___2: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_r_Ynm1.make (<<1>>)
			create l_r_Yn.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_nmin < 0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if not (a_nmax < a_nmin) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if not (a_x <= 0.0) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						l_j := 0
						from
						until
							not (l_j <= a_nmax - a_nmin)
						loop
							a_result_array[l_j] := 0.0
							l_j := l_j + 1
						end
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("error"), ce_string ("bessel_Yn.c"), 176, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_stat_nm1 := (gsl_sf_bessel_Yn_e (a_nmin, a_x, l_r_Ynm1))
						l_stat_n := (gsl_sf_bessel_Yn_e (a_nmin + 1, a_x, l_r_Yn))
						l_Yn := l_r_Yn.item.val
						l_Ynm1 := l_r_Ynm1.item.val
						if l_stat_nm1 /= 0 then
							l_tmp___2 := l_stat_nm1
						else
							if l_stat_n /= 0 then
								l_tmp___1 := l_stat_n
							else
								l_tmp___1 := 0
							end
							l_tmp___2 := l_tmp___1
						end
						l_stat := l_tmp___2
						if l_stat = 0 then
							l_n := a_nmin + 1
							from
							until
								not (l_n <= a_nmax + 1)
							loop
								a_result_array[((l_n - a_nmin) - 1)] := l_Ynm1
								l_Ynp1 := -l_Ynm1 + ((2.0 * l_n.to_double) / a_x) * l_Yn
								l_Ynm1 := l_Yn
								l_Yn := l_Ynp1
								l_n := l_n + 1
							end
						else
							l_n := a_nmin
							from
							until
								not (l_n <= a_nmax)
							loop
								a_result_array[(l_n - a_nmin)] := 0.0
								l_n := l_n + 1
							end
						end
						Result := l_stat
						js.return
				when 10 then -- if done | if done
						if not js.return_called then
							Result := 0
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_sf_bessel_Yn_e (a_n: INTEGER_32; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_n: INTEGER_32
			l_sign: INTEGER_32
			l_status: INTEGER_32
			l_status___0: INTEGER_32
			l_status___1: INTEGER_32
			l_status___2: INTEGER_32
			l_status___3: INTEGER_32
			l_two_over_x: REAL_64
			l_r_by: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_r_bym: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_1: INTEGER_32
			l_stat_0: INTEGER_32
			l_bym: REAL_64
			l_by: REAL_64
			l_byp: REAL_64
			l_j: INTEGER_32
			l_tmp___8: REAL_64
			l_tmp___9: REAL_64
			l_tmp___12: INTEGER_32
			l_tmp___13: INTEGER_32
			js: JUMP_STATE
		do
			create l_r_by.make (<<1>>)
			create l_r_bym.make (<<1>>)

			l_n := a_n
			l_sign := 1
			if l_n < 0 then
				l_n := -l_n
				if ((l_n).bit_and (1)).to_boolean then
					l_sign := -1
				end
			end
			if l_n = 0 then
				l_status := (bessel_y0_inst.gsl_sf_bessel_Y0_e (a_x, a_result))
				a_result.item.val := a_result.item.val * l_sign.to_double
				Result := l_status
			elseif l_n = 1 then
				l_status___0 := (bessel_y1_inst.gsl_sf_bessel_Y1_e (a_x, a_result))
				a_result.item.val := a_result.item.val * l_sign.to_double
				Result := l_status___0
			else
				if a_x <= 0.0 then
					from
					until
						js.loop_done
					loop
						a_result.item.val := (builtin_nanf (ce_string (""))).to_double
						a_result.item.err := (builtin_nanf (ce_string (""))).to_double
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Yn.c"), 125, 1)
							Result := 1
							js.return
							if not js.return_called then
								js.break
							end
						end
						if not js.return_called then
							js.break
						end
					end
				end
				if not js.return_called then
					if a_x < 5.0 then
						l_status___1 := (bessel_Yn_small_x (l_n, a_x, a_result))
						a_result.item.val := a_result.item.val * l_sign.to_double
						Result := l_status___1
					elseif 6.0554544523933429e-06 * a_x > (l_n * l_n).to_double + 1.0 then
						l_status___2 := (bessel_inst.gsl_sf_bessel_Ynu_asympx_e (l_n.to_double, a_x, a_result))
						a_result.item.val := a_result.item.val * l_sign.to_double
						Result := l_status___2
					elseif l_n > 50 then
						l_status___3 := (bessel_olver_inst.gsl_sf_bessel_Ynu_asymp_Olver_e (l_n.to_double, a_x, a_result))
						a_result.item.val := a_result.item.val * l_sign.to_double
						Result := l_status___3
					else
						l_two_over_x := 2.0 / a_x
						l_stat_1 := (bessel_y1_inst.gsl_sf_bessel_Y1_e (a_x, l_r_by))
						l_stat_0 := (bessel_y0_inst.gsl_sf_bessel_Y0_e (a_x, l_r_bym))
						l_bym := l_r_bym.item.val
						l_by := l_r_by.item.val
						l_j := 1
						from
						until
							not (l_j < l_n)
						loop
							l_byp := (l_j.to_double * l_two_over_x) * l_by - l_bym
							l_bym := l_by
							l_by := l_byp
							l_j := l_j + 1
						end
						a_result.item.val := l_sign.to_double * l_by
						l_tmp___8 := fabs (a_result.item.val)
						l_tmp___9 := fabs (l_r_by.item.err / l_r_by.item.val)
						a_result.item.err := l_tmp___8 * (l_tmp___9 + (fabs (l_r_bym.item.err / l_r_bym.item.val)))
						a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (a_result.item.val)))
						if l_stat_1 /= 0 then
							l_tmp___13 := l_stat_1
						else
							if l_stat_0 /= 0 then
								l_tmp___12 := l_stat_0
							else
								l_tmp___12 := 0
							end
							l_tmp___13 := l_tmp___12
						end
						Result := l_tmp___13
					end
				end
			end
		end

end
