class
	P_BESSEL_YNU

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_sf_bessel_Ynu (a_nu: REAL_64; a_x: REAL_64): REAL_64
		local
			l_result: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (gsl_sf_bessel_Ynu_e (a_nu, a_x, l_result))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("gsl_sf_bessel_Ynu_e(nu, x, &result)"), ce_string ("bessel_Ynu.c"), 130, l_status)
					Result := (l_result.item.val)
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item.val)
			end
		end

	gsl_sf_bessel_Ynu_e (a_nu: REAL_64; a_x: REAL_64; a_result: CE_POINTER [PD_GSL_SF_RESULT_STRUCT]): INTEGER_32
		local
			l_N: INTEGER_32
			l_mu: REAL_64
			l_Y_mu: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_Y_mup1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_mu: INTEGER_32
			l_Ynm1: REAL_64
			l_Yn: REAL_64
			l_Ynp1: REAL_64
			l_n2: INTEGER_32
			l_J_mu: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_J_mup1: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_Y_mu.make (<<1>>)
			create l_Y_mup1.make (<<1>>)
			create l_J_mu.make (<<1>>)
			create l_J_mup1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_x <= 0.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (a_nu < 0.0) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							a_result.item.val := (builtin_nanf (ce_string (""))).to_double
							a_result.item.err := (builtin_nanf (ce_string (""))).to_double
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("domain error"), ce_string ("bessel_Ynu.c"), 73, 1)
								Result := 1
								js.return
								if not js.return_called then
									js.break
								end
							end
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						if a_nu > 50.0 then
							Result := (bessel_olver_inst.gsl_sf_bessel_Ynu_asymp_Olver_e (a_nu, a_x, a_result))
							js.return
						else
							l_N := (a_nu + 0.5).truncated_to_integer
							l_mu := a_nu - l_N.to_double
							if a_x < 2.0 then
								l_stat_mu := bessel_temme_inst.gsl_sf_bessel_Y_temme (l_mu, a_x, l_Y_mu, l_Y_mup1)
							else
								l_stat_mu := bessel_inst.gsl_sf_bessel_JY_mu_restricted (l_mu, a_x, l_J_mu, l_J_mup1, l_Y_mu, l_Y_mup1)
							end
							l_Ynm1 := l_Y_mu.item.val
							l_Yn := l_Y_mup1.item.val
							l_n2 := 1
							from
							until
								not (l_n2 <= l_N)
							loop
								l_Ynp1 := ((2.0 * (l_mu + l_n2.to_double)) / a_x) * l_Yn - l_Ynm1
								l_Ynm1 := l_Yn
								l_Yn := l_Ynp1
								l_n2 := l_n2 + 1
							end
							a_result.item.val := l_Ynm1
							l_tmp___2 := fabs (l_Ynm1)
							l_tmp___3 := fabs (l_Y_mu.item.err / l_Y_mu.item.val)
							a_result.item.err := ((l_N.to_double + 1.0) * l_tmp___2) * (l_tmp___3 + (fabs (l_Y_mup1.item.err / l_Y_mup1.item.val)))
							a_result.item.err := a_result.item.err + ((2.0 * 2.2204460492503131e-16) * (fabs (l_Ynm1)))
							Result := l_stat_mu
							js.return
						end
				when 7 then -- if done
						if not js.return_called then
							Result := 0
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
