class
	P_DHT

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	dht_bessel_zeros (a_t: CE_POINTER [PD_GSL_DHT_STRUCT]): INTEGER_32
		local
			l_s: NATURAL_32
			l_z: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_stat_z: INTEGER_32
			js: JUMP_STATE
		do
			create l_z.make (<<1>>)

			l_stat_z := 0
			a_t.item.j[0] := 0.0
			l_s := {NATURAL_32} 1
			from
			until
				not (l_s < a_t.item.size + {NATURAL_32} 2)
			loop
				l_stat_z := l_stat_z + (bessel_zero_inst.gsl_sf_bessel_zero_Jnu_e (a_t.item.nu, l_s, l_z))
				a_t.item.j[l_s.to_integer_32] := l_z.item.val
				l_s := l_s + 1
			end
			if l_stat_z /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("could not compute bessel zeroes"), ce_string ("dht.c"), 91, 5)
					Result := 5
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_dht_alloc (a_size: NATURAL_32): CE_POINTER [PD_GSL_DHT_STRUCT]
		local
			l_t: CE_POINTER [PD_GSL_DHT_STRUCT]
			js: JUMP_STATE
		do
			create l_t.make

			if a_size = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size == 0"), ce_string ("dht.c"), 37, 1)
					Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_t := create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_cast ((malloc ((create {PD_GSL_DHT_STRUCT}.make).structure_size.to_natural_32)))
				if l_t.is_void then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("out of memory"), ce_string ("dht.c"), 43, 8)
						Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_t.item.size := a_size
				l_t.item.xmax := -1.0
				l_t.item.nu := -1.0
				l_t.item.j := create {CE_POINTER [REAL_64]}.make_cast ((malloc ((a_size + {NATURAL_32} 2) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_t.item.j).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_t))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("could not allocate memory for j"), ce_string ("dht.c"), 55, 8)
						Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_t.item.Jjj := create {CE_POINTER [REAL_64]}.make_cast ((malloc (((a_size * (a_size + {NATURAL_32} 1)) // {NATURAL_32} 2) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_t.item.Jjj).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_t.item.j))
					free (create {CE_POINTER [ANY]}.make_cast (l_t))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("could not allocate memory for Jjj"), ce_string ("dht.c"), 63, 8)
						Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_t.item.J2 := create {CE_POINTER [REAL_64]}.make_cast ((malloc ((a_size + {NATURAL_32} 1) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_t.item.J2).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_t.item.Jjj))
					free (create {CE_POINTER [ANY]}.make_cast (l_t.item.j))
					free (create {CE_POINTER [ANY]}.make_cast (l_t))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("could not allocate memory for J2"), ce_string ("dht.c"), 72, 8)
						Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := l_t
			end
		end

	gsl_dht_apply (a_t: CE_POINTER [PD_GSL_DHT_STRUCT]; a_f_in: CE_POINTER [REAL_64]; a_f_out: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_jN: REAL_64
			l_r___3: REAL_64
			l_m___4: NATURAL_32
			l_i: NATURAL_32
			l_sum: REAL_64
			l_Y: REAL_64
			l_m_local: NATURAL_32
			l_n_local: NATURAL_32
			js: JUMP_STATE
		do
			l_jN := a_t.item.j[(a_t.item.size + {NATURAL_32} 1).to_integer_32]
			l_r___3 := (a_t.item.xmax / l_jN)
			l_m___4 := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_m___4 < a_t.item.size))
			loop
				l_sum := 0.0
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_t.item.size)
				loop
					if l_i < l_m___4 then
						l_m_local := l_i
						l_n_local := l_m___4
					else
						l_m_local := l_m___4
						l_n_local := l_i
					end
					l_Y := a_t.item.Jjj[((l_n_local * (l_n_local + {NATURAL_32} 1)) // {NATURAL_32} 2 + l_m_local).to_integer_32] / a_t.item.J2[(l_i + {NATURAL_32} 1).to_integer_32]
					l_sum := l_sum + (l_Y * a_f_in[l_i.to_integer_32])
					l_i := l_i + 1
				end
				a_f_out[l_m___4.to_integer_32] := ((l_sum * 2.0) * l_r___3) * l_r___3
				l_m___4 := l_m___4 + 1
			end
			Result := 0
		end

	gsl_dht_free (a_t: CE_POINTER [PD_GSL_DHT_STRUCT])
		local
			js: JUMP_STATE
		do
			if a_t.is_void then
				js.return
			end
			if not js.return_called then
				free (create {CE_POINTER [ANY]}.make_cast (a_t.item.J2))
				free (create {CE_POINTER [ANY]}.make_cast (a_t.item.Jjj))
				free (create {CE_POINTER [ANY]}.make_cast (a_t.item.j))
				free (create {CE_POINTER [ANY]}.make_cast (a_t))
			end
		end

	gsl_dht_init (a_t: CE_POINTER [PD_GSL_DHT_STRUCT]; a_nu: REAL_64; a_xmax: REAL_64): INTEGER_32
		local
			l_n: NATURAL_32
			l_m___4: NATURAL_32
			l_stat_bz: INTEGER_32
			l_stat_J: INTEGER_32
			l_jN: REAL_64
			l_J: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			l_arg: REAL_64
			l_J___0: CE_ARRAY [PD_GSL_SF_RESULT_STRUCT]
			js: JUMP_STATE
		do
			create l_J.make (<<1>>)
			create l_J___0.make (<<1>>)

			if a_xmax <= 0.0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("xmax is not positive"), ce_string ("dht.c"), 120, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_nu < 0.0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("nu is negative"), ce_string ("dht.c"), 122, 1)
					Result := 1
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				l_stat_bz := 0
				l_stat_J := 0
				if a_nu /= a_t.item.nu then
					a_t.item.nu := a_nu
					l_stat_bz := dht_bessel_zeros (a_t)
				end
				l_jN := a_t.item.j[(a_t.item.size + {NATURAL_32} 1).to_integer_32]
				a_t.item.xmax := a_xmax
				a_t.item.kmax := l_jN / a_xmax
				a_t.item.J2[0] := 0.0
				l_m___4 := (1).to_natural_32
				from
				until
					not (l_m___4 < a_t.item.size + {NATURAL_32} 1)
				loop
					l_stat_J := l_stat_J + (bessel_jnu_inst.gsl_sf_bessel_Jnu_e (a_nu + 1.0, a_t.item.j[l_m___4.to_integer_32], l_J))
					a_t.item.J2[l_m___4.to_integer_32] := l_J.item.val * l_J.item.val
					l_m___4 := l_m___4 + 1
				end
				l_n := (1).to_natural_32
				from
				until
					not (not js.loop_done and then (l_n < a_t.item.size + {NATURAL_32} 1))
				loop
					l_m___4 := (1).to_natural_32
					from
					until
						not (l_m___4 <= l_n)
					loop
						l_arg := (a_t.item.j[l_n.to_integer_32] * a_t.item.j[l_m___4.to_integer_32]) / l_jN
						l_stat_J := l_stat_J + (bessel_jnu_inst.gsl_sf_bessel_Jnu_e (a_nu, l_arg, l_J___0))
						a_t.item.Jjj[(((l_n * (l_n - {NATURAL_32} 1)) // {NATURAL_32} 2 + l_m___4) - {NATURAL_32} 1).to_integer_32] := l_J___0.item.val
						l_m___4 := l_m___4 + 1
					end
					l_n := l_n + 1
				end
				if l_stat_J /= 0 then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("error computing bessel function"), ce_string ("dht.c"), 160, 5)
						Result := 5
						js.return
						if not js.return_called then
							js.break
						end
					end
				else
					Result := l_stat_bz
					js.return
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_dht_k_sample (a_t: CE_POINTER [PD_GSL_DHT_STRUCT]; a_n: INTEGER_32): REAL_64
		do
			Result := (a_t.item.j[(a_n + 1)] / a_t.item.xmax)
		end

	gsl_dht_new (a_size: NATURAL_32; a_nu: REAL_64; a_xmax: REAL_64): CE_POINTER [PD_GSL_DHT_STRUCT]
		local
			l_status: INTEGER_32
			l_dht: CE_POINTER [PD_GSL_DHT_STRUCT]
			js: JUMP_STATE
		do
			create l_dht.make

			l_dht := (gsl_dht_alloc (a_size))
			if l_dht.is_void then
				Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
				js.return
			end
			if not js.return_called then
				l_status := gsl_dht_init (l_dht, a_nu, a_xmax)
				if l_status.to_boolean then
					Result := (create {CE_POINTER [PD_GSL_DHT_STRUCT]}.make_shared (default_pointer))
					js.return
				end
			end
			if not js.return_called then
				Result := l_dht
			end
		end

	gsl_dht_x_sample (a_t: CE_POINTER [PD_GSL_DHT_STRUCT]; a_n: INTEGER_32): REAL_64
		do
			Result := ((a_t.item.j[(a_n + 1)] / a_t.item.j[(a_t.item.size + {NATURAL_32} 1).to_integer_32]) * a_t.item.xmax)
		end

end
