class
	P_DIRICHLET

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_ran_dirichlet (a_r___3: CE_POINTER [PD_GSL_RNG]; a_K: NATURAL_32; a_alpha: CE_POINTER [REAL_64]; a_theta: CE_POINTER [REAL_64])
		local
			l_i: NATURAL_32
			l_norm___0: REAL_64
			js: JUMP_STATE
		do
			l_norm___0 := 0.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				a_theta[l_i.to_integer_32] := gamma_inst.gsl_ran_gamma (a_r___3, a_alpha[l_i.to_integer_32], 1.0)
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				l_norm___0 := l_norm___0 + (a_theta[l_i.to_integer_32])
				l_i := l_i + 1
			end
			if l_norm___0 < 1.4916681462400413e-154 then
				ran_dirichlet_small (a_r___3, a_K, a_alpha, a_theta)
				js.return
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (l_i < a_K)
				loop
					a_theta[l_i.to_integer_32] := a_theta[l_i.to_integer_32] / l_norm___0
					l_i := l_i + 1
				end
			end
		end

	gsl_ran_dirichlet_lnpdf (a_K: NATURAL_32; a_alpha: CE_POINTER [REAL_64]; a_theta: CE_POINTER [REAL_64]): REAL_64
		local
			l_i: NATURAL_32
			l_log_p: REAL_64
			l_sum_alpha: REAL_64
		do
			l_log_p := 0.0
			l_sum_alpha := 0.0
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				l_log_p := l_log_p + ((a_alpha[l_i.to_integer_32] - 1.0) * (log (a_theta[l_i.to_integer_32])))
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				l_sum_alpha := l_sum_alpha + (a_alpha[l_i.to_integer_32])
				l_i := l_i + 1
			end
			l_log_p := l_log_p + (gamma_inst.gsl_sf_lngamma (l_sum_alpha))
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				l_log_p := l_log_p - (gamma_inst.gsl_sf_lngamma (a_alpha[l_i.to_integer_32]))
				l_i := l_i + 1
			end
			Result := l_log_p
		end

	gsl_ran_dirichlet_pdf (a_K: NATURAL_32; a_alpha: CE_POINTER [REAL_64]; a_theta: CE_POINTER [REAL_64]): REAL_64
		do
			Result := (exp ((gsl_ran_dirichlet_lnpdf (a_K, a_alpha, a_theta))))
		end

	ran_dirichlet_small (a_r___3: CE_POINTER [PD_GSL_RNG]; a_K: NATURAL_32; a_alpha: CE_POINTER [REAL_64]; a_theta: CE_POINTER [REAL_64])
		local
			l_i: NATURAL_32
			l_norm___0: REAL_64
			l_umax: REAL_64
			l_u: REAL_64
		do
			l_norm___0 := 0.0
			l_umax := (0).to_double
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				l_u := (log ((gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3)))) / a_alpha[l_i.to_integer_32]
				a_theta[l_i.to_integer_32] := l_u
				if l_u > l_umax then
					l_umax := l_u
				elseif l_i = {NATURAL_32} 0 then
					l_umax := l_u
				end
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				a_theta[l_i.to_integer_32] := exp (a_theta[l_i.to_integer_32] - l_umax)
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				a_theta[l_i.to_integer_32] := a_theta[l_i.to_integer_32] * (gamma_inst.gsl_ran_gamma (a_r___3, (a_alpha[l_i.to_integer_32] + 1.0), 1.0))
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				l_norm___0 := l_norm___0 + (a_theta[l_i.to_integer_32])
				l_i := l_i + 1
			end
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_K)
			loop
				a_theta[l_i.to_integer_32] := a_theta[l_i.to_integer_32] / l_norm___0
				l_i := l_i + 1
			end
		end

end
