class
	P_GENHERMV

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	genhermv_normalize_eigenvectors (a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX])
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_vi: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_scale: REAL_64
		do
			create l_vi.make (<<1>>)

			l_N := a_evec.item.size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_vi.item := (rowcol_source_inst.gsl_matrix_complex_column (a_evec, l_i)).twin
				l_scale := 1.0 / (blas_inst.gsl_blas_dznrm2 ((create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))))
				blas_inst.gsl_blas_zdscal (l_scale, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
				l_i := l_i + 1
			end
		end

	gsl_eigen_genhermv (a_A___0: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_B: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_eval: CE_POINTER [PD_GSL_VECTOR]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_N := a_A___0.item.size1
					if l_N = a_A___0.item.size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("genhermv.c"), 126, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 3 then -- else:
						if l_N = a_B.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 6 then -- else:
						if l_N = a_B.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("B matrix dimensions must match A"), ce_string ("genhermv.c"), 130, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						if a_eval.item.size /= l_N then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("genhermv.c"), 134, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
						elseif a_evec.item.size1 /= a_evec.item.size2 then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("eigenvector matrix must be square"), ce_string ("genhermv.c"), 138, 20)
								Result := 20
								js.return
								if not js.return_called then
									js.break
								end
							end
						elseif a_evec.item.size1 /= l_N then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("genhermv.c"), 142, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
						elseif a_w.item.size /= l_N then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("matrix size does not match workspace"), ce_string ("genhermv.c"), 146, 19)
								Result := 19
								js.return
								if not js.return_called then
									js.break
								end
							end
						else
							l_s := choleskyc_inst.gsl_linalg_complex_cholesky_decomp (a_B)
							if l_s /= 0 then
								Result := l_s
								js.return
							end
							if not js.return_called then
								genherm_inst.gsl_eigen_genherm_standardize (a_A___0, a_B).do_nothing
								l_s := hermv_inst.gsl_eigen_hermv (a_A___0, a_eval, a_evec, a_w.item.hermv_workspace_p)
								if l_s /= 0 then
									Result := l_s
									js.return
								end
							end
							if not js.return_called then
								blas_inst.gsl_blas_ztrsm ({PC_CBLAS_SIDE}.CblasLeft, {PC_CBLAS_UPLO}.CblasLower, {PC_CBLAS_TRANSPOSE}.CblasConjTrans, {PC_CBLAS_DIAG}.CblasNonUnit, (gsl_gsl_complex_math_inst.gsl_complex_rect (1.0, 0.0)).twin, a_B, a_evec).do_nothing
								genhermv_normalize_eigenvectors (a_evec)
								Result := 0
								js.return
							end
						end
				when 10 then -- if done | if done
						if not js.return_called then
							Result := 0
						end
				when 11 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_eigen_genhermv_alloc (a_n: NATURAL_32): CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]
		local
			l_w: CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]
			js: JUMP_STATE
		do
			create l_w.make

			if a_n = {NATURAL_32} 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("matrix dimension must be positive integer"), ce_string ("genhermv.c"), 59, 4)
					Result := (create {CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_w := create {CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]}.make_cast ((calloc ((1).to_natural_32, (create {PD_GSL_EIGEN_GENHERMV_WORKSPACE}.make).structure_size.to_natural_32)))
				if l_w.is_void then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for workspace"), ce_string ("genhermv.c"), 66, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_w.item.size := a_n
				l_w.item.hermv_workspace_p := hermv_inst.gsl_eigen_hermv_alloc (a_n)
				if (l_w.item.hermv_workspace_p).is_void then
					gsl_eigen_genhermv_free (l_w)
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for hermv workspace"), ce_string ("genhermv.c"), 75, 8)
						Result := (create {CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := l_w
			end
		end

	gsl_eigen_genhermv_free (a_w: CE_POINTER [PD_GSL_EIGEN_GENHERMV_WORKSPACE])
		local
			js: JUMP_STATE
		do
			if a_w.is_void then
				js.return
			end
			if not js.return_called then
				if (not (a_w.item.hermv_workspace_p).is_void) then
					hermv_inst.gsl_eigen_hermv_free (a_w.item.hermv_workspace_p)
				end
				free (create {CE_POINTER [ANY]}.make_cast (a_w))
			end
		end

end
