class
	P_INTERP

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_interp_alloc (a_T: CE_POINTER [PD_GSL_INTERP_TYPE]; a_size: NATURAL_32): CE_POINTER [PD_GSL_INTERP]
		local
			l_interp: CE_POINTER [PD_GSL_INTERP]
			js: JUMP_STATE
		do
			create l_interp.make

			if a_size < a_T.item.min_size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("insufficient number of points for interpolation type"), ce_string ("interp.c"), 38, 4)
					Result := (create {CE_POINTER [PD_GSL_INTERP]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_interp := create {CE_POINTER [PD_GSL_INTERP]}.make_cast ((malloc ((create {PD_GSL_INTERP}.make).structure_size.to_natural_32)))
				if l_interp.is_void then
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for interp struct"), ce_string ("interp.c"), 46, 8)
						Result := (create {CE_POINTER [PD_GSL_INTERP]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_interp.item.type := a_T
				l_interp.item.size := a_size
				if ((l_interp.item.type).item.alloc).is_void then
					l_interp.item.state := create {CE_POINTER [ANY]}.make_shared (default_pointer)
					Result := l_interp
					js.return
				end
			end
			if not js.return_called then
				l_interp.item.state := create {CE_POINTER [ANY]}.make_cast (((l_interp.item.type).item.alloc).pointer_item ([a_size]))
				if (l_interp.item.state).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_interp))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for interp state"), ce_string ("interp.c"), 63, 8)
						Result := (create {CE_POINTER [PD_GSL_INTERP]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := l_interp
			end
		end

	gsl_interp_eval (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_y: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_y.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, l_y]))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 147, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_y.item)
			end
		end

	gsl_interp_eval_deriv (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_dydx: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_dydx.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval_deriv).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, l_dydx]))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 181, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_dydx.item)
			end
		end

	gsl_interp_eval_deriv2 (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_d2: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_d2.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval_deriv2).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, l_d2]))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 215, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_d2.item)
			end
		end

	gsl_interp_eval_deriv2_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_d2: CE_POINTER [REAL_64]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_x < a_interp.item.xmin then
				a_d2.item := 0.0
				Result := 1
				js.return
			elseif a_x > a_interp.item.xmax then
				a_d2.item := 0.0
				Result := 1
				js.return
			end
			if not js.return_called then
				Result := (((a_interp.item.type).item.eval_deriv2).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, a_d2]))
			end
		end

	gsl_interp_eval_deriv_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_dydx: CE_POINTER [REAL_64]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_x < a_interp.item.xmin then
				a_dydx.item := 0.0
				Result := 1
				js.return
			elseif a_x > a_interp.item.xmax then
				a_dydx.item := 0.0
				Result := 1
				js.return
			end
			if not js.return_called then
				Result := (((a_interp.item.type).item.eval_deriv).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, a_dydx]))
			end
		end

	gsl_interp_eval_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_x: REAL_64; a_a___5: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_y: CE_POINTER [REAL_64]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_x < a_interp.item.xmin then
				a_y.item := a_ya[0]
				Result := 1
				js.return
			elseif a_x > a_interp.item.xmax then
				a_y.item := a_ya[(a_interp.item.size - {NATURAL_32} 1).to_integer_32]
				Result := 1
				js.return
			end
			if not js.return_called then
				Result := (((a_interp.item.type).item.eval).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_x, a_a___5, a_y]))
			end
		end

	gsl_interp_eval_integ (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_a___5: REAL_64; a_b___0: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]): REAL_64
		local
			l_result: CE_ARRAY [REAL_64]
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			create l_result.make (<<1>>)

			l_status := (((a_interp.item.type).item.eval_integ).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_acc, a_a___5, a_b___0, l_result]))
			if l_status /= 0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("interpolation error"), ce_string ("interp.c"), 252, l_status)
					Result := (builtin_nanf (ce_string (""))).to_double
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				Result := (l_result.item)
			end
		end

	gsl_interp_eval_integ_e (a_interp: CE_POINTER [PD_GSL_INTERP]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_a___5: REAL_64; a_b___0: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_a___5 > a_b___0 then
				a_result.item := 0.0
				Result := 1
				js.return
			elseif a_a___5 < a_interp.item.xmin then
				a_result.item := 0.0
				Result := 1
				js.return
			elseif a_b___0 > a_interp.item.xmax then
				a_result.item := 0.0
				Result := 1
				js.return
			elseif a_a___5 = a_b___0 then
				a_result.item := 0.0
				Result := 0
				js.return
			end
			if not js.return_called then
				Result := (((a_interp.item.type).item.eval_integ).integer_32_item ([a_interp.item.state, a_xa, a_ya, a_interp.item.size, a_acc, a_a___5, a_b___0, a_result]))
			end
		end

	gsl_interp_free (a_interp: CE_POINTER [PD_GSL_INTERP])
		local
			js: JUMP_STATE
		do
			if a_interp.is_void then
				js.return
			end
			if not js.return_called then
				if (not ((a_interp.item.type).item.free).is_void) then
					((a_interp.item.type).item.free).call ([a_interp.item.state])
				end
				free (create {CE_POINTER [ANY]}.make_cast (a_interp))
			end
		end

	gsl_interp_init (a_interp: CE_POINTER [PD_GSL_INTERP]; a_x_array: CE_POINTER [REAL_64]; a_y_array: CE_POINTER [REAL_64]; a_size: NATURAL_32): INTEGER_32
		local
			l_i: NATURAL_32
			l_status: INTEGER_32
			js: JUMP_STATE
		do
			if a_size /= a_interp.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("data must match size of interpolation object"), ce_string ("interp.c"), 76, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_i := (1).to_natural_32
				from
				until
					not (not js.loop_done and then (l_i < a_size))
				loop
					if not (a_x_array[(l_i - {NATURAL_32} 1).to_integer_32] < a_x_array[l_i.to_integer_32]) then
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("x values must be monotonically increasing"), ce_string ("interp.c"), 83, 4)
							Result := 4
							js.return
							if not js.return_called then
								js.break
							end
						end
					end
					if not js.return_called then
						l_i := l_i + 1
					end
				end
			end
			if not js.return_called then
				a_interp.item.xmin := a_x_array[0]
				a_interp.item.xmax := a_x_array[(a_size - {NATURAL_32} 1).to_integer_32]
				l_status := (((a_interp.item.type).item.init).integer_32_item ([a_interp.item.state, a_x_array, a_y_array, a_size]))
				Result := l_status
			end
		end

	gsl_interp_min_size (a_interp: CE_POINTER [PD_GSL_INTERP]): NATURAL_32
		do
			Result := ((a_interp.item.type).item.min_size)
		end

	gsl_interp_name (a_interp: CE_POINTER [PD_GSL_INTERP]): CE_POINTER [INTEGER_8]
		do
			Result := ((a_interp.item.type).item.name)
		end

end
