class
	P_NBINOMIAL

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_cdf_negative_binomial_P (a_k: NATURAL_32; a_p: REAL_64; a_n: REAL_64): REAL_64
		local
			l_P___2: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_p > 1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_p < 0.0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("p < 0 or p > 1"), ce_string ("nbinomial.c"), 42, 1)
							Result := (builtin_nanf (ce_string (""))).to_double
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if done
						if not js.return_called and then (a_n < (0).to_double) then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("n < 0"), ce_string ("nbinomial.c"), 47, 1)
								Result := (builtin_nanf (ce_string (""))).to_double
								js.return
								if not js.return_called then
									js.break
								end
							end
						end
						if not js.return_called then
							l_a___5 := a_n
							l_b___0 := a_k.to_real_64 + 1.0
							l_P___2 := beta_inst.gsl_cdf_beta_P (a_p, l_a___5, l_b___0)
							Result := l_P___2
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_cdf_negative_binomial_Q (a_k: NATURAL_32; a_p: REAL_64; a_n: REAL_64): REAL_64
		local
			l_Q___1: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if not (a_p > 1.0) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 5
				when 3 then -- else:
						if not (a_p < 0.0) then
							l_pos := -1 + 5 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("p < 0 or p > 1"), ce_string ("nbinomial.c"), 70, 1)
							Result := (builtin_nanf (ce_string (""))).to_double
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if done
						if not js.return_called and then (a_n < (0).to_double) then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("n < 0"), ce_string ("nbinomial.c"), 75, 1)
								Result := (builtin_nanf (ce_string (""))).to_double
								js.return
								if not js.return_called then
									js.break
								end
							end
						end
						if not js.return_called then
							l_a___5 := a_n
							l_b___0 := a_k.to_real_64 + 1.0
							l_Q___1 := beta_inst.gsl_cdf_beta_Q (a_p, l_a___5, l_b___0)
							Result := l_Q___1
						end
				when 6 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_ran_negative_binomial (a_r___3: CE_POINTER [PD_GSL_RNG]; a_p: REAL_64; a_n: REAL_64): NATURAL_32
		local
			l_X: REAL_64
			l_k: NATURAL_32
		do
			l_X := (gamma_inst.gsl_ran_gamma (a_r___3, a_n, 1.0))
			l_k := (poisson_inst.gsl_ran_poisson (a_r___3, (l_X * ((1).to_double - a_p)) / a_p))
			Result := l_k
		end

	gsl_ran_negative_binomial_pdf (a_k: NATURAL_32; a_p: REAL_64; a_n: REAL_64): REAL_64
		local
			l_P___2: REAL_64
			l_f: REAL_64
			l_a___5: REAL_64
			l_b___0: REAL_64
			l_tmp___2: REAL_64
			l_tmp___3: REAL_64
		do
			l_f := (gamma_inst.gsl_sf_lngamma (a_k.to_real_64 + a_n))
			l_a___5 := (gamma_inst.gsl_sf_lngamma (a_n))
			l_b___0 := (gamma_inst.gsl_sf_lngamma (a_k.to_real_64 + 1.0))
			l_tmp___2 := exp ((l_f - l_a___5) - l_b___0)
			l_tmp___3 := pow (a_p, a_n)
			l_P___2 := (l_tmp___2 * l_tmp___3) * (pow ((1).to_double - a_p, a_k.to_real_64))
			Result := l_P___2
		end

end
