class
	P_NONSYMMV

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_eigen_nonsymmv (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]): INTEGER_32
		local
			l_N: NATURAL_32
			l_s: INTEGER_32
			l_Z: CE_ARRAY [PD_GSL_MATRIX]
			js: JUMP_STATE
		do
			create l_Z.make (<<1>>)

			l_N := a_A___0.item.size1
			if l_N /= a_A___0.item.size2 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("matrix must be square to compute eigenvalues"), ce_string ("nonsymmv.c"), 165, 20)
					Result := 20
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_eval.item.size /= l_N then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("nonsymmv.c"), 169, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_evec.item.size1 /= a_evec.item.size2 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("eigenvector matrix must be square"), ce_string ("nonsymmv.c"), 173, 20)
					Result := 20
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_evec.item.size1 /= l_N then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("nonsymmv.c"), 177, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				l_Z.item.size1 := l_N
				l_Z.item.size2 := l_N
				l_Z.item.tda := {NATURAL_32} 2 * l_N
				l_Z.item.data := a_evec.item.data
				l_Z.item.block := create {CE_POINTER [PD_GSL_BLOCK_STRUCT]}.make_shared (default_pointer)
				l_Z.item.owner := 0
				l_s := nonsymm_inst.gsl_eigen_nonsymm_Z (a_A___0, a_eval, l_Z, a_w.item.nonsymm_workspace_p)
				if (not (a_w.item.Z).is_void) then
					copy_source_inst.gsl_matrix_memcpy (a_w.item.Z, l_Z).do_nothing
				end
				if l_s = 0 then
					nonsymmv_get_right_eigenvectors (a_A___0, l_Z, a_eval, a_evec, a_w)
					nonsymmv_normalize_eigenvectors (a_eval, a_evec)
				end
				Result := l_s
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_eigen_nonsymmv_Z (a_A___0: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_Z: CE_POINTER [PD_GSL_MATRIX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]): INTEGER_32
		local
			l_s: INTEGER_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_A___0.item.size1 = a_A___0.item.size2 then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("matrix must be square to compute eigenvalues/eigenvectors"), ce_string ("nonsymmv.c"), 247, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 3 then -- else:
						if a_eval.item.size = a_A___0.item.size1 then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("eigenvalue vector must match matrix size"), ce_string ("nonsymmv.c"), 251, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 6 then -- else:
						if a_evec.item.size1 = a_evec.item.size2 then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("eigenvector matrix must be square"), ce_string ("nonsymmv.c"), 255, 20)
							Result := 20
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 9 then -- else:
						if a_evec.item.size1 = a_A___0.item.size1 then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("eigenvector matrix has wrong size"), ce_string ("nonsymmv.c"), 259, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 12 then -- else:
						if a_Z.item.size1 = a_Z.item.size2 then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if:
						l_pos := -1 + 16 -- goto _L
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 15 then -- else:
						if a_Z.item.size1 = a_A___0.item.size1 then
							l_pos := -1 + 18 -- skip if block
						end
				when 16 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("Z matrix has wrong dimensions"), ce_string ("nonsymmv.c"), 263, 19)
							Result := 19
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 17 then -- if block done, skip else block:
						l_pos := -1 + 19
				when 18 then -- else:
						a_w.item.Z := a_Z
						l_s := gsl_eigen_nonsymmv (a_A___0, a_eval, a_evec, a_w)
						a_w.item.Z := create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						Result := l_s
						js.return
				when 19 then -- if done | if done | if done | if done | if done
						if not js.return_called then
							Result := 0
						end
				when 20 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_eigen_nonsymmv_alloc (a_n: NATURAL_32): CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]
		local
			l_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_w.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					if a_n = {NATURAL_32} 0 then
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("matrix dimension must be positive integer"), ce_string ("nonsymmv.c"), 73, 4)
							Result := (create {CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]}.make_shared (default_pointer))
							js.return
							if not js.return_called then
								js.break
							end
						end
					end
					if not js.return_called then
						l_w := create {CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]}.make_cast ((calloc ((1).to_natural_32, (create {PD_GSL_EIGEN_NONSYMMV_WORKSPACE}.make).structure_size.to_natural_32)))
						if l_w.is_void then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("failed to allocate space for workspace"), ce_string ("nonsymmv.c"), 81, 8)
								Result := (create {CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]}.make_shared (default_pointer))
								js.return
								if not js.return_called then
									js.break
								end
							end
						end
					end
					if not js.return_called then
						l_w.item.size := a_n
						l_w.item.Z := create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (default_pointer)
						l_w.item.nonsymm_workspace_p := nonsymm_inst.gsl_eigen_nonsymm_alloc (a_n)
						if (l_w.item.nonsymm_workspace_p).is_void then
							gsl_eigen_nonsymmv_free (l_w)
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("failed to allocate space for nonsymm workspace"), ce_string ("nonsymmv.c"), 91, 8)
								Result := (create {CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]}.make_shared (default_pointer))
								js.return
								if not js.return_called then
									js.break
								end
							end
						end
					end
					if js.return_called then
						l_pos := -1 + 9 -- skip if block
					end
				when 1 then -- if:
						nonsymm_inst.gsl_eigen_nonsymm_params (1, 1, l_w.item.nonsymm_workspace_p)
						l_w.item.work := init_source_inst.gsl_vector_alloc (a_n)
						l_w.item.work2 := init_source_inst.gsl_vector_alloc (a_n)
						l_w.item.work3 := init_source_inst.gsl_vector_alloc (a_n)
						if not ((l_w.item.work).is_void) then
							l_pos := -1 + 4 -- skip if block
						end
				when 2 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 3 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 4 then -- else:
						if not ((l_w.item.work2).is_void) then
							l_pos := -1 + 7 -- skip if block
						end
				when 5 then -- if:
						l_pos := -1 + 8 -- goto _L
				when 6 then -- if block done, skip else block:
						l_pos := -1 + 9
				when 7 then -- else:
						if not ((l_w.item.work3).is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 8 then -- if: | _L:
						gsl_eigen_nonsymmv_free (l_w)
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("failed to allocate space for nonsymmv additional workspace"), ce_string ("nonsymmv.c"), 106, 8)
							Result := (create {CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE]}.make_shared (default_pointer))
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 9 then -- if done | if done | if done
						if not js.return_called then
							Result := l_w
						end
				when 10 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	gsl_eigen_nonsymmv_free (a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE])
		local
			js: JUMP_STATE
		do
			if a_w.is_void then
				js.return
			end
			if not js.return_called then
				if (not (a_w.item.nonsymm_workspace_p).is_void) then
					nonsymm_inst.gsl_eigen_nonsymm_free (a_w.item.nonsymm_workspace_p)
				end
				if (not (a_w.item.work).is_void) then
					init_source_inst.gsl_vector_free (a_w.item.work)
				end
				if (not (a_w.item.work2).is_void) then
					init_source_inst.gsl_vector_free (a_w.item.work2)
				end
				if (not (a_w.item.work3).is_void) then
					init_source_inst.gsl_vector_free (a_w.item.work3)
				end
				free (create {CE_POINTER [ANY]}.make_cast (a_w))
			end
		end

	nonsymmv_get_right_eigenvectors (a_T: CE_POINTER [PD_GSL_MATRIX]; a_Z: CE_POINTER [PD_GSL_MATRIX]; a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX]; a_w: CE_POINTER [PD_GSL_EIGEN_NONSYMMV_WORKSPACE])
		local
			l_N: NATURAL_32
			l_smlnum: REAL_64
			l_bignum: REAL_64
			l_i: INTEGER_32
			l_iu: NATURAL_32
			l_ju: NATURAL_32
			l_ii: NATURAL_32
			l_lambda: CE_ARRAY [PD_GSL_COMPLEX]
			l_lambda_re: REAL_64
			l_lambda_im: REAL_64
			l_Tv: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_Zv: CE_ARRAY [PD_GSL_MATRIX_VIEW]
			l_y: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_y2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_ev: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_ev2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_dat: CE_ARRAY [REAL_64]
			l_dat_X: CE_ARRAY [REAL_64]
			l_scale: CE_ARRAY [REAL_64]
			l_xnorm: CE_ARRAY [REAL_64]
			l_ecol: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_ecol2: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_complex_pair: INTEGER_32
			l_smin: REAL_64
			l_tmp: REAL_64
			l_tmp___4: REAL_64
			l_tmp___11: REAL_64
			l_tmp___13: REAL_64
			l_k: INTEGER_32
			l_l: INTEGER_32
			l_bv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_xv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_lu: NATURAL_32
			l_x: REAL_64
			l_wv: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_x11___0: REAL_64
			l_x21: REAL_64
			l_beta: REAL_64
			l_tmp___25: REAL_64
			l_wv___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_x___0: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_a___5: REAL_64
			l_bv___0: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_xv___0: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_k___0: NATURAL_32
			l_l___0: INTEGER_32
			l_lambda2: PD_GSL_COMPLEX
			l_tmp___33: REAL_64
			l_tmp___36: REAL_64
			l_tmp___38: REAL_64
			l_lu___0: NATURAL_32
			l_bval: PD_GSL_COMPLEX
			l_x___1: PD_GSL_COMPLEX
			l_wv___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2___1: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_b1___0: PD_GSL_COMPLEX
			l_b2: PD_GSL_COMPLEX
			l_x1___0: PD_GSL_COMPLEX
			l_x2___1: PD_GSL_COMPLEX
			l_beta___0: REAL_64
			l_tmp___46: REAL_64
			l_wv___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v1___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v2___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v3: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_v4: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_x___2: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_a___6: REAL_64
			l_tmp___56: REAL_64
			l_tmp___59: REAL_64
			l_a___7: REAL_64
			js: JUMP_STATE
		do
			create l_lambda.make (<<1>>)
			create l_Tv.make (<<1>>)
			create l_Zv.make (<<1>>)
			create l_y.make (<<1>>)
			create l_y2.make (<<1>>)
			create l_ev.make (<<1>>)
			create l_ev2.make (<<1>>)
			create l_dat.make (<<4>>)
			create l_dat_X.make (<<4>>)
			create l_scale.make (<<1>>)
			create l_xnorm.make (<<1>>)
			create l_ecol.make (<<1>>)
			create l_ecol2.make (<<1>>)
			create l_bv.make (<<1>>)
			create l_xv.make (<<1>>)
			create l_wv.make (<<1>>)
			create l_v1.make (<<1>>)
			create l_v2.make (<<1>>)
			create l_wv___0.make (<<1>>)
			create l_v1___0.make (<<1>>)
			create l_v2___0.make (<<1>>)
			create l_x___0.make (<<1>>)
			create l_bv___0.make (<<1>>)
			create l_xv___0.make (<<1>>)
			create l_lambda2.make
			create l_bval.make
			create l_x___1.make
			create l_wv___1.make (<<1>>)
			create l_v1___1.make (<<1>>)
			create l_v2___1.make (<<1>>)
			create l_b1___0.make
			create l_b2.make
			create l_x1___0.make
			create l_x2___1.make
			create l_wv___2.make (<<1>>)
			create l_v1___2.make (<<1>>)
			create l_v2___2.make (<<1>>)
			create l_v3.make (<<1>>)
			create l_v4.make (<<1>>)
			create l_x___2.make (<<1>>)

			l_N := a_T.item.size1
			l_smlnum := (2.2250738585072014e-308 * l_N.to_real_64) / 2.2204460492503131e-16
			l_bignum := (1.0 - 2.2204460492503131e-16) / l_smlnum
			gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work3, (0).to_natural_32, 0.0)
			l_ju := (1).to_natural_32
			from
			until
				not (not js.loop_done and then (l_ju < l_N))
			loop
				gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work3, l_ju, 0.0)
				l_iu := (0).to_natural_32
				from
				until
					not (l_iu < l_ju)
				loop
					l_tmp := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_ju)
					gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work3, l_ju, l_tmp + (fabs ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu, l_ju)))))
					l_iu := l_iu + 1
				end
				l_ju := l_ju + 1
			end
			l_i := l_N.to_integer_32 - 1
			from
			until
				not (not js.loop_done and then (l_i >= 0))
			loop
				l_iu := l_i.to_natural_32
				l_lambda_re := gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu, l_iu)
				if l_iu /= {NATURAL_32} 0 then
					if (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu, l_iu - {NATURAL_32} 1)) /= 0.0 then
						l_tmp___4 := sqrt ((fabs ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu, l_iu - {NATURAL_32} 1)))))
						l_lambda_im := l_tmp___4 * (sqrt ((fabs ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu - {NATURAL_32} 1, l_iu))))))
					else
						l_lambda_im := 0.0
					end
				else
					l_lambda_im := 0.0
				end
				from
				until
					js.loop_done
				loop
					l_lambda.item.dat[0] := l_lambda_re
					l_lambda.item.dat[1] := l_lambda_im
					js.break
				end
				l_tmp___13 := fabs (l_lambda_re)
				if 2.2204460492503131e-16 * (l_tmp___13 + (fabs (l_lambda_im))) > l_smlnum then
					l_tmp___11 := fabs (l_lambda_re)
					l_smin := 2.2204460492503131e-16 * (l_tmp___11 + (fabs (l_lambda_im)))
				else
					l_smin := l_smlnum
				end
				if l_smin > 2.0 * 2.2250738585072014e-308 then
					l_smin := l_smin
				else
					l_smin := 2.0 * 2.2250738585072014e-308
				end
				if l_lambda_im = 0.0 then
					gsl_gsl_vector_complex_double_inst.gsl_vector_complex_set (a_eval, l_iu, (l_lambda.item).twin)
					l_k := 0
					from
					until
						not (l_k < l_i)
					loop
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_k.to_natural_32, -(gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_k.to_natural_32, l_iu)))
						l_k := l_k + 1
					end
					gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_iu, 1.0)
					l_l := l_i - 1
					from
					until
						not (l_l >= 0)
					loop
						l_lu := l_l.to_natural_32
						if l_lu = {NATURAL_32} 0 then
							l_complex_pair := 0
						else
							l_complex_pair := ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_lu, l_lu - {NATURAL_32} 1)) /= 0.0).to_integer
						end
						if not l_complex_pair.to_boolean then
							l_Tv.item := (submatrix_source_inst.gsl_matrix_submatrix (a_T, l_lu, l_lu, (1).to_natural_32, (1).to_natural_32)).twin
							l_bv.item := (view_source_inst.gsl_vector_view_array (l_dat, (1).to_natural_32)).twin
							gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), (0).to_natural_32, (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_lu)))
							l_xv.item := (view_source_inst.gsl_vector_view_array (l_dat_X, (1).to_natural_32)).twin
							schur_inst.gsl_schur_solve_equation (1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_lambda_re, 1.0, 1.0, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							l_x := gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (0).to_natural_32)
							if l_xnorm.item > 1.0 then
								if (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu)) > l_bignum / l_xnorm.item then
									l_x := l_x / (l_xnorm.item)
									l_scale.item := l_scale.item / (l_xnorm.item)
								end
							end
							if l_scale.item /= 1.0 then
								l_wv.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
							end
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_lu, l_x)
							if l_lu > {NATURAL_32} 0 then
								l_v1.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_T, l_lu, (0).to_natural_32, l_lu)).twin
								l_v2.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_lu)).twin
								blas_inst.gsl_blas_daxpy (-l_x, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
							end
						else
							l_Tv.item := (submatrix_source_inst.gsl_matrix_submatrix (a_T, l_lu - {NATURAL_32} 1, l_lu - {NATURAL_32} 1, (2).to_natural_32, (2).to_natural_32)).twin
							l_bv.item := (view_source_inst.gsl_vector_view_array (l_dat, (2).to_natural_32)).twin
							gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), (0).to_natural_32, (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_lu - {NATURAL_32} 1)))
							gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), (1).to_natural_32, (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_lu)))
							l_xv.item := (view_source_inst.gsl_vector_view_array (l_dat_X, (2).to_natural_32)).twin
							schur_inst.gsl_schur_solve_equation (1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_lambda_re, 1.0, 1.0, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_bv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							l_x11___0 := gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (0).to_natural_32)
							l_x21 := gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_xv.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (1).to_natural_32)
							if l_xnorm.item > 1.0 then
								l_tmp___25 := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu - {NATURAL_32} 1)
								if l_tmp___25 > (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu)) then
									l_beta := (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu - {NATURAL_32} 1))
								else
									l_beta := (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu))
								end
								if l_beta > l_bignum / l_xnorm.item then
									l_x11___0 := l_x11___0 / (l_xnorm.item)
									l_x21 := l_x21 / (l_xnorm.item)
									l_scale.item := l_scale.item / (l_xnorm.item)
								end
							end
							if l_scale.item /= 1.0 then
								l_wv___0.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
							end
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_lu - {NATURAL_32} 1, l_x11___0)
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_lu, l_x21)
							if l_lu > {NATURAL_32} 1 then
								l_v1___0.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_T, l_lu - {NATURAL_32} 1, (0).to_natural_32, l_lu - {NATURAL_32} 1)).twin
								l_v2___0.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_lu - {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_daxpy (-l_x11___0, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								l_v1___0.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_T, l_lu, (0).to_natural_32, l_lu - {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_daxpy (-l_x21, (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
							end
							l_l := l_l - 1
						end
						l_l := l_l - 1
					end
					l_ecol.item := (rowcol_source_inst.gsl_matrix_complex_column (a_evec, l_iu)).twin
					l_y.item := (rowcol_source_inst.gsl_matrix_column (a_Z, l_iu)).twin
					if l_iu > {NATURAL_32} 0 then
						l_Zv.item := (submatrix_source_inst.gsl_matrix_submatrix (a_Z, (0).to_natural_32, (0).to_natural_32, l_N, l_iu)).twin
						l_x___0.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_iu)).twin
						blas_inst.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Zv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x___0.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_iu)), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					end
					l_ev.item := (reim_source_inst.gsl_vector_complex_real (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_ev2.item := (reim_source_inst.gsl_vector_complex_imag (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_scale.item := 0.0
					l_ii := (0).to_natural_32
					from
					until
						not (l_ii < l_N)
					loop
						l_a___5 := (gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_ii))
						gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_ii, l_a___5)
						gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_ii, 0.0)
						if (fabs (l_a___5)) > l_scale.item then
							l_scale.item := fabs (l_a___5)
						end
						l_ii := l_ii + 1
					end
					if l_scale.item /= 0.0 then
						l_scale.item := 1.0 / l_scale.item
					end
					blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				else
					from
					until
						js.loop_done
					loop
						l_lambda2.dat[0] := l_lambda.item.dat[0]
						js.break
					end
					from
					until
						js.loop_done
					loop
						l_lambda2.dat[1] := -l_lambda.item.dat[1]
						js.break
					end
					gsl_gsl_vector_complex_double_inst.gsl_vector_complex_set (a_eval, l_iu - {NATURAL_32} 1, (l_lambda.item).twin)
					gsl_gsl_vector_complex_double_inst.gsl_vector_complex_set (a_eval, l_iu, l_lambda2.twin)
					l_tmp___33 := fabs ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu - {NATURAL_32} 1, l_iu)))
					if l_tmp___33 >= (fabs ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu, l_iu - {NATURAL_32} 1)))) then
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_iu - {NATURAL_32} 1, 1.0)
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_iu, l_lambda_im / (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu - {NATURAL_32} 1, l_iu)))
					else
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_iu - {NATURAL_32} 1, -l_lambda_im / (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_iu, l_iu - {NATURAL_32} 1)))
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_iu, 1.0)
					end
					gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_iu, 0.0)
					gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_iu - {NATURAL_32} 1, 0.0)
					l_k___0 := (0).to_natural_32
					from
					until
						not (l_k___0 < l_iu - {NATURAL_32} 1)
					loop
						l_tmp___36 := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_iu - {NATURAL_32} 1)
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_k___0, -l_tmp___36 * (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_k___0, l_iu - {NATURAL_32} 1)))
						l_tmp___38 := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work2, l_iu)
						gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_k___0, -l_tmp___38 * (gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_k___0, l_iu)))
						l_k___0 := l_k___0 + 1
					end
					l_l___0 := l_i - 2
					from
					until
						not (not js.loop_done and then (l_l___0 >= 0))
					loop
						l_lu___0 := l_l___0.to_natural_32
						if l_lu___0 = {NATURAL_32} 0 then
							l_complex_pair := 0
						else
							l_complex_pair := ((gsl_gsl_matrix_double_inst.gsl_matrix_get (a_T, l_lu___0, l_lu___0 - {NATURAL_32} 1)) /= 0.0).to_integer
						end
						if not l_complex_pair.to_boolean then
							l_Tv.item := (submatrix_source_inst.gsl_matrix_submatrix (a_T, l_lu___0, l_lu___0, (1).to_natural_32, (1).to_natural_32)).twin
							l_bv___0.item := (view_source_inst.gsl_vector_complex_view_array (l_dat, (1).to_natural_32)).twin
							l_xv___0.item := (view_source_inst.gsl_vector_complex_view_array (l_dat_X, (1).to_natural_32)).twin
							from
							until
								js.loop_done
							loop
								l_bval.dat[0] := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_lu___0)
								l_bval.dat[1] := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work2, l_lu___0)
								js.break
							end
							gsl_gsl_vector_complex_double_inst.gsl_vector_complex_set (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), (0).to_natural_32, l_bval.twin)
							schur_inst.gsl_schur_solve_equation_z (1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_lambda, 1.0, 1.0, (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							if l_xnorm.item > 1.0 then
								if (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu___0)) > l_bignum / l_xnorm.item then
									blas_inst.gsl_blas_zdscal (1.0 / l_xnorm.item, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
									l_scale.item := l_scale.item / (l_xnorm.item)
								end
							end
							if l_scale.item /= 1.0 then
								l_wv___1.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
								l_wv___1.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work2, (0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
							end
							l_x___1 := (gsl_gsl_vector_complex_double_inst.gsl_vector_complex_get ((create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), (0).to_natural_32)).twin
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_lu___0, l_x___1.dat[0])
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_lu___0, l_x___1.dat[1])
							if l_lu___0 > {NATURAL_32} 0 then
								l_v1___1.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_T, l_lu___0, (0).to_natural_32, l_lu___0)).twin
								l_v2___1.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_lu___0)).twin
								blas_inst.gsl_blas_daxpy (-l_x___1.dat[0], (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								l_v2___1.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work2, (0).to_natural_32, l_lu___0)).twin
								blas_inst.gsl_blas_daxpy (-l_x___1.dat[1], (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___1.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
							end
						else
							l_Tv.item := (submatrix_source_inst.gsl_matrix_submatrix (a_T, l_lu___0 - {NATURAL_32} 1, l_lu___0 - {NATURAL_32} 1, (2).to_natural_32, (2).to_natural_32)).twin
							l_bv___0.item := (view_source_inst.gsl_vector_complex_view_array (l_dat, (2).to_natural_32)).twin
							l_xv___0.item := (view_source_inst.gsl_vector_complex_view_array (l_dat_X, (2).to_natural_32)).twin
							from
							until
								js.loop_done
							loop
								l_b1___0.dat[0] := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_lu___0 - {NATURAL_32} 1)
								l_b1___0.dat[1] := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work2, l_lu___0 - {NATURAL_32} 1)
								js.break
							end
							from
							until
								js.loop_done
							loop
								l_b2.dat[0] := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_lu___0)
								l_b2.dat[1] := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work2, l_lu___0)
								js.break
							end
							gsl_gsl_vector_complex_double_inst.gsl_vector_complex_set (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), (0).to_natural_32, l_b1___0.twin)
							gsl_gsl_vector_complex_double_inst.gsl_vector_complex_set (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), (1).to_natural_32, l_b2.twin)
							schur_inst.gsl_schur_solve_equation_z (1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Tv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), l_lambda, 1.0, 1.0, (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_bv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")), l_scale, l_xnorm, l_smin).do_nothing
							l_x1___0 := (gsl_gsl_vector_complex_double_inst.gsl_vector_complex_get ((create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), (0).to_natural_32)).twin
							l_x2___1 := (gsl_gsl_vector_complex_double_inst.gsl_vector_complex_get ((create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector"))), (1).to_natural_32)).twin
							if l_xnorm.item > 1.0 then
								l_tmp___46 := gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu___0 - {NATURAL_32} 1)
								if l_tmp___46 > (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu___0)) then
									l_beta___0 := (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu___0 - {NATURAL_32} 1))
								else
									l_beta___0 := (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work3, l_lu___0))
								end
								if l_beta___0 > l_bignum / l_xnorm.item then
									blas_inst.gsl_blas_zdscal (1.0 / l_xnorm.item, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_xv___0.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
									l_scale.item := l_scale.item / (l_xnorm.item)
								end
							end
							if l_scale.item /= 1.0 then
								l_wv___2.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
								l_wv___2.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work2, (0).to_natural_32, l_iu + {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_dscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_wv___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
							end
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_lu___0 - {NATURAL_32} 1, l_x1___0.dat[0])
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work, l_lu___0, l_x2___1.dat[0])
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_lu___0 - {NATURAL_32} 1, l_x1___0.dat[1])
							gsl_gsl_vector_double_inst.gsl_vector_set (a_w.item.work2, l_lu___0, l_x2___1.dat[1])
							if l_lu___0 > {NATURAL_32} 1 then
								l_v1___2.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_T, l_lu___0 - {NATURAL_32} 1, (0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								l_v4.item := (rowcol_source_inst.gsl_matrix_subcolumn (a_T, l_lu___0, (0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								l_v2___2.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								l_v3.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work2, (0).to_natural_32, l_lu___0 - {NATURAL_32} 1)).twin
								blas_inst.gsl_blas_daxpy (-l_x1___0.dat[0], (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								blas_inst.gsl_blas_daxpy (-l_x2___1.dat[0], (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v4.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v2___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								blas_inst.gsl_blas_daxpy (-l_x1___0.dat[1], (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v1___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
								blas_inst.gsl_blas_daxpy (-l_x2___1.dat[1], (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v4.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_v3.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
							end
							l_l___0 := l_l___0 - 1
						end
						l_l___0 := l_l___0 - 1
					end
					l_y.item := (rowcol_source_inst.gsl_matrix_column (a_Z, l_iu - {NATURAL_32} 1)).twin
					l_y2.item := (rowcol_source_inst.gsl_matrix_column (a_Z, l_iu)).twin
					if l_iu > {NATURAL_32} 1 then
						l_Zv.item := (submatrix_source_inst.gsl_matrix_submatrix (a_Z, (0).to_natural_32, (0).to_natural_32, l_N, l_iu - {NATURAL_32} 1)).twin
						l_x___2.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work, (0).to_natural_32, l_iu - {NATURAL_32} 1)).twin
						blas_inst.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Zv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_iu - {NATURAL_32} 1)), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
						l_x___2.item := (subvector_source_inst.gsl_vector_subvector (a_w.item.work2, (0).to_natural_32, l_iu - {NATURAL_32} 1)).twin
						blas_inst.gsl_blas_dgemv ({PC_CBLAS_TRANSPOSE}.CblasNoTrans, 1.0, (create {CE_POINTER [PD_GSL_MATRIX]}.make_shared (l_Zv.item.c_pointer + (create {PD_GSL_MATRIX_VIEW}.make).offset_of ("matrix"))), (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_x___2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), (gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work2, l_iu)), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))).do_nothing
					else
						blas_inst.gsl_blas_dscal ((gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work, l_iu - {NATURAL_32} 1)), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
						blas_inst.gsl_blas_dscal ((gsl_gsl_vector_double_inst.gsl_vector_get (a_w.item.work2, l_iu)), create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
					end
					l_ecol.item := (rowcol_source_inst.gsl_matrix_complex_column (a_evec, l_iu - {NATURAL_32} 1)).twin
					l_ecol2.item := (rowcol_source_inst.gsl_matrix_complex_column (a_evec, l_iu)).twin
					l_ev.item := (reim_source_inst.gsl_vector_complex_imag (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_ev2.item := (reim_source_inst.gsl_vector_complex_imag (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol2.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_scale.item := 0.0
					l_ii := (0).to_natural_32
					from
					until
						not (l_ii < l_N)
					loop
						l_a___6 := (gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_ii))
						l_tmp___59 := fabs (l_a___6)
						if l_scale.item > l_tmp___59 + (fabs ((gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_ii)))) then
							l_scale.item := l_scale.item
						else
							l_tmp___56 := fabs (l_a___6)
							l_scale.item := l_tmp___56 + (fabs ((gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_ii))))
						end
						gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_ii, l_a___6)
						gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_ii, -l_a___6)
						l_ii := l_ii + 1
					end
					l_ev.item := (reim_source_inst.gsl_vector_complex_real (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_ev2.item := (reim_source_inst.gsl_vector_complex_real (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol2.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_ii := (0).to_natural_32
					from
					until
						not (l_ii < l_N)
					loop
						l_a___7 := (gsl_gsl_vector_double_inst.gsl_vector_get ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_y.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))), l_ii))
						gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_ii, l_a___7)
						gsl_gsl_vector_double_inst.gsl_vector_set (create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_ev2.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")), l_ii, l_a___7)
						l_ii := l_ii + 1
					end
					if l_scale.item /= 0.0 then
						l_scale.item := 1.0 / l_scale.item
					end
					blas_inst.gsl_blas_zdscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
					blas_inst.gsl_blas_zdscal (l_scale.item, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_ecol2.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
					l_i := l_i - 1
				end
				l_i := l_i - 1
			end
		end

	nonsymmv_normalize_eigenvectors (a_eval: CE_POINTER [PD_GSL_VECTOR_COMPLEX]; a_evec: CE_POINTER [PD_GSL_MATRIX_COMPLEX])
		local
			l_N: NATURAL_32
			l_i: NATURAL_32
			l_ei: PD_GSL_COMPLEX
			l_vi: CE_ARRAY [PD_GSL_VECTOR_COMPLEX_VIEW]
			l_re: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_im: CE_ARRAY [PD_GSL_VECTOR_VIEW]
			l_scale: REAL_64
			l_tmp___0: REAL_64
		do
			create l_ei.make
			create l_vi.make (<<1>>)
			create l_re.make (<<1>>)
			create l_im.make (<<1>>)

			l_N := a_evec.item.size1
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_N)
			loop
				l_ei := (gsl_gsl_vector_complex_double_inst.gsl_vector_complex_get (a_eval, l_i)).twin
				l_vi.item := (rowcol_source_inst.gsl_matrix_complex_column (a_evec, l_i)).twin
				l_re.item := (reim_source_inst.gsl_vector_complex_real (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
				if l_ei.dat[1] = 0.0 then
					l_scale := 1.0 / (blas_inst.gsl_blas_dnrm2 ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))))
					blas_inst.gsl_blas_dscal (l_scale, create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector")))
				elseif l_ei.dat[1] > 0.0 then
					l_im.item := (reim_source_inst.gsl_vector_complex_imag (create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))).twin
					l_tmp___0 := blas_inst.gsl_blas_dnrm2 ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_im.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))))
					l_scale := 1.0 / (hypot_inst.gsl_hypot ((blas_inst.gsl_blas_dnrm2 ((create {CE_POINTER [PD_GSL_VECTOR]}.make_shared (l_re.item.c_pointer + (create {PD_GSL_VECTOR_VIEW}.make).offset_of ("vector"))))), l_tmp___0))
					blas_inst.gsl_blas_zdscal (l_scale, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
					l_vi.item := (rowcol_source_inst.gsl_matrix_complex_column (a_evec, l_i + {NATURAL_32} 1)).twin
					blas_inst.gsl_blas_zdscal (l_scale, create {CE_POINTER [PD_GSL_VECTOR_COMPLEX]}.make_shared (l_vi.item.c_pointer + (create {PD_GSL_VECTOR_COMPLEX_VIEW}.make).offset_of ("vector")))
				end
				l_i := l_i + 1
			end
		end

end
