class
	P_POISSON

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_cdf_poisson_P (a_k: NATURAL_32; a_mu: REAL_64): REAL_64
		local
			l_P___2: REAL_64
			l_a___5: REAL_64
			js: JUMP_STATE
		do
			if a_mu <= 0.0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("mu <= 0"), ce_string ("poisson.c"), 56, 1)
					Result := (builtin_nanf (ce_string (""))).to_double
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_a___5 := a_k.to_real_64 + 1.0
				l_P___2 := gamma_inst.gsl_cdf_gamma_Q (a_mu, l_a___5, 1.0)
				Result := l_P___2
			end
		end

	gsl_cdf_poisson_Q (a_k: NATURAL_32; a_mu: REAL_64): REAL_64
		local
			l_Q___1: REAL_64
			l_a___5: REAL_64
			js: JUMP_STATE
		do
			if a_mu <= 0.0 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("mu <= 0"), ce_string ("poisson.c"), 77, 1)
					Result := (builtin_nanf (ce_string (""))).to_double
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_a___5 := a_k.to_real_64 + 1.0
				l_Q___1 := gamma_inst.gsl_cdf_gamma_P (a_mu, l_a___5, 1.0)
				Result := l_Q___1
			end
		end

	gsl_ran_poisson (a_r___3: CE_POINTER [PD_GSL_RNG]; a_mu: REAL_64): NATURAL_32
		local
			l_mu: REAL_64
			l_emu: REAL_64
			l_prod: REAL_64
			l_k: NATURAL_32
			l_m___4: NATURAL_32
			l_X: REAL_64
			js: JUMP_STATE
		do
			l_mu := a_mu
			l_prod := 1.0
			l_k := {NATURAL_32} 0
			from
			until
				not (not js.loop_done and then (l_mu > (10).to_double))
			loop
				l_m___4 := (l_mu * (7.0 / 8.0)).truncated_to_integer_64.to_natural_32
				l_X := (gamma_inst.gsl_ran_gamma_int (a_r___3, l_m___4))
				if l_X >= l_mu then
					Result := (l_k + (binomial_tpe_inst.gsl_ran_binomial (a_r___3, l_mu / l_X, l_m___4 - {NATURAL_32} 1)))
					js.return
				else
					l_k := l_k + l_m___4
					l_mu := l_mu - l_X
				end
			end
			if not js.return_called then
				l_emu := exp (-l_mu)
				from
				until
					js.loop_done
				loop
					l_prod := l_prod * (gsl_gsl_rng_inst.gsl_rng_uniform (a_r___3))
					l_k := l_k + 1
					if not (l_prod > l_emu) then
						js.break
					end
				end
				Result := (l_k - {NATURAL_32} 1)
			end
		end

	gsl_ran_poisson_array (a_r___3: CE_POINTER [PD_GSL_RNG]; a_n: NATURAL_32; a_array: CE_POINTER [NATURAL_32]; a_mu: REAL_64)
		local
			l_i: NATURAL_32
		do
			l_i := (0).to_natural_32
			from
			until
				not (l_i < a_n)
			loop
				a_array[l_i.to_integer_32] := gsl_ran_poisson (a_r___3, a_mu)
				l_i := l_i + 1
			end
		end

	gsl_ran_poisson_pdf (a_k: NATURAL_32; a_mu: REAL_64): REAL_64
		local
			l_p: REAL_64
			l_lf: REAL_64
		do
			l_lf := (gamma_inst.gsl_sf_lnfact (a_k))
			l_p := exp (((log (a_mu)) * a_k.to_real_64 - l_lf) - a_mu)
			Result := l_p
		end

end
