class
	P_POLY

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_interp_polynomial.make
			create polynomial_type.make (<<1>>)
		end

feature {ANY} -- attributes

	gsl_interp_polynomial: CE_POINTER [PD_GSL_INTERP_TYPE] assign set_gsl_interp_polynomial

	set_gsl_interp_polynomial (a_gsl_interp_polynomial: CE_POINTER [PD_GSL_INTERP_TYPE])
		do
			gsl_interp_polynomial := a_gsl_interp_polynomial
		end

	polynomial_type: CE_ARRAY [PD_GSL_INTERP_TYPE]

feature {ANY} -- routines

	polynomial_alloc (a_size: NATURAL_32): CE_POINTER [ANY]
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
			js: JUMP_STATE
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast ((malloc ((create {PD_POLYNOMIAL_STATE_T}.make).structure_size.to_natural_32)))
			if l_state.is_void then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("failed to allocate space for polynomial state"), ce_string ("poly.c"), 43, 8)
					Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_state.item.d := create {CE_POINTER [REAL_64]}.make_cast ((malloc ({PLATFORM}.Real_64_bytes.to_natural_32 * a_size)))
				if (l_state.item.d).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_state))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for d"), ce_string ("poly.c"), 51, 8)
						Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_state.item.coeff := create {CE_POINTER [REAL_64]}.make_cast ((malloc ({PLATFORM}.Real_64_bytes.to_natural_32 * a_size)))
				if (l_state.item.coeff).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_state.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_state))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for d"), ce_string ("poly.c"), 60, 8)
						Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_state.item.work := create {CE_POINTER [REAL_64]}.make_cast ((malloc ({PLATFORM}.Real_64_bytes.to_natural_32 * a_size)))
				if (l_state.item.work).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_state.item.coeff))
					free (create {CE_POINTER [ANY]}.make_cast (l_state.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_state))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for d"), ce_string ("poly.c"), 70, 8)
						Result := (create {CE_POINTER [ANY]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := (create {CE_POINTER [ANY]}.make_cast (l_state))
			end
		end

	polynomial_alloc_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_alloc)
		end

	polynomial_deriv (a_vstate: CE_POINTER [ANY]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_size: NATURAL_32; a_x: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_y: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast (a_vstate)
			dd_inst.gsl_poly_dd_taylor (l_state.item.coeff, a_x, l_state.item.d, a_xa, a_size, l_state.item.work).do_nothing
			a_y.item := l_state.item.coeff[1]
			Result := 0
		end

	polynomial_deriv_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_deriv)
		end

	polynomial_deriv2 (a_vstate: CE_POINTER [ANY]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_size: NATURAL_32; a_x: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_y: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast (a_vstate)
			dd_inst.gsl_poly_dd_taylor (l_state.item.coeff, a_x, l_state.item.d, a_xa, a_size, l_state.item.work).do_nothing
			a_y.item := 2.0 * l_state.item.coeff[2]
			Result := 0
		end

	polynomial_deriv2_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_deriv2)
		end

	polynomial_eval (a_vstate: CE_POINTER [ANY]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_size: NATURAL_32; a_x: REAL_64; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_y: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast (a_vstate)
			a_y.item := gsl_gsl_poly_inst.gsl_poly_dd_eval (l_state.item.d, a_xa, a_size, a_x)
			Result := 0
		end

	polynomial_eval_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_eval)
		end

	polynomial_free (a_vstate: CE_POINTER [ANY])
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast (a_vstate)
			free (create {CE_POINTER [ANY]}.make_cast (l_state.item.d))
			free (create {CE_POINTER [ANY]}.make_cast (l_state.item.coeff))
			free (create {CE_POINTER [ANY]}.make_cast (l_state.item.work))
			free (create {CE_POINTER [ANY]}.make_cast (l_state))
		end

	polynomial_free_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_free)
		end

	polynomial_init (a_vstate: CE_POINTER [ANY]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_size: NATURAL_32): INTEGER_32
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
			l_status: INTEGER_32
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast (a_vstate)
			l_status := (dd_inst.gsl_poly_dd_init (l_state.item.d, a_xa, a_ya, a_size))
			Result := l_status
		end

	polynomial_init_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_init)
		end

	polynomial_integ (a_vstate: CE_POINTER [ANY]; a_xa: CE_POINTER [REAL_64]; a_ya: CE_POINTER [REAL_64]; a_size: NATURAL_32; a_acc: CE_POINTER [PD_GSL_INTERP_ACCEL]; a_a___5: REAL_64; a_b___0: REAL_64; a_result: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_POLYNOMIAL_STATE_T]
			l_i: NATURAL_32
			l_sum: REAL_64
			l_tmp: REAL_64
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_POLYNOMIAL_STATE_T]}.make_cast (a_vstate)
			dd_inst.gsl_poly_dd_taylor (l_state.item.coeff, 0.0, l_state.item.d, a_xa, a_size, l_state.item.work).do_nothing
			l_sum := l_state.item.coeff[0] * (a_b___0 - a_a___5)
			l_i := (1).to_natural_32
			from
			until
				not (l_i < a_size)
			loop
				l_tmp := pow (a_b___0, (l_i + {NATURAL_32} 1).to_real_64)
				l_sum := l_sum + ((l_state.item.coeff[l_i.to_integer_32] * (l_tmp - (pow (a_a___5, (l_i + {NATURAL_32} 1).to_real_64)))) / (l_i.to_real_64 + 1.0))
				l_i := l_i + 1
			end
			a_result.item := l_sum
			Result := 0
		end

	polynomial_integ_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent polynomial_integ)
		end

end
