class
	P_QUAD_GOLDEN

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_min_fminimizer_quad_golden.make
			create quad_golden_type.make (<<1>>)
		end

feature {ANY} -- attributes

	gsl_min_fminimizer_quad_golden: CE_POINTER [PD_GSL_MIN_FMINIMIZER_TYPE] assign set_gsl_min_fminimizer_quad_golden

	set_gsl_min_fminimizer_quad_golden (a_gsl_min_fminimizer_quad_golden: CE_POINTER [PD_GSL_MIN_FMINIMIZER_TYPE])
		do
			gsl_min_fminimizer_quad_golden := a_gsl_min_fminimizer_quad_golden
		end

	quad_golden_type: CE_ARRAY [PD_GSL_MIN_FMINIMIZER_TYPE]

feature {ANY} -- routines

	quad_golden_init (a_vstate: CE_POINTER [ANY]; a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_x_minimum: REAL_64; a_f_minimum: REAL_64; a_x_lower: REAL_64; a_f_lower: REAL_64; a_x_upper: REAL_64; a_f_upper: REAL_64): INTEGER_32
		local
			l_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]
			l_x_lower: REAL_64
			l_f_lower: REAL_64
			l_x_upper: REAL_64
			l_f_upper: REAL_64
			l_state: CE_POINTER [PD_QUAD_GOLDEN_STATE_T]
		do
			create l_f.make
			create l_state.make

			l_f := a_f
			l_x_lower := a_x_lower
			l_f_lower := a_f_lower
			l_x_upper := a_x_upper
			l_f_upper := a_f_upper
			l_state := create {CE_POINTER [PD_QUAD_GOLDEN_STATE_T]}.make_cast (a_vstate)
			l_state.item.x_prev_small := a_x_minimum
			l_state.item.x_small := a_x_minimum
			l_state.item.f_prev_small := a_f_minimum
			l_state.item.f_small := a_f_minimum
			l_state.item.step_size := 0.0
			l_state.item.stored_step := 0.0
			l_state.item.prev_stored_step := 0.0
			l_state.item.num_iter := {NATURAL_32} 0
			l_x_lower := (0).to_double
			l_x_upper := (0).to_double
			l_f_lower := (0).to_double
			l_f_upper := (0).to_double
			l_f := create {CE_POINTER [PD_GSL_FUNCTION_STRUCT]}.make_shared (default_pointer)
			Result := 0
		end

	quad_golden_init_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent quad_golden_init)
		end

	quad_golden_iterate (a_vstate: CE_POINTER [ANY]; a_f: CE_POINTER [PD_GSL_FUNCTION_STRUCT]; a_x_minimum: CE_POINTER [REAL_64]; a_f_minimum: CE_POINTER [REAL_64]; a_x_lower: CE_POINTER [REAL_64]; a_f_lower: CE_POINTER [REAL_64]; a_x_upper: CE_POINTER [REAL_64]; a_f_upper: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_state: CE_POINTER [PD_QUAD_GOLDEN_STATE_T]
			l_x_m: REAL_64
			l_f_m: REAL_64
			l_x_l: REAL_64
			l_x_u: REAL_64
			l_x_small: REAL_64
			l_f_small: REAL_64
			l_x_prev_small: REAL_64
			l_f_prev_small: REAL_64
			l_stored_step: REAL_64
			l_prev_stored_step: REAL_64
			l_step_size: REAL_64
			l_quad_step_size: REAL_64
			l_x_trial: REAL_64
			l_x_eval: REAL_64
			l_f_eval: REAL_64
			l_x_midpoint: REAL_64
			l_tol: REAL_64
			l_c3___3: REAL_64
			l_c2___0: REAL_64
			l_c1___2: REAL_64
			l_tmp___2: REAL_64
			l_outside_interval: REAL_64
			l_inside_interval: REAL_64
			l_tmp___4: REAL_64
			l_step: REAL_64
			l_scale_factor: REAL_64
			l_tmp___7: REAL_64
			l_step___0: REAL_64
			l_tmp___9: REAL_64
			l_tmp___11: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_state.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state := create {CE_POINTER [PD_QUAD_GOLDEN_STATE_T]}.make_cast (a_vstate)
					l_x_m := a_x_minimum.item
					l_f_m := a_f_minimum.item
					l_x_l := a_x_lower.item
					l_x_u := a_x_upper.item
					l_x_small := l_state.item.x_small
					l_f_small := l_state.item.f_small
					l_x_prev_small := l_state.item.x_prev_small
					l_f_prev_small := l_state.item.f_prev_small
					l_stored_step := l_state.item.stored_step
					l_prev_stored_step := l_state.item.prev_stored_step
					l_step_size := l_state.item.step_size
					l_quad_step_size := l_prev_stored_step
					l_x_midpoint := 0.5 * (l_x_l + l_x_u)
					l_tol := 1.0e-06 * (fabs (l_x_m)) + 1.0e-10
					if (fabs (l_stored_step)) - l_tol > -2.0 * 2.2204460492503131e-16 then
						l_c3___3 := (l_x_m - l_x_small) * (l_f_m - l_f_prev_small)
						l_c2___0 := (l_x_m - l_x_prev_small) * (l_f_m - l_f_small)
						l_c1___2 := (l_x_m - l_x_prev_small) * l_c2___0 - (l_x_m - l_x_small) * l_c3___3
						l_c2___0 := 2.0 * (l_c2___0 - l_c3___3)
						if (fabs (l_c2___0)) > 2.2204460492503131e-16 then
							if l_c2___0 > 0.0 then
								l_c1___2 := -l_c1___2
							end
							l_c2___0 := fabs (l_c2___0)
							l_quad_step_size := l_c1___2 / l_c2___0
						else
							l_quad_step_size := l_stored_step
						end
						l_prev_stored_step := l_stored_step
						l_stored_step := l_step_size
					end
					l_x_trial := l_x_m + l_quad_step_size
					l_tmp___9 := fabs (l_quad_step_size)
					if not (l_tmp___9 < (fabs (0.5 * l_prev_stored_step))) then
						l_pos := -1 + 13 -- skip if block
					end
				when 1 then -- if:
						if not (l_x_trial > l_x_l) then
							l_pos := -1 + 11 -- skip if block
						end
				when 2 then -- if:
						if not (l_x_trial < l_x_u) then
							l_pos := -1 + 9 -- skip if block
						end
				when 3 then -- if:
						l_step_size := l_quad_step_size
						if not (l_x_trial - l_x_l < 2.0 * l_tol) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 7 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 8
				when 6 then -- else:
						if not (l_x_u - l_x_trial < 2.0 * l_tol) then
							l_pos := -1 + 8 -- skip if block
						end
				when 7 then -- if: | _L:
						if l_x_midpoint >= l_x_m then
							l_tmp___2 := 1.0
						else
							l_tmp___2 := -1.0
						end
						l_step_size := l_tmp___2 * (fabs (l_tol))
				when 8 then -- if done | if done | if block done, skip else block:
						l_pos := -1 + 10
				when 9 then -- else:
						l_pos := -1 + 13 -- goto _L___6
				when 10 then -- if done | if block done, skip else block:
						l_pos := -1 + 12
				when 11 then -- else:
						l_pos := -1 + 13 -- goto _L___6
				when 12 then -- if done | if block done, skip else block:
						l_pos := -1 + 26
				when 13 then -- else: | _L___6:
						if l_x_small = l_x_prev_small then
							l_pos := -1 + 16 -- skip if block
						end
				when 14 then -- if:
						if l_x_small < l_x_m then
							if l_x_prev_small < l_x_m then
								l_pos := -1 + 19 -- goto _L___0
							else
								l_pos := -1 + 16 -- goto _L___4
							end
						else
							l_pos := -1 + 16 -- goto _L___4
						end
				when 15 then -- if block done, skip else block:
						l_pos := -1 + 26
				when 16 then -- else: | _L___4:
						if l_x_small = l_x_prev_small then
							l_pos := -1 + 25 -- skip if block
						end
				when 17 then -- if:
						if not (l_x_small > l_x_m) then
							l_pos := -1 + 23 -- skip if block
						end
				when 18 then -- if:
						if not (l_x_prev_small > l_x_m) then
							l_pos := -1 + 21 -- skip if block
						end
				when 19 then -- if: | _L___0:
						if l_x_small < l_x_m then
							l_outside_interval := l_x_l - l_x_m
							l_inside_interval := l_x_u - l_x_m
						else
							l_outside_interval := l_x_u - l_x_m
							l_inside_interval := l_x_l - l_x_m
						end
						if (fabs (l_inside_interval)) <= l_tol then
							l_tmp___4 := l_outside_interval
							l_outside_interval := l_inside_interval
							l_inside_interval := l_tmp___4
						end
						l_step := l_inside_interval
						l_tmp___7 := fabs (l_outside_interval)
						if l_tmp___7 < (fabs (l_inside_interval)) then
							l_scale_factor := 0.5 * (sqrt (-l_outside_interval / l_inside_interval))
						else
							l_scale_factor := (5.0 / 11.0) * (0.1 - l_inside_interval / l_outside_interval)
						end
						l_state.item.stored_step := l_step
						l_step_size := l_scale_factor * l_step
				when 20 then -- if block done, skip else block:
						l_pos := -1 + 22
				when 21 then -- else:
						l_pos := -1 + 25 -- goto _L___2
				when 22 then -- if done | if block done, skip else block:
						l_pos := -1 + 24
				when 23 then -- else:
						l_pos := -1 + 25 -- goto _L___2
				when 24 then -- if done | if block done, skip else block:
						l_pos := -1 + 26
				when 25 then -- else: | _L___2:
						if l_x_m < l_x_midpoint then
							l_step___0 := l_x_u - l_x_m
						else
							l_step___0 := l_x_l - l_x_m
						end
						l_state.item.stored_step := l_step___0
						l_step_size := 0.3819660112501052 * l_step___0
				when 26 then -- if done | if done
						if (fabs (l_step_size)) > l_tol then
							l_x_eval := l_x_m + l_step_size
						else
							if l_step_size >= (0).to_double then
								l_tmp___11 := 1.0
							else
								l_tmp___11 := -1.0
							end
							l_x_eval := l_x_m + l_tmp___11 * (fabs (l_tol))
						end
						from
						until
							js.loop_done
						loop
							l_f_eval := (a_f.item.function).real_64_item ([l_x_eval, a_f.item.params])
							if not (infnan_inst.gsl_finite (l_f_eval)).to_boolean then
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("computed function value is infinite or NaN"), ce_string ("quad_golden.c"), 268, 9)
									Result := 9
									js.return
									if not js.return_called then
										js.break
									end
								end
							end
							if not js.return_called then
								js.break
							end
						end
						if not js.return_called then
							if l_f_eval <= l_f_m then
								if l_x_eval < l_x_m then
									a_x_upper.item := l_x_m
									a_f_upper.item := l_f_m
								else
									a_x_lower.item := l_x_m
									a_f_upper.item := l_f_m
								end
								l_state.item.x_prev_small := l_x_small
								l_state.item.f_prev_small := l_f_small
								l_state.item.x_small := l_x_m
								l_state.item.f_small := l_f_m
								a_x_minimum.item := l_x_eval
								a_f_minimum.item := l_f_eval
							else
								if l_x_eval < l_x_m then
									a_x_lower.item := l_x_eval
									a_f_lower.item := l_f_eval
								else
									a_x_upper.item := l_x_eval
									a_f_upper.item := l_f_eval
								end
								if l_f_eval <= l_f_small then
									l_state.item.x_prev_small := l_x_small
									l_state.item.f_prev_small := l_f_small
									l_state.item.x_small := l_x_eval
									l_state.item.f_small := l_f_eval
								elseif (fabs (l_x_small - l_x_m)) < 2.0 * 2.2204460492503131e-16 then
									l_state.item.x_prev_small := l_x_small
									l_state.item.f_prev_small := l_f_small
									l_state.item.x_small := l_x_eval
									l_state.item.f_small := l_f_eval
								elseif l_f_eval <= l_f_prev_small then
									l_state.item.x_prev_small := l_x_eval
									l_state.item.f_prev_small := l_f_eval
								elseif (fabs (l_x_prev_small - l_x_m)) < 2.0 * 2.2204460492503131e-16 then
									l_state.item.x_prev_small := l_x_eval
									l_state.item.f_prev_small := l_f_eval
								elseif (fabs (l_x_prev_small - l_x_small)) < 2.0 * 2.2204460492503131e-16 then
									l_state.item.x_prev_small := l_x_eval
									l_state.item.f_prev_small := l_f_eval
								end
							end
							l_state.item.stored_step := l_stored_step
							l_state.item.prev_stored_step := l_prev_stored_step
							l_state.item.step_size := l_step_size
							l_state.item.num_iter := (l_state.item.num_iter) + 1
							Result := 0
						end
				when 27 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	quad_golden_iterate_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent quad_golden_iterate)
		end

end
