class
	P_RANLXS

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_rng_ranlxs0.make
			create gsl_rng_ranlxs1.make
			create gsl_rng_ranlxs2.make
			create next.make (<<12>>)
			create ranlxs0_type.make (<<1>>)
			create ranlxs1_type.make (<<1>>)
			create ranlxs2_type.make (<<1>>)
			create snext.make (<<24>>)
		end

feature {ANY} -- attributes

	gsl_rng_ranlxs0: CE_POINTER [PD_GSL_RNG_TYPE] assign set_gsl_rng_ranlxs0

	set_gsl_rng_ranlxs0 (a_gsl_rng_ranlxs0: CE_POINTER [PD_GSL_RNG_TYPE])
		do
			gsl_rng_ranlxs0 := a_gsl_rng_ranlxs0
		end

	gsl_rng_ranlxs1: CE_POINTER [PD_GSL_RNG_TYPE] assign set_gsl_rng_ranlxs1

	set_gsl_rng_ranlxs1 (a_gsl_rng_ranlxs1: CE_POINTER [PD_GSL_RNG_TYPE])
		do
			gsl_rng_ranlxs1 := a_gsl_rng_ranlxs1
		end

	gsl_rng_ranlxs2: CE_POINTER [PD_GSL_RNG_TYPE] assign set_gsl_rng_ranlxs2

	set_gsl_rng_ranlxs2 (a_gsl_rng_ranlxs2: CE_POINTER [PD_GSL_RNG_TYPE])
		do
			gsl_rng_ranlxs2 := a_gsl_rng_ranlxs2
		end

	next: CE_ARRAY [INTEGER_32]

	one_bit: REAL_64 assign set_one_bit

	set_one_bit (a_one_bit: REAL_64)
		do
			one_bit := a_one_bit
		end

	ranlxs0_type: CE_ARRAY [PD_GSL_RNG_TYPE]

	ranlxs1_type: CE_ARRAY [PD_GSL_RNG_TYPE]

	ranlxs2_type: CE_ARRAY [PD_GSL_RNG_TYPE]

	sbase: REAL_64 assign set_sbase

	set_sbase (a_sbase: REAL_64)
		do
			sbase := a_sbase
		end

	shift: REAL_64 assign set_shift

	set_shift (a_shift: REAL_64)
		do
			shift := a_shift
		end

	snext: CE_ARRAY [INTEGER_32]

	sone_bit: REAL_64 assign set_sone_bit

	set_sone_bit (a_sone_bit: REAL_64)
		do
			sone_bit := a_sone_bit
		end

feature {ANY} -- routines

	increment_state (a_state: CE_POINTER [PD_RANLXS_STATE_T])
		local
			l_k: INTEGER_32
			l_kmax___0: INTEGER_32
			l_m___4: INTEGER_32
			l_x: REAL_64
			l_y1: REAL_64
			l_y2: REAL_64
			l_y3: REAL_64
			l_xflt: CE_POINTER [REAL_32]
			l_xdbl: CE_POINTER [REAL_64]
			l_ydbl: CE_POINTER [REAL_64]
			l_carry: REAL_64
			l_ir: NATURAL_32
			l_jr: NATURAL_32
			l_tmp: INTEGER_32
			l_tmp___0: INTEGER_32
		do
			create l_xflt.make
			create l_xdbl.make
			create l_ydbl.make

			l_xflt := a_state.item.xflt
			l_xdbl := a_state.item.xdbl
			l_ydbl := a_state.item.ydbl
			l_carry := a_state.item.carry
			l_ir := a_state.item.ir
			l_jr := a_state.item.jr
			l_k := 0
			from
			until
				not (l_ir > {NATURAL_32} 0)
			loop
				l_y1 := l_xdbl[l_jr.to_integer_32] - l_xdbl[l_ir.to_integer_32]
				l_y2 := l_y1 - l_carry
				if l_y2 < (0).to_double then
					l_carry := one_bit
					l_y2 := l_y2 + (1).to_double
				else
					l_carry := (0).to_double
				end
				l_xdbl[l_ir.to_integer_32] := l_y2
				l_ir := (next[l_ir.to_integer_32]).to_natural_32
				l_jr := (next[l_jr.to_integer_32]).to_natural_32
				l_k := l_k + 1
			end
			l_kmax___0 := (a_state.item.pr - {NATURAL_32} 12).to_integer_32
			from
			until
				not (l_k <= l_kmax___0)
			loop
				l_y1 := l_xdbl[7] - l_xdbl[0]
				l_y1 := l_y1 - l_carry
				l_y2 := l_xdbl[8] - l_xdbl[1]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - one_bit
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[0] := l_y1
				l_y3 := l_xdbl[9] - l_xdbl[2]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - one_bit
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[1] := l_y2
				l_y1 := l_xdbl[10] - l_xdbl[3]
				if l_y3 < (0).to_double then
					l_y1 := l_y1 - one_bit
					l_y3 := l_y3 + (1).to_double
				end
				l_xdbl[2] := l_y3
				l_y2 := l_xdbl[11] - l_xdbl[4]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - one_bit
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[3] := l_y1
				l_y3 := l_xdbl[0] - l_xdbl[5]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - one_bit
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[4] := l_y2
				l_y1 := l_xdbl[1] - l_xdbl[6]
				if l_y3 < (0).to_double then
					l_y1 := l_y1 - one_bit
					l_y3 := l_y3 + (1).to_double
				end
				l_xdbl[5] := l_y3
				l_y2 := l_xdbl[2] - l_xdbl[7]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - one_bit
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[6] := l_y1
				l_y3 := l_xdbl[3] - l_xdbl[8]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - one_bit
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[7] := l_y2
				l_y1 := l_xdbl[4] - l_xdbl[9]
				if l_y3 < (0).to_double then
					l_y1 := l_y1 - one_bit
					l_y3 := l_y3 + (1).to_double
				end
				l_xdbl[8] := l_y3
				l_y2 := l_xdbl[5] - l_xdbl[10]
				if l_y1 < (0).to_double then
					l_y2 := l_y2 - one_bit
					l_y1 := l_y1 + (1).to_double
				end
				l_xdbl[9] := l_y1
				l_y3 := l_xdbl[6] - l_xdbl[11]
				if l_y2 < (0).to_double then
					l_y3 := l_y3 - one_bit
					l_y2 := l_y2 + (1).to_double
				end
				l_xdbl[10] := l_y2
				if l_y3 < (0).to_double then
					l_carry := one_bit
					l_y3 := l_y3 + (1).to_double
				else
					l_carry := (0).to_double
				end
				l_xdbl[11] := l_y3
				l_k := l_k + (12)
			end
			l_kmax___0 := (a_state.item.pr).to_integer_32
			from
			until
				not (l_k < l_kmax___0)
			loop
				l_y1 := l_xdbl[l_jr.to_integer_32] - l_xdbl[l_ir.to_integer_32]
				l_y2 := l_y1 - l_carry
				if l_y2 < (0).to_double then
					l_carry := one_bit
					l_y2 := l_y2 + (1).to_double
				else
					l_carry := (0).to_double
				end
				l_xdbl[l_ir.to_integer_32] := l_y2
				l_ydbl[l_ir.to_integer_32] := l_y2 + shift
				l_ir := (next[l_ir.to_integer_32]).to_natural_32
				l_jr := (next[l_jr.to_integer_32]).to_natural_32
				l_k := l_k + 1
			end
			l_ydbl[l_ir.to_integer_32] := l_xdbl[l_ir.to_integer_32] + shift
			l_k := next[l_ir.to_integer_32]
			from
			until
				not (l_k > 0)
			loop
				l_ydbl[l_k] := l_xdbl[l_k] + shift
				l_k := next[l_k]
			end
			l_k := 0
			l_m___4 := 0
			from
			until
				not (l_k < 12)
			loop
				l_x := l_xdbl[l_k]
				l_y2 := l_ydbl[l_k] - shift
				if l_y2 > l_x then
					l_y2 := l_y2 - sone_bit
				end
				l_y1 := (l_x - l_y2) * sbase
				l_tmp := l_m___4
				l_m___4 := l_m___4 + 1
				l_xflt[l_tmp] := l_y1.truncated_to_real
				l_tmp___0 := l_m___4
				l_m___4 := l_m___4 + 1
				l_xflt[l_tmp___0] := l_y2.truncated_to_real
				l_k := l_k + 1
			end
			a_state.item.ir := l_ir
			a_state.item.is := {NATURAL_32} 2 * l_ir
			a_state.item.is_old := {NATURAL_32} 2 * l_ir
			a_state.item.jr := l_jr
			a_state.item.carry := l_carry
		end

	ranlxs0_set (a_vstate: CE_POINTER [ANY]; a_s: NATURAL_32)
		do
			ranlxs_set_lux (a_vstate, a_s, {NATURAL_32} 109)
		end

	ranlxs0_set_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ranlxs0_set)
		end

	ranlxs1_set (a_vstate: CE_POINTER [ANY]; a_s: NATURAL_32)
		do
			ranlxs_set_lux (a_vstate, a_s, {NATURAL_32} 202)
		end

	ranlxs1_set_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ranlxs1_set)
		end

	ranlxs2_set (a_vstate: CE_POINTER [ANY]; a_s: NATURAL_32)
		do
			ranlxs_set_lux (a_vstate, a_s, {NATURAL_32} 397)
		end

	ranlxs2_set_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ranlxs2_set)
		end

	ranlxs_get (a_vstate: CE_POINTER [ANY]): NATURAL_32
		do
			Result := ((ranlxs_get_double (a_vstate)) * 16777216.0).truncated_to_integer_64.to_natural_32
		end

	ranlxs_get_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ranlxs_get)
		end

	ranlxs_get_double (a_vstate: CE_POINTER [ANY]): REAL_64
		local
			l_state: CE_POINTER [PD_RANLXS_STATE_T]
			l_is: NATURAL_32
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_RANLXS_STATE_T]}.make_cast (a_vstate)
			l_is := (snext[(l_state.item.is).to_integer_32]).to_natural_32
			l_state.item.is := l_is
			if l_is = l_state.item.is_old then
				increment_state (l_state)
			end
			Result := (l_state.item.xflt[(l_state.item.is).to_integer_32]).to_double
		end

	ranlxs_get_double_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent ranlxs_get_double)
		end

	ranlxs_set_lux (a_vstate: CE_POINTER [ANY]; a_s: NATURAL_32; a_luxury: NATURAL_32)
		local
			l_s: NATURAL_32
			l_state: CE_POINTER [PD_RANLXS_STATE_T]
			l_ibit: INTEGER_32
			l_jbit: INTEGER_32
			l_i: INTEGER_32
			l_k: INTEGER_32
			l_m___4: INTEGER_32
			l_xbit: CE_ARRAY [INTEGER_32]
			l_x: REAL_64
			l_y: REAL_64
			l_seed: INTEGER_32
			js: JUMP_STATE
		do
			create l_state.make
			create l_xbit.make (<<31>>)

			l_s := a_s
			l_state := create {CE_POINTER [PD_RANLXS_STATE_T]}.make_cast (a_vstate)
			if l_s = {NATURAL_32} 0 then
				l_s := {NATURAL_32} 1
			end
			l_seed := l_s.to_integer_32
			l_i := ((l_seed.to_natural_32).bit_and ({NATURAL_32} 4_294_967_295)).to_integer_32
			l_k := 0
			from
			until
				not (l_k < 31)
			loop
				l_xbit[l_k] := l_i \\ 2
				l_i := l_i // (2)
				l_k := l_k + 1
			end
			l_ibit := 0
			l_jbit := 18
			l_k := 0
			from
			until
				not (not js.loop_done and then (l_k < 12))
			loop
				l_x := (0).to_double
				l_m___4 := 1
				from
				until
					not (l_m___4 <= 48)
				loop
					l_y := (l_xbit[l_ibit]).to_double
					l_x := l_x + (l_x + l_y)
					l_xbit[l_ibit] := (l_xbit[l_ibit] + l_xbit[l_jbit]) \\ 2
					l_ibit := (l_ibit + 1) \\ 31
					l_jbit := (l_jbit + 1) \\ 31
					l_m___4 := l_m___4 + 1
				end
				l_state.item.xdbl[l_k] := (one_bit * l_x)
				l_k := l_k + 1
			end
			l_state.item.carry := (0).to_double
			l_state.item.ir := {NATURAL_32} 0
			l_state.item.jr := {NATURAL_32} 7
			l_state.item.is := {NATURAL_32} 23
			l_state.item.is_old := {NATURAL_32} 0
			l_state.item.pr := a_luxury
		end

end
