class
	P_RAYLEIGH

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_cdf_rayleigh_P (a_x: REAL_64; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_P___2: REAL_64
		do
			l_u := a_x / a_sigma
			l_P___2 := -(expm1 ((-l_u * l_u) / (2).to_double))
			Result := l_P___2
		end

	gsl_cdf_rayleigh_Q (a_x: REAL_64; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_Q___1: REAL_64
		do
			l_u := a_x / a_sigma
			l_Q___1 := (exp ((-l_u * l_u) / (2).to_double))
			Result := l_Q___1
		end

	gsl_ran_rayleigh (a_r___3: CE_POINTER [PD_GSL_RNG]; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
		do
			l_u := (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))
			Result := (a_sigma * (sqrt (-2.0 * (log (l_u)))))
		end

	gsl_ran_rayleigh_pdf (a_x: REAL_64; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_p: REAL_64
		do
			if a_x < (0).to_double then
				Result := (0).to_double
			else
				l_u := a_x / a_sigma
				l_p := (l_u / a_sigma) * (exp ((-l_u * l_u) / 2.0))
				Result := l_p
			end
		end

	gsl_ran_rayleigh_tail (a_r___3: CE_POINTER [PD_GSL_RNG]; a_a___5: REAL_64; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
		do
			l_u := (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))
			Result := (sqrt (a_a___5 * a_a___5 - ((2.0 * a_sigma) * a_sigma) * (log (l_u))))
		end

	gsl_ran_rayleigh_tail_pdf (a_x: REAL_64; a_a___5: REAL_64; a_sigma: REAL_64): REAL_64
		local
			l_u: REAL_64
			l_v: REAL_64
			l_p: REAL_64
		do
			if a_x < a_a___5 then
				Result := (0).to_double
			else
				l_u := a_x / a_sigma
				l_v := a_a___5 / a_sigma
				l_p := (l_u / a_sigma) * (exp (((l_v + l_u) * (l_v - l_u)) / 2.0))
				Result := l_p
			end
		end

end
