class
	P_READ

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_ieee_read_mode_string (a_description: CE_POINTER [INTEGER_8]; a_precision: CE_POINTER [INTEGER_32]; a_rounding: CE_POINTER [INTEGER_32]; a_exception_mask: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_start: CE_POINTER [INTEGER_8]
			l_end: CE_POINTER [INTEGER_8]
			l_p: CE_POINTER [INTEGER_8]
			l_precision_count: INTEGER_32
			l_rounding_count: INTEGER_32
			l_exception_count: INTEGER_32
			l_status: INTEGER_32
			l_new_precision: CE_ARRAY [INTEGER_32]
			l_new_rounding: CE_ARRAY [INTEGER_32]
			l_new_exception: CE_ARRAY [INTEGER_32]
			js: JUMP_STATE
		do
			create l_start.make
			create l_end.make
			create l_p.make
			create l_new_precision.make (<<1>>)
			create l_new_rounding.make (<<1>>)
			create l_new_exception.make (<<1>>)

			l_precision_count := 0
			l_rounding_count := 0
			l_exception_count := 0
			l_start := create {CE_POINTER [INTEGER_8]}.make_cast ((malloc ((strlen (a_description)) + {NATURAL_32} 1)))
			if l_start.is_void then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("no memory to parse mode string"), ce_string ("read.c"), 48, 8)
					Result := 8
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				strcpy (l_start, a_description).do_nothing
				l_p := l_start
				a_precision.item := 0
				a_rounding.item := 0
				a_exception_mask.item := 0
				from
				until
					js.loop_done
				loop
					l_end := strchr (l_p, (',').code)
					if (not l_end.is_void) then
						l_end[0] := (('%/0/').code).to_integer_8
						from
						until
							js.loop_done
						loop
							l_end := l_end + 1
							if (l_end[0]).to_integer_32 /= 32 then
								if (l_end[0]).to_integer_32 /= 44 then
									js.break
								end
							end
						end
					end
					l_new_precision.item := 0
					l_new_rounding.item := 0
					l_new_exception.item := 0
					l_status := lookup_string (l_p, l_new_precision, l_new_rounding, l_new_exception)
					if l_status.to_boolean then
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("unrecognized GSL_IEEE_MODE string.%NValid settings are:%N%N  single-precision double-precision extended-precision%N  round-to-nearest round-down round-up round-to-zero%N  mask-invalid mask-denormalized mask-division-by-zero%N  mask-overflow mask-underflow mask-all%N  trap-common trap-inexact%N%Nseparated by commas. (e.g. GSL_IEEE_MODE=%"round-down,mask-underflow%")"), ce_string ("read.c"), 91, 4)
							Result := 4
							js.return
							if not js.return_called then
								js.break
							end
						end
					end
					if not js.return_called and then ((l_new_precision.item).to_boolean) then
						a_precision.item := l_new_precision.item
						l_precision_count := l_precision_count + 1
						if l_precision_count > 1 then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("attempted to set IEEE precision twice"), ce_string ("read.c"), 98, 4)
								Result := 4
								js.return
								if not js.return_called then
									js.break
								end
							end
						end
					end
					if not js.return_called and then ((l_new_rounding.item).to_boolean) then
						a_rounding.item := l_new_rounding.item
						l_rounding_count := l_rounding_count + 1
						if l_rounding_count > 1 then
							from
							until
								js.loop_done
							loop
								error_inst.gsl_error (ce_string ("attempted to set IEEE rounding mode twice"), ce_string ("read.c"), 106, 4)
								Result := 4
								js.return
								if not js.return_called then
									js.break
								end
							end
						end
					end
					if not js.return_called then
						if (l_new_exception.item).to_boolean then
							a_exception_mask.item := (a_exception_mask.item).bit_or ((l_new_exception.item))
							l_exception_count := l_exception_count + 1
						end
						l_p := l_end
						if (not l_end.is_void) then
							if (l_p[0]).to_integer_32 = 0 then
								js.break
							end
						else
							js.break
						end
					end
				end
			end
			if not js.return_called then
				free (create {CE_POINTER [ANY]}.make_cast (l_start))
				Result := 0
			end
		end

	lookup_string (a_p: CE_POINTER [INTEGER_8]; a_precision: CE_POINTER [INTEGER_32]; a_rounding: CE_POINTER [INTEGER_32]; a_exception_mask: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if (strcmp (a_p, ce_string ("single-precision"))) = 0 then
				a_precision.item := 1
			elseif (strcmp (a_p, ce_string ("double-precision"))) = 0 then
				a_precision.item := 2
			elseif (strcmp (a_p, ce_string ("extended-precision"))) = 0 then
				a_precision.item := 3
			elseif (strcmp (a_p, ce_string ("round-to-nearest"))) = 0 then
				a_rounding.item := 1
			elseif (strcmp (a_p, ce_string ("round-down"))) = 0 then
				a_rounding.item := 2
			elseif (strcmp (a_p, ce_string ("round-up"))) = 0 then
				a_rounding.item := 3
			elseif (strcmp (a_p, ce_string ("round-to-zero"))) = 0 then
				a_rounding.item := 4
			elseif (strcmp (a_p, ce_string ("mask-all"))) = 0 then
				a_exception_mask.item := 31
			elseif (strcmp (a_p, ce_string ("mask-invalid"))) = 0 then
				a_exception_mask.item := 1
			elseif (strcmp (a_p, ce_string ("mask-denormalized"))) = 0 then
				a_exception_mask.item := 2
			elseif (strcmp (a_p, ce_string ("mask-division-by-zero"))) = 0 then
				a_exception_mask.item := 4
			elseif (strcmp (a_p, ce_string ("mask-overflow"))) = 0 then
				a_exception_mask.item := 8
			elseif (strcmp (a_p, ce_string ("mask-underflow"))) = 0 then
				a_exception_mask.item := 16
			elseif (strcmp (a_p, ce_string ("trap-inexact"))) = 0 then
				a_exception_mask.item := 32
			elseif (strcmp (a_p, ce_string ("trap-common"))) = 0 then
				Result := 0
				js.return
			else
				Result := 1
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

end
