class
	P_STEEPEST_DESCENT

inherit
	LIBGSL_DATA
		redefine
			default_create
		end
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {NONE} -- Initialization

	default_create
		do
			create gsl_multimin_fdfminimizer_steepest_descent.make
			create steepest_descent_type.make (<<1>>)
		end

feature {ANY} -- attributes

	gsl_multimin_fdfminimizer_steepest_descent: CE_POINTER [PD_GSL_MULTIMIN_FDFMINIMIZER_TYPE] assign set_gsl_multimin_fdfminimizer_steepest_descent

	set_gsl_multimin_fdfminimizer_steepest_descent (a_gsl_multimin_fdfminimizer_steepest_descent: CE_POINTER [PD_GSL_MULTIMIN_FDFMINIMIZER_TYPE])
		do
			gsl_multimin_fdfminimizer_steepest_descent := a_gsl_multimin_fdfminimizer_steepest_descent
		end

	steepest_descent_type: CE_ARRAY [PD_GSL_MULTIMIN_FDFMINIMIZER_TYPE]

feature {ANY} -- routines

	steepest_descent_alloc (a_vstate: CE_POINTER [ANY]; a_n: NATURAL_32): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]
			js: JUMP_STATE
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]}.make_cast (a_vstate)
			l_state.item.x1 := init_source_inst.gsl_vector_alloc (a_n)
			if (l_state.item.x1).is_void then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("failed to allocate space for x1"), ce_string ("steepest_descent.c"), 48, 8)
					Result := 8
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_state.item.g1 := init_source_inst.gsl_vector_alloc (a_n)
				if (l_state.item.g1).is_void then
					init_source_inst.gsl_vector_free (l_state.item.x1)
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for g1"), ce_string ("steepest_descent.c"), 56, 8)
						Result := 8
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	steepest_descent_alloc_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steepest_descent_alloc)
		end

	steepest_descent_free (a_vstate: CE_POINTER [ANY])
		local
			l_state: CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]}.make_cast (a_vstate)
			init_source_inst.gsl_vector_free (l_state.item.x1)
			init_source_inst.gsl_vector_free (l_state.item.g1)
		end

	steepest_descent_free_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steepest_descent_free)
		end

	steepest_descent_iterate (a_vstate: CE_POINTER [ANY]; a_fdf: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_FDF_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]; a_f: CE_POINTER [REAL_64]; a_gradient: CE_POINTER [PD_GSL_VECTOR]; a_dx: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]
			l_x1___0: CE_POINTER [PD_GSL_VECTOR]
			l_g1: CE_POINTER [PD_GSL_VECTOR]
			l_f0: REAL_64
			l_f1: CE_ARRAY [REAL_64]
			l_step: REAL_64
			l_tol: REAL_64
			l_failed___0: INTEGER_32
			l_gnorm: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_state.make
			create l_x1___0.make
			create l_g1.make
			create l_f1.make (<<1>>)

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_state := create {CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]}.make_cast (a_vstate)
					l_x1___0 := l_state.item.x1
					l_g1 := l_state.item.g1
					l_f0 := a_f.item
					l_step := l_state.item.step
					l_tol := l_state.item.tol
					l_failed___0 := 0
					l_gnorm := (blas_inst.gsl_blas_dnrm2 (a_gradient))
					if l_gnorm = 0.0 then
						init_source_inst.gsl_vector_set_zero (a_dx)
						Result := 27
						js.return
					end
					if js.return_called then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if: | trial:
						init_source_inst.gsl_vector_set_zero (a_dx)
						blas_inst.gsl_blas_daxpy (-l_step / l_gnorm, a_gradient, a_dx).do_nothing
						copy_source_inst.gsl_vector_memcpy (l_x1___0, a_x).do_nothing
						blas_inst.gsl_blas_daxpy (1.0, a_dx, l_x1___0).do_nothing;
						(a_fdf.item.fdf).call ([l_x1___0, a_fdf.item.params, l_f1, l_g1])
						if l_f1.item > l_f0 then
							l_failed___0 := 1
							l_step := l_step * l_tol
							l_pos := -1 + 1 -- goto trial
						end
				when 2 then
						if l_failed___0.to_boolean then
							l_step := l_step * l_tol
						else
							l_step := l_step * (2.0)
						end
						l_state.item.step := l_step
						copy_source_inst.gsl_vector_memcpy (a_x, l_x1___0).do_nothing
						copy_source_inst.gsl_vector_memcpy (a_gradient, l_g1).do_nothing
						a_f.item := l_f1.item
						Result := 0
				when 3 then
					l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
		end

	steepest_descent_iterate_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steepest_descent_iterate)
		end

	steepest_descent_restart (a_vstate: CE_POINTER [ANY]): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]}.make_cast (a_vstate)
			l_state.item.step := l_state.item.max_step
			Result := 0
		end

	steepest_descent_restart_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steepest_descent_restart)
		end

	steepest_descent_set (a_vstate: CE_POINTER [ANY]; a_fdf: CE_POINTER [PD_GSL_MULTIMIN_FUNCTION_FDF_STRUCT]; a_x: CE_POINTER [PD_GSL_VECTOR]; a_f: CE_POINTER [REAL_64]; a_gradient: CE_POINTER [PD_GSL_VECTOR]; a_step_size: REAL_64; a_tol: REAL_64): INTEGER_32
		local
			l_state: CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]
		do
			create l_state.make

			l_state := create {CE_POINTER [PD_STEEPEST_DESCENT_STATE_T]}.make_cast (a_vstate);
			(a_fdf.item.fdf).call ([a_x, a_fdf.item.params, a_f, a_gradient])
			l_state.item.step := a_step_size
			l_state.item.max_step := a_step_size
			l_state.item.tol := a_tol
			Result := 0
		end

	steepest_descent_set_address: CE_ROUTINE
		once ("OBJECT")
			create Result.make_agent (agent steepest_descent_set)
		end

end
