class
	P_TRIDIAG

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	gsl_linalg_solve_cyc_tridiag (a_diag: CE_POINTER [PD_GSL_VECTOR]; a_abovediag: CE_POINTER [PD_GSL_VECTOR]; a_belowdiag: CE_POINTER [PD_GSL_VECTOR]; a_rhs: CE_POINTER [PD_GSL_VECTOR]; a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_diag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 553, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_abovediag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of abovediag must match rhs"), ce_string ("tridiag.c"), 557, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_belowdiag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of belowdiag must match rhs"), ce_string ("tridiag.c"), 561, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 565, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_diag.item.size < {NATURAL_32} 3 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of cyclic system must be 3 or more"), ce_string ("tridiag.c"), 569, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				Result := (solve_cyc_tridiag_nonsym (a_diag.item.data, a_diag.item.stride, a_abovediag.item.data, a_abovediag.item.stride, a_belowdiag.item.data, a_belowdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, a_diag.item.size))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_linalg_solve_symm_cyc_tridiag (a_diag: CE_POINTER [PD_GSL_VECTOR]; a_offdiag: CE_POINTER [PD_GSL_VECTOR]; a_rhs: CE_POINTER [PD_GSL_VECTOR]; a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_diag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 519, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_offdiag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of offdiag must match rhs"), ce_string ("tridiag.c"), 523, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 527, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_diag.item.size < {NATURAL_32} 3 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of cyclic system must be 3 or more"), ce_string ("tridiag.c"), 531, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				Result := (solve_cyc_tridiag (a_diag.item.data, a_diag.item.stride, a_offdiag.item.data, a_offdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, a_diag.item.size))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_linalg_solve_symm_tridiag (a_diag: CE_POINTER [PD_GSL_VECTOR]; a_offdiag: CE_POINTER [PD_GSL_VECTOR]; a_rhs: CE_POINTER [PD_GSL_VECTOR]; a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_diag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 453, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_offdiag.item.size /= a_rhs.item.size - {NATURAL_32} 1 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of offdiag must match rhs-1"), ce_string ("tridiag.c"), 457, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 461, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				Result := (solve_tridiag (a_diag.item.data, a_diag.item.stride, a_offdiag.item.data, a_offdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, a_diag.item.size))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_linalg_solve_tridiag (a_diag: CE_POINTER [PD_GSL_VECTOR]; a_abovediag: CE_POINTER [PD_GSL_VECTOR]; a_belowdiag: CE_POINTER [PD_GSL_VECTOR]; a_rhs: CE_POINTER [PD_GSL_VECTOR]; a_solution: CE_POINTER [PD_GSL_VECTOR]): INTEGER_32
		local
			js: JUMP_STATE
		do
			if a_diag.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of diag must match rhs"), ce_string ("tridiag.c"), 484, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_abovediag.item.size /= a_rhs.item.size - {NATURAL_32} 1 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of abovediag must match rhs-1"), ce_string ("tridiag.c"), 488, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_belowdiag.item.size /= a_rhs.item.size - {NATURAL_32} 1 then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of belowdiag must match rhs-1"), ce_string ("tridiag.c"), 492, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			elseif a_solution.item.size /= a_rhs.item.size then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("size of solution must match rhs"), ce_string ("tridiag.c"), 496, 19)
					Result := 19
					js.return
					if not js.return_called then
						js.break
					end
				end
			else
				Result := (solve_tridiag_nonsym (a_diag.item.data, a_diag.item.stride, a_abovediag.item.data, a_abovediag.item.stride, a_belowdiag.item.data, a_belowdiag.item.stride, a_rhs.item.data, a_rhs.item.stride, a_solution.item.data, a_solution.item.stride, a_diag.item.size))
				js.return
			end
			if not js.return_called then
				Result := 0
			end
		end

	solve_cyc_tridiag (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_offdiag: CE_POINTER [REAL_64]; a_o_stride: NATURAL_32; a_b___0: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride: NATURAL_32; a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_delta___0: CE_POINTER [REAL_64]
			l_gamma___0: CE_POINTER [REAL_64]
			l_alpha: CE_POINTER [REAL_64]
			l_c___1: CE_POINTER [REAL_64]
			l_z: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_sum: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_delta___0.make
			create l_gamma___0.make
			create l_alpha.make
			create l_c___1.make
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_delta___0 := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_gamma___0 := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_alpha := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_c___1 := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_z := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_delta___0.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 3 then -- else:
						if not (l_gamma___0.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 6 then -- else:
						if not (l_alpha.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 9 then -- else:
						if not (l_c___1.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if:
						l_pos := -1 + 13 -- goto _L
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 12 then -- else:
						if not (l_z.is_void) then
							l_pos := -1 + 15 -- skip if block
						end
				when 13 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 225, 8)
							Result := 8
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 14 then -- if block done, skip else block:
						l_pos := -1 + 16
				when 15 then -- else:
						l_sum := 0.0
						if a_N = {NATURAL_32} 1 then
							a_x[0] := (a_b___0[0] / a_diag[0])
							Result := 0
							js.return
						end
						if not js.return_called then
							l_alpha[0] := a_diag[0]
							l_gamma___0[0] := (a_offdiag[0] / l_alpha[0])
							l_delta___0[0] := (a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 1)).to_integer_32] / l_alpha[0])
							if l_alpha[0] = (0).to_double then
								l_status := 12
							end
							l_i := (1).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 2)
							loop
								l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - a_offdiag[(a_o_stride * (l_i - {NATURAL_32} 1)).to_integer_32] * l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32])
								l_gamma___0[l_i.to_integer_32] := (a_offdiag[(a_o_stride * l_i).to_integer_32] / l_alpha[l_i.to_integer_32])
								l_delta___0[l_i.to_integer_32] := (-l_delta___0[(l_i - {NATURAL_32} 1).to_integer_32] * a_offdiag[(a_o_stride * (l_i - {NATURAL_32} 1)).to_integer_32]) / l_alpha[l_i.to_integer_32]
								if l_alpha[l_i.to_integer_32] = (0).to_double then
									l_status := 12
								end
								l_i := l_i + 1
							end
							l_i := (0).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 2)
							loop
								l_sum := l_sum + ((l_alpha[l_i.to_integer_32] * l_delta___0[l_i.to_integer_32]) * l_delta___0[l_i.to_integer_32])
								l_i := l_i + 1
							end
							l_alpha[(a_N - {NATURAL_32} 2).to_integer_32] := (a_diag[(a_d_stride * (a_N - {NATURAL_32} 2)).to_integer_32] - a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 3)).to_integer_32] * l_gamma___0[(a_N - {NATURAL_32} 3).to_integer_32])
							l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32] := ((a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 2)).to_integer_32] - a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 3)).to_integer_32] * l_delta___0[(a_N - {NATURAL_32} 3).to_integer_32]) / l_alpha[(a_N - {NATURAL_32} 2).to_integer_32])
							l_alpha[(a_N - {NATURAL_32} 1).to_integer_32] := ((a_diag[(a_d_stride * (a_N - {NATURAL_32} 1)).to_integer_32] - l_sum) - ((l_alpha[(a_N - {NATURAL_32} 2).to_integer_32] * l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32]) * l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32]))
							l_z[0] := a_b___0[0]
							l_i := (1).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 1)
							loop
								l_z[l_i.to_integer_32] := (a_b___0[(a_b_stride * l_i).to_integer_32] - (l_z[(l_i - {NATURAL_32} 1).to_integer_32] * l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32]))
								l_i := l_i + 1
							end
							l_sum := 0.0
							l_i := (0).to_natural_32
							from
							until
								not (l_i < a_N - {NATURAL_32} 2)
							loop
								l_sum := l_sum + (l_delta___0[l_i.to_integer_32] * l_z[l_i.to_integer_32])
								l_i := l_i + 1
							end
							l_z[(a_N - {NATURAL_32} 1).to_integer_32] := ((a_b___0[(a_b_stride * (a_N - {NATURAL_32} 1)).to_integer_32] - l_sum) - (l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32] * l_z[(a_N - {NATURAL_32} 2).to_integer_32]))
							l_i := (0).to_natural_32
							from
							until
								not (l_i < a_N)
							loop
								l_c___1[l_i.to_integer_32] := l_z[l_i.to_integer_32] / l_alpha[l_i.to_integer_32]
								l_i := l_i + 1
							end
							a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_c___1[(a_N - {NATURAL_32} 1).to_integer_32]
							a_x[(a_x_stride * (a_N - {NATURAL_32} 2)).to_integer_32] := l_c___1[(a_N - {NATURAL_32} 2).to_integer_32] - l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32] * a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32]
							if a_N >= {NATURAL_32} 3 then
								l_i := a_N - {NATURAL_32} 3
								l_j := (0).to_natural_32
								from
								until
									not (l_j <= a_N - {NATURAL_32} 3)
								loop
									a_x[(a_x_stride * l_i).to_integer_32] := (l_c___1[l_i.to_integer_32] - l_gamma___0[l_i.to_integer_32] * a_x[(a_x_stride * (l_i + {NATURAL_32} 1)).to_integer_32]) - l_delta___0[l_i.to_integer_32] * a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32]
									l_j := l_j + 1
									l_i := l_i - 1
								end
							end
						end
				when 16 then -- if done | if done | if done | if done
						if not js.return_called then
							if not l_z.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_z))
							end
							if not l_c___1.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_c___1))
							end
							if not l_alpha.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_alpha))
							end
							if not l_gamma___0.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_gamma___0))
							end
							if not l_delta___0.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_delta___0))
							end
							if l_status = 12 then
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 310, l_status)
									Result := l_status
									js.return
									if not js.return_called then
										js.break
									end
								end
							end
						end
						if not js.return_called then
							Result := l_status
						end
				when 17 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	solve_cyc_tridiag_nonsym (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_abovediag: CE_POINTER [REAL_64]; a_a_stride: NATURAL_32; a_belowdiag: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_rhs: CE_POINTER [REAL_64]; a_r_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride: NATURAL_32; a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_alpha: CE_POINTER [REAL_64]
			l_zb: CE_POINTER [REAL_64]
			l_zu: CE_POINTER [REAL_64]
			l_w: CE_POINTER [REAL_64]
			l_beta: REAL_64
			l_q___3: REAL_64
			l_tmp___4: REAL_64
			l_i: NATURAL_32
			l_t: REAL_64
			l_i___0: NATURAL_32
			l_t___0: REAL_64
			l_i___1: NATURAL_32
			l_j: NATURAL_32
			l_vw: REAL_64
			l_vx: REAL_64
			l_i___2: NATURAL_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_alpha.make
			create l_zb.make
			create l_zu.make
			create l_w.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_alpha := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_zb := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_zu := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_w := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_alpha.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if not (l_zb.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (l_zu.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if not (l_w.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 343, 8)
							Result := 8
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_zb[0] := a_rhs[0]
						if a_diag[0] /= (0).to_double then
							l_beta := (-a_diag[0])
						else
							l_beta := (1).to_double
						end
						l_q___3 := ((1).to_double - (a_abovediag[0] * a_belowdiag[0]) / (a_diag[0] * a_diag[a_d_stride.to_integer_32]))
						if (fabs (l_q___3 / l_beta)) > 0.5 then
							if (fabs (l_q___3 / l_beta)) < (2).to_double then
								if (fabs (l_q___3 / l_beta)) < (1).to_double then
									l_tmp___4 := 0.5
								else
									l_tmp___4 := (2).to_double
								end
								l_beta := l_beta * l_tmp___4
							end
						end
						l_zu[0] := l_beta
						l_alpha[0] := (a_diag[0] - l_beta)
						if l_alpha[0] = (0).to_double then
							l_status := 12
						end
						l_i := (1).to_natural_32
						from
						until
							not (l_i + {NATURAL_32} 1 < a_N)
						loop
							l_t := (a_belowdiag[(a_b_stride * (l_i - {NATURAL_32} 1)).to_integer_32] / l_alpha[(l_i - {NATURAL_32} 1).to_integer_32])
							l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - (l_t * a_abovediag[(a_a_stride * (l_i - {NATURAL_32} 1)).to_integer_32]))
							l_zb[l_i.to_integer_32] := (a_rhs[(a_r_stride * l_i).to_integer_32] - (l_t * l_zb[(l_i - {NATURAL_32} 1).to_integer_32]))
							l_zu[l_i.to_integer_32] := -l_t * l_zu[(l_i - {NATURAL_32} 1).to_integer_32]
							if l_alpha[l_i.to_integer_32] = (0).to_double then
								l_status := 12
							end
							l_i := l_i + 1
						end
						l_i___0 := a_N - {NATURAL_32} 1
						l_t___0 := (a_belowdiag[(a_b_stride * (l_i___0 - {NATURAL_32} 1)).to_integer_32] / l_alpha[(l_i___0 - {NATURAL_32} 1).to_integer_32])
						l_alpha[l_i___0.to_integer_32] := ((a_diag[(a_d_stride * l_i___0).to_integer_32] - (a_abovediag[(a_a_stride * l_i___0).to_integer_32] * a_belowdiag[(a_b_stride * l_i___0).to_integer_32]) / l_beta) - (l_t___0 * a_abovediag[(a_a_stride * (l_i___0 - {NATURAL_32} 1)).to_integer_32]))
						l_zb[l_i___0.to_integer_32] := (a_rhs[(a_r_stride * l_i___0).to_integer_32] - (l_t___0 * l_zb[(l_i___0 - {NATURAL_32} 1).to_integer_32]))
						l_zu[l_i___0.to_integer_32] := (a_abovediag[(a_a_stride * l_i___0).to_integer_32] - (l_t___0 * l_zu[(l_i___0 - {NATURAL_32} 1).to_integer_32]))
						if l_alpha[l_i___0.to_integer_32] = (0).to_double then
							l_status := 12
						end
						l_w[(a_N - {NATURAL_32} 1).to_integer_32] := l_zu[(a_N - {NATURAL_32} 1).to_integer_32] / l_alpha[(a_N - {NATURAL_32} 1).to_integer_32]
						a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_zb[(a_N - {NATURAL_32} 1).to_integer_32] / l_alpha[(a_N - {NATURAL_32} 1).to_integer_32]
						l_i___1 := a_N - {NATURAL_32} 2
						l_j := (0).to_natural_32
						from
						until
							not (l_j <= a_N - {NATURAL_32} 2)
						loop
							l_w[l_i___1.to_integer_32] := (l_zu[l_i___1.to_integer_32] - (a_abovediag[(a_a_stride * l_i___1).to_integer_32] * l_w[(l_i___1 + {NATURAL_32} 1).to_integer_32])) / l_alpha[l_i___1.to_integer_32]
							a_x[(l_i___1 * a_x_stride).to_integer_32] := (l_zb[l_i___1.to_integer_32] - (a_abovediag[(a_a_stride * l_i___1).to_integer_32] * a_x[(a_x_stride * (l_i___1 + {NATURAL_32} 1)).to_integer_32])) / l_alpha[l_i___1.to_integer_32]
							l_j := l_j + 1
							l_i___1 := l_i___1 - 1
						end
						l_vw := l_w[0] + ((a_belowdiag[(a_b_stride * (a_N - {NATURAL_32} 1)).to_integer_32] / l_beta) * l_w[(a_N - {NATURAL_32} 1).to_integer_32])
						l_vx := a_x[0] + ((a_belowdiag[(a_b_stride * (a_N - {NATURAL_32} 1)).to_integer_32] / l_beta) * a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32])
						if l_vw + (1).to_double = (0).to_double then
							l_status := 12
						end
						l_i___2 := (0).to_natural_32
						from
						until
							not (l_i___2 < a_N)
						loop
							a_x[(l_i___2 * a_x_stride).to_integer_32] := a_x[(l_i___2 * a_x_stride).to_integer_32] - ((l_vx / ((1).to_double + l_vw)) * l_w[l_i___2.to_integer_32])
							l_i___2 := l_i___2 + 1
						end
				when 13 then -- if done | if done | if done
						if not js.return_called then
							if not l_zb.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_zb))
							end
							if not l_zu.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_zu))
							end
							if not l_w.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_w))
							end
							if not l_alpha.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_alpha))
							end
							if l_status = 12 then
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 438, l_status)
									Result := l_status
									js.return
									if not js.return_called then
										js.break
									end
								end
							end
						end
						if not js.return_called then
							Result := l_status
						end
				when 14 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	solve_tridiag (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_offdiag: CE_POINTER [REAL_64]; a_o_stride: NATURAL_32; a_b___0: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride: NATURAL_32; a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_gamma___0: CE_POINTER [REAL_64]
			l_alpha: CE_POINTER [REAL_64]
			l_c___1: CE_POINTER [REAL_64]
			l_z: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_gamma___0.make
			create l_alpha.make
			create l_c___1.make
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_gamma___0 := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_alpha := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_c___1 := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_z := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_gamma___0.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 3 then -- else:
						if not (l_alpha.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 6 then -- else:
						if not (l_c___1.is_void) then
							l_pos := -1 + 9 -- skip if block
						end
				when 7 then -- if:
						l_pos := -1 + 10 -- goto _L
				when 8 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 9 then -- else:
						if not (l_z.is_void) then
							l_pos := -1 + 12 -- skip if block
						end
				when 10 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 53, 8)
							Result := 8
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 11 then -- if block done, skip else block:
						l_pos := -1 + 13
				when 12 then -- else:
						l_alpha[0] := a_diag[0]
						l_gamma___0[0] := (a_offdiag[0] / l_alpha[0])
						if l_alpha[0] = (0).to_double then
							l_status := 12
						end
						l_i := (1).to_natural_32
						from
						until
							not (l_i < a_N - {NATURAL_32} 1)
						loop
							l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - a_offdiag[(a_o_stride * (l_i - {NATURAL_32} 1)).to_integer_32] * l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32])
							l_gamma___0[l_i.to_integer_32] := (a_offdiag[(a_o_stride * l_i).to_integer_32] / l_alpha[l_i.to_integer_32])
							if l_alpha[l_i.to_integer_32] = (0).to_double then
								l_status := 12
							end
							l_i := l_i + 1
						end
						if a_N > {NATURAL_32} 1 then
							l_alpha[(a_N - {NATURAL_32} 1).to_integer_32] := (a_diag[(a_d_stride * (a_N - {NATURAL_32} 1)).to_integer_32] - a_offdiag[(a_o_stride * (a_N - {NATURAL_32} 2)).to_integer_32] * l_gamma___0[(a_N - {NATURAL_32} 2).to_integer_32])
						end
						l_z[0] := a_b___0[0]
						l_i := (1).to_natural_32
						from
						until
							not (l_i < a_N)
						loop
							l_z[l_i.to_integer_32] := (a_b___0[(a_b_stride * l_i).to_integer_32] - (l_gamma___0[(l_i - {NATURAL_32} 1).to_integer_32] * l_z[(l_i - {NATURAL_32} 1).to_integer_32]))
							l_i := l_i + 1
						end
						l_i := (0).to_natural_32
						from
						until
							not (l_i < a_N)
						loop
							l_c___1[l_i.to_integer_32] := l_z[l_i.to_integer_32] / l_alpha[l_i.to_integer_32]
							l_i := l_i + 1
						end
						a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_c___1[(a_N - {NATURAL_32} 1).to_integer_32]
						if a_N >= {NATURAL_32} 2 then
							l_i := a_N - {NATURAL_32} 2
							l_j := (0).to_natural_32
							from
							until
								not (l_j <= a_N - {NATURAL_32} 2)
							loop
								a_x[(a_x_stride * l_i).to_integer_32] := l_c___1[l_i.to_integer_32] - l_gamma___0[l_i.to_integer_32] * a_x[(a_x_stride * (l_i + {NATURAL_32} 1)).to_integer_32]
								l_j := l_j + 1
								l_i := l_i - 1
							end
						end
				when 13 then -- if done | if done | if done
						if not js.return_called then
							if not l_z.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_z))
							end
							if not l_c___1.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_c___1))
							end
							if not l_alpha.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_alpha))
							end
							if not l_gamma___0.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_gamma___0))
							end
							if l_status = 12 then
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 117, l_status)
									Result := l_status
									js.return
									if not js.return_called then
										js.break
									end
								end
							end
						end
						if not js.return_called then
							Result := l_status
						end
				when 14 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

	solve_tridiag_nonsym (a_diag: CE_POINTER [REAL_64]; a_d_stride: NATURAL_32; a_abovediag: CE_POINTER [REAL_64]; a_a_stride: NATURAL_32; a_belowdiag: CE_POINTER [REAL_64]; a_b_stride: NATURAL_32; a_rhs: CE_POINTER [REAL_64]; a_r_stride: NATURAL_32; a_x: CE_POINTER [REAL_64]; a_x_stride: NATURAL_32; a_N: NATURAL_32): INTEGER_32
		local
			l_status: INTEGER_32
			l_alpha: CE_POINTER [REAL_64]
			l_z: CE_POINTER [REAL_64]
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_t: REAL_64
			js: JUMP_STATE
			l_pos: INTEGER_32
		do
			create l_alpha.make
			create l_z.make

			from until l_pos = -1 loop -- -1: return
				inspect l_pos

				-- <function body>
				when 0 then -- execution start:
					l_status := 0
					l_alpha := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					l_z := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_N * {PLATFORM}.Real_64_bytes.to_natural_32)))
					if not (l_alpha.is_void) then
						l_pos := -1 + 3 -- skip if block
					end
				when 1 then -- if:
						l_pos := -1 + 4 -- goto _L
				when 2 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 3 then -- else:
						if not (l_z.is_void) then
							l_pos := -1 + 6 -- skip if block
						end
				when 4 then -- if: | _L:
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("failed to allocate working space"), ce_string ("tridiag.c"), 146, 8)
							Result := 8
							js.return
							if not js.return_called then
								js.break
							end
						end
				when 5 then -- if block done, skip else block:
						l_pos := -1 + 7
				when 6 then -- else:
						l_alpha[0] := a_diag[0]
						l_z[0] := a_rhs[0]
						if l_alpha[0] = (0).to_double then
							l_status := 12
						end
						l_i := (1).to_natural_32
						from
						until
							not (l_i < a_N)
						loop
							l_t := (a_belowdiag[(a_b_stride * (l_i - {NATURAL_32} 1)).to_integer_32] / l_alpha[(l_i - {NATURAL_32} 1).to_integer_32])
							l_alpha[l_i.to_integer_32] := (a_diag[(a_d_stride * l_i).to_integer_32] - (l_t * a_abovediag[(a_a_stride * (l_i - {NATURAL_32} 1)).to_integer_32]))
							l_z[l_i.to_integer_32] := (a_rhs[(a_r_stride * l_i).to_integer_32] - (l_t * l_z[(l_i - {NATURAL_32} 1).to_integer_32]))
							if l_alpha[l_i.to_integer_32] = (0).to_double then
								l_status := 12
							end
							l_i := l_i + 1
						end
						a_x[(a_x_stride * (a_N - {NATURAL_32} 1)).to_integer_32] := l_z[(a_N - {NATURAL_32} 1).to_integer_32] / l_alpha[(a_N - {NATURAL_32} 1).to_integer_32]
						if a_N >= {NATURAL_32} 2 then
							l_i := a_N - {NATURAL_32} 2
							l_j := (0).to_natural_32
							from
							until
								not (l_j <= a_N - {NATURAL_32} 2)
							loop
								a_x[(a_x_stride * l_i).to_integer_32] := (l_z[l_i.to_integer_32] - (a_abovediag[(a_a_stride * l_i).to_integer_32] * a_x[(a_x_stride * (l_i + {NATURAL_32} 1)).to_integer_32])) / l_alpha[l_i.to_integer_32]
								l_j := l_j + 1
								l_i := l_i - 1
							end
						end
				when 7 then -- if done
						if not js.return_called then
							if not l_z.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_z))
							end
							if not l_alpha.is_void then
								free (create {CE_POINTER [ANY]}.make_cast (l_alpha))
							end
							if l_status = 12 then
								from
								until
									js.loop_done
								loop
									error_inst.gsl_error (ce_string ("matrix must be positive definite"), ce_string ("tridiag.c"), 191, l_status)
									Result := l_status
									js.return
									if not js.return_called then
										js.break
									end
								end
							end
						end
						if not js.return_called then
							Result := l_status
						end
				when 8 then
						l_pos := -1 + -1 -- end of function
				end
				-- </function body>

				l_pos := l_pos + 1
			end
			end

end
