class
	P_VEGAS

inherit
	LIBGSL_DATA
	S_GCC_BUILTIN
	S_LIBIO
	S_STDIO
	S_STDLIB
	S_MATH
	S_STRING
	S_ASSERT

feature {ANY} -- routines

	accumulate_distribution (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_bin: CE_POINTER [INTEGER_32]; a_y: REAL_64)
		local
			l_j: NATURAL_32
			l_dim: NATURAL_32
			l_i: INTEGER_32
		do
			l_dim := a_s.item.dim
			l_j := (0).to_natural_32
			from
			until
				not (l_j < l_dim)
			loop
				l_i := a_bin[l_j.to_integer_32]
				a_s.item.d[(l_i.to_natural_32 * a_s.item.dim + l_j).to_integer_32] := a_s.item.d[(l_i.to_natural_32 * a_s.item.dim + l_j).to_integer_32] + a_y
				l_j := l_j + 1
			end
		end

	change_box_coord (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_box: CE_POINTER [INTEGER_32]): INTEGER_32
		local
			l_j: INTEGER_32
			l_ng: INTEGER_32
			js: JUMP_STATE
		do
			l_j := (a_s.item.dim - {NATURAL_32} 1).to_integer_32
			l_ng := (a_s.item.boxes).to_integer_32
			from
			until
				not (not js.loop_done and then (l_j >= 0))
			loop
				a_box[l_j] := (a_box[l_j] + 1) \\ l_ng
				if a_box[l_j] /= 0 then
					Result := 1
					js.return
				end
				if not js.return_called then
					l_j := l_j - 1
				end
			end
			if not js.return_called then
				Result := 0
			end
		end

	gsl_monte_vegas_alloc (a_dim: NATURAL_32): CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]
		local
			l_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]
			js: JUMP_STATE
		do
			create l_s.make

			l_s := create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_cast ((malloc ((create {PD_GSL_MONTE_VEGAS_STATE}.make).structure_size.to_natural_32)))
			if l_s.is_void then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("failed to allocate space for vegas state struct"), ce_string ("vegas.c"), 390, 8)
					Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_s.item.delx := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_dim * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.delx).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for delx"), ce_string ("vegas.c"), 398, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.d := create {CE_POINTER [REAL_64]}.make_cast ((malloc (({NATURAL_32} 50 * a_dim) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.d).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for d"), ce_string ("vegas.c"), 407, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.xi := create {CE_POINTER [REAL_64]}.make_cast ((malloc (({NATURAL_32} 51 * a_dim) * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.xi).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for xi"), ce_string ("vegas.c"), 417, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.xin := create {CE_POINTER [REAL_64]}.make_cast ((malloc ({NATURAL_32} 51 * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.xin).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xi))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for xin"), ce_string ("vegas.c"), 428, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.weight := create {CE_POINTER [REAL_64]}.make_cast ((malloc ({NATURAL_32} 50 * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.weight).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xin))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xi))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for xin"), ce_string ("vegas.c"), 440, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.box := create {CE_POINTER [INTEGER_32]}.make_cast ((malloc (a_dim * {PLATFORM}.Integer_32_bytes.to_natural_32)))
				if (l_s.item.box).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.weight))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xin))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xi))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for box"), ce_string ("vegas.c"), 453, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.bin := create {CE_POINTER [INTEGER_32]}.make_cast ((malloc (a_dim * {PLATFORM}.Integer_32_bytes.to_natural_32)))
				if (l_s.item.bin).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.box))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.weight))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xin))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xi))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for bin"), ce_string ("vegas.c"), 467, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.x := create {CE_POINTER [REAL_64]}.make_cast ((malloc (a_dim * {PLATFORM}.Real_64_bytes.to_natural_32)))
				if (l_s.item.x).is_void then
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.bin))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.box))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.weight))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xin))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.xi))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.d))
					free (create {CE_POINTER [ANY]}.make_cast (l_s.item.delx))
					free (create {CE_POINTER [ANY]}.make_cast (l_s))
					from
					until
						js.loop_done
					loop
						error_inst.gsl_error (ce_string ("failed to allocate space for x"), ce_string ("vegas.c"), 482, 8)
						Result := (create {CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]}.make_shared (default_pointer))
						js.return
						if not js.return_called then
							js.break
						end
					end
				end
			end
			if not js.return_called then
				l_s.item.dim := a_dim
				l_s.item.bins_max := (50).to_natural_32
				gsl_monte_vegas_init (l_s).do_nothing
				Result := l_s
			end
		end

	gsl_monte_vegas_chisq (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]): REAL_64
		do
			Result := (a_s.item.chisq)
		end

	gsl_monte_vegas_free (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE])
		local
			js: JUMP_STATE
		do
			if a_s.is_void then
				js.return
			end
			if not js.return_called then
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.x))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.delx))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.d))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.xi))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.xin))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.weight))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.box))
				free (create {CE_POINTER [ANY]}.make_cast (a_s.item.bin))
				free (create {CE_POINTER [ANY]}.make_cast (a_s))
			end
		end

	gsl_monte_vegas_init (a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]): INTEGER_32
		do
			a_state.item.stage := 0
			a_state.item.alpha := 1.5
			a_state.item.verbose := -1
			a_state.item.iterations := {NATURAL_32} 5
			a_state.item.mode := 1
			a_state.item.chisq := (0).to_double
			a_state.item.bins := a_state.item.bins_max
			a_state.item.ostream := (create {CE_POINTER [SD_IO_FILE]}.make_shared (Io.output.file_pointer))
			Result := 0
		end

	gsl_monte_vegas_integrate (a_f: CE_POINTER [PD_GSL_MONTE_FUNCTION_STRUCT]; a_xl: CE_POINTER [REAL_64]; a_xu: CE_POINTER [REAL_64]; a_dim: NATURAL_32; a_calls: NATURAL_32; a_r___3: CE_POINTER [PD_GSL_RNG]; a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_result: CE_POINTER [REAL_64]; a_abserr: CE_POINTER [REAL_64]): INTEGER_32
		local
			l_calls: NATURAL_32
			l_cum_int: REAL_64
			l_cum_sig: REAL_64
			l_i: NATURAL_32
			l_k: NATURAL_32
			l_it: NATURAL_32
			l_bins: NATURAL_32
			l_boxes: NATURAL_32
			l_box_per_bin: INTEGER_32
			l_tmp___1: NATURAL_32
			l_tot_boxes: REAL_64
			l_intgrl: REAL_64
			l_intgrl_sq: REAL_64
			l_tss: REAL_64
			l_wgt: REAL_64
			l_var: REAL_64
			l_sig: REAL_64
			l_calls_per_box: NATURAL_32
			l_jacbin: REAL_64
			l_x: CE_POINTER [REAL_64]
			l_bin: CE_POINTER [INTEGER_32]
			l_m___4: REAL_64
			l_q___3: REAL_64
			l_f_sq_sum: REAL_64
			l_fval: REAL_64
			l_bin_vol: CE_ARRAY [REAL_64]
			l_d: REAL_64
			l_f_sq: REAL_64
			l_sum_wgts: REAL_64
			l_wtd_int_sum: REAL_64
			l_m___5: REAL_64
			l_tmp___6: REAL_64
			l_q___4: REAL_64
			js: JUMP_STATE
		do
			create l_x.make
			create l_bin.make
			create l_bin_vol.make (<<1>>)

			l_calls := a_calls
			if a_dim /= a_state.item.dim then
				from
				until
					js.loop_done
				loop
					error_inst.gsl_error (ce_string ("number of dimensions must match allocated size"), ce_string ("vegas.c"), 121, 4)
					Result := 4
					js.return
					if not js.return_called then
						js.break
					end
				end
			end
			if not js.return_called then
				l_i := (0).to_natural_32
				from
				until
					not (not js.loop_done and then (l_i < a_dim))
				loop
					if a_xu[l_i.to_integer_32] <= a_xl[l_i.to_integer_32] then
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("xu must be greater than xl"), ce_string ("vegas.c"), 128, 4)
							Result := 4
							js.return
							if not js.return_called then
								js.break
							end
						end
					end
					if not js.return_called and then (a_xu[l_i.to_integer_32] - a_xl[l_i.to_integer_32] > 1.7976931348623157e+308) then
						from
						until
							js.loop_done
						loop
							error_inst.gsl_error (ce_string ("Range of integration is too large, please rescale"), ce_string ("vegas.c"), 134, 4)
							Result := 4
							js.return
							if not js.return_called then
								js.break
							end
						end
					end
					if not js.return_called then
						l_i := l_i + 1
					end
				end
			end
			if not js.return_called then
				if a_state.item.stage = 0 then
					init_grid (a_state, a_xl, a_xu, a_dim)
					if a_state.item.verbose >= 0 then
						print_lim (a_state, a_xl, a_xu, a_dim.to_natural_32)
					end
				end
				if a_state.item.stage <= 1 then
					a_state.item.wtd_int_sum := (0).to_double
					a_state.item.sum_wgts := (0).to_double
					a_state.item.chi_sum := (0).to_double
					a_state.item.it_num := {NATURAL_32} 1
					a_state.item.samples := {NATURAL_32} 0
					a_state.item.chisq := (0).to_double
				end
				if a_state.item.stage <= 2 then
					l_bins := a_state.item.bins_max
					l_boxes := {NATURAL_32} 1
					if a_state.item.mode /= 0 then
						l_boxes := (floor ((pow (l_calls.to_real_64 / 2.0, 1.0 / a_dim.to_real_64)))).truncated_to_integer_64.to_natural_32
						a_state.item.mode := 1
						if {NATURAL_32} 2 * l_boxes >= a_state.item.bins_max then
							if l_boxes // a_state.item.bins_max > {NATURAL_32} 1 then
								l_tmp___1 := l_boxes // a_state.item.bins_max
							else
								l_tmp___1 := {NATURAL_32} 1
							end
							l_box_per_bin := l_tmp___1.to_integer_32
							if l_boxes // l_box_per_bin.to_natural_32 < a_state.item.bins_max then
								l_bins := l_boxes // l_box_per_bin.to_natural_32
							else
								l_bins := a_state.item.bins_max
							end
							l_boxes := l_box_per_bin.to_natural_32 * l_bins
							a_state.item.mode := -1
						end
					end
					l_tot_boxes := (pow_int_inst.gsl_pow_int (l_boxes.to_real_64, a_dim.to_integer_32))
					if l_calls.to_real_64 / l_tot_boxes > (2).to_double then
						a_state.item.calls_per_box := (l_calls.to_real_64 / l_tot_boxes).truncated_to_integer_64.to_natural_32
					else
						a_state.item.calls_per_box := {NATURAL_32} 2
					end
					l_calls := ((a_state.item.calls_per_box).to_real_64 * l_tot_boxes).truncated_to_integer_64.to_natural_32
					a_state.item.jac := (a_state.item.vol * (pow (l_bins.to_real_64, a_dim.to_real_64))) / l_calls.to_real_64
					a_state.item.boxes := l_boxes
					if l_bins /= a_state.item.bins then
						resize_grid (a_state, l_bins)
						if a_state.item.verbose > 1 then
							print_grid (a_state, a_dim.to_natural_32)
						end
					end
					if a_state.item.verbose >= 0 then
						print_head (a_state, a_dim.to_natural_32, l_calls.to_natural_32, a_state.item.it_num, a_state.item.bins, a_state.item.boxes)
					end
				end
				a_state.item.it_start := a_state.item.it_num
				l_cum_int := 0.0
				l_cum_sig := 0.0
				l_it := (0).to_natural_32
				from
				until
					not (not js.loop_done and then (l_it < a_state.item.iterations))
				loop
					l_intgrl := 0.0
					l_intgrl_sq := 0.0
					l_tss := 0.0
					l_calls_per_box := a_state.item.calls_per_box
					l_jacbin := a_state.item.jac
					l_x := a_state.item.x
					l_bin := a_state.item.bin
					a_state.item.it_num := a_state.item.it_start + l_it
					reset_grid_values (a_state)
					init_box_coord (a_state, a_state.item.box)
					from
					until
						js.loop_done
					loop
						l_m___4 := (0).to_double
						l_q___3 := (0).to_double
						l_f_sq_sum := 0.0
						l_k := (0).to_natural_32
						from
						until
							not (l_k < l_calls_per_box)
						loop
							random_point (l_x, l_bin, l_bin_vol, a_state.item.box, a_xl, a_xu, a_state, a_r___3)
							l_fval := ((l_jacbin * l_bin_vol.item) * ((a_f.item.f).real_64_item ([l_x, a_f.item.dim, a_f.item.params])))
							l_d := (l_fval - l_m___4)
							l_m___4 := l_m___4 + (l_d / (l_k.to_real_64 + 1.0))
							l_q___3 := l_q___3 + ((l_d * l_d) * (l_k.to_real_64 / (l_k.to_real_64 + 1.0)))
							if a_state.item.mode /= -1 then
								l_f_sq := (l_fval * l_fval)
								accumulate_distribution (a_state, l_bin, l_f_sq)
							end
							l_k := l_k + 1
						end
						l_intgrl := l_intgrl + (l_m___4 * l_calls_per_box.to_real_64)
						l_f_sq_sum := (l_q___3 * l_calls_per_box.to_real_64)
						l_tss := l_tss + l_f_sq_sum
						if a_state.item.mode = -1 then
							accumulate_distribution (a_state, l_bin, l_f_sq_sum)
						end
						if not (change_box_coord (a_state, a_state.item.box)).to_boolean then
							js.break
						end
					end
					l_var := l_tss / (l_calls_per_box.to_real_64 - 1.0)
					if l_var > (0).to_double then
						l_wgt := 1.0 / l_var
					elseif a_state.item.sum_wgts > (0).to_double then
						l_wgt := a_state.item.sum_wgts / (a_state.item.samples).to_real_64
					else
						l_wgt := 0.0
					end
					l_intgrl_sq := l_intgrl * l_intgrl
					l_sig := sqrt (l_var)
					a_state.item.result2 := l_intgrl
					a_state.item.sigma := l_sig
					if l_wgt > 0.0 then
						l_sum_wgts := a_state.item.sum_wgts
						l_wtd_int_sum := a_state.item.wtd_int_sum
						if l_sum_wgts > (0).to_double then
							l_tmp___6 := l_wtd_int_sum / l_sum_wgts
						else
							l_tmp___6 := (0).to_double
						end
						l_m___5 := l_tmp___6
						l_q___4 := l_intgrl - l_m___5
						a_state.item.samples := (a_state.item.samples) + 1
						a_state.item.sum_wgts := a_state.item.sum_wgts + l_wgt
						a_state.item.wtd_int_sum := a_state.item.wtd_int_sum + (l_intgrl * l_wgt)
						a_state.item.chi_sum := a_state.item.chi_sum + (l_intgrl_sq * l_wgt)
						l_cum_int := a_state.item.wtd_int_sum / a_state.item.sum_wgts
						l_cum_sig := sqrt ((1).to_double / a_state.item.sum_wgts)
						if a_state.item.samples = {NATURAL_32} 1 then
							a_state.item.chisq := (0).to_double
						else
							a_state.item.chisq := a_state.item.chisq * ((a_state.item.samples).to_real_64 - 2.0)
							a_state.item.chisq := a_state.item.chisq + (((l_wgt / ((1).to_double + l_wgt / l_sum_wgts)) * l_q___4) * l_q___4)
							a_state.item.chisq := a_state.item.chisq / ((a_state.item.samples).to_real_64 - 1.0)
						end
					else
						l_cum_int := l_cum_int + ((l_intgrl - l_cum_int) / (l_it.to_real_64 + 1.0))
						l_cum_sig := 0.0
					end
					if a_state.item.verbose >= 0 then
						print_res (a_state, a_state.item.it_num, l_intgrl, l_sig, l_cum_int, l_cum_sig, a_state.item.chisq)
						if l_it + {NATURAL_32} 1 = a_state.item.iterations then
							if a_state.item.verbose > 0 then
								print_grid (a_state, a_dim.to_natural_32)
							end
						end
					end
					if a_state.item.verbose > 1 then
						print_dist (a_state, a_dim.to_natural_32)
					end
					refine_grid (a_state)
					if a_state.item.verbose > 1 then
						print_grid (a_state, a_dim.to_natural_32)
					end
					l_it := l_it + 1
				end
				a_state.item.stage := 1
				a_result.item := l_cum_int
				a_abserr.item := l_cum_sig
				Result := 0
			end
		end

	gsl_monte_vegas_params_get (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_p: CE_POINTER [PD_GSL_MONTE_VEGAS_PARAMS])
		do
			a_p.item.alpha := a_s.item.alpha
			a_p.item.iterations := a_s.item.iterations
			a_p.item.stage := a_s.item.stage
			a_p.item.mode := a_s.item.mode
			a_p.item.verbose := a_s.item.verbose
			a_p.item.ostream := a_s.item.ostream
		end

	gsl_monte_vegas_params_set (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_p: CE_POINTER [PD_GSL_MONTE_VEGAS_PARAMS])
		do
			a_s.item.alpha := a_p.item.alpha
			a_s.item.iterations := a_p.item.iterations
			a_s.item.stage := a_p.item.stage
			a_s.item.mode := a_p.item.mode
			a_s.item.verbose := a_p.item.verbose
			a_s.item.ostream := a_p.item.ostream
		end

	gsl_monte_vegas_runval (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_result: CE_POINTER [REAL_64]; a_sigma: CE_POINTER [REAL_64])
		do
			a_result.item := a_s.item.result2
			a_sigma.item := a_s.item.sigma
		end

	init_box_coord (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_box: CE_POINTER [INTEGER_32])
		local
			l_i: NATURAL_32
			l_dim: NATURAL_32
		do
			l_dim := a_s.item.dim
			l_i := (0).to_natural_32
			from
			until
				not (l_i < l_dim)
			loop
				a_box[l_i.to_integer_32] := 0
				l_i := l_i + 1
			end
		end

	init_grid (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_xl: CE_POINTER [REAL_64]; a_xu: CE_POINTER [REAL_64]; a_dim: NATURAL_32)
		local
			l_j: NATURAL_32
			l_vol: REAL_64
			l_dx: REAL_64
		do
			l_vol := 1.0
			a_s.item.bins := {NATURAL_32} 1
			l_j := (0).to_natural_32
			from
			until
				not (l_j < a_dim)
			loop
				l_dx := a_xu[l_j.to_integer_32] - a_xl[l_j.to_integer_32]
				a_s.item.delx[l_j.to_integer_32] := l_dx
				l_vol := l_vol * l_dx
				a_s.item.xi[l_j.to_integer_32] := 0.0
				a_s.item.xi[(a_s.item.dim + l_j).to_integer_32] := 1.0
				l_j := l_j + 1
			end
			a_s.item.vol := l_vol
		end

	print_dist (a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_dim: NATURAL_32)
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_p: INTEGER_32
			js: JUMP_STATE
		do
			l_p := a_state.item.verbose
			if l_p < 1 then
				js.return
			end
			if not js.return_called then
				l_j := {NATURAL_32} 0
				from
				until
					not (not js.loop_done and then (l_j < a_dim))
				loop
					fprintf ([a_state.item.ostream, ce_string ("%N axis %%lu %N"), l_j]).do_nothing
					fprintf ([a_state.item.ostream, ce_string ("      x   g%N")]).do_nothing
					l_i := {NATURAL_32} 0
					from
					until
						not (l_i < (a_state.item.bins).to_natural_32)
					loop
						fprintf ([a_state.item.ostream, ce_string ("weight [%%11.2e , %%11.2e] = "), a_state.item.xi[(l_i * (a_state.item.dim).to_natural_32 + l_j).to_integer_32], a_state.item.xi[((l_i + {NATURAL_32} 1) * (a_state.item.dim).to_natural_32 + l_j).to_integer_32]]).do_nothing
						fprintf ([a_state.item.ostream, ce_string (" %%11.2e%N"), a_state.item.d[(l_i * (a_state.item.dim).to_natural_32 + l_j).to_integer_32]]).do_nothing
						l_i := l_i + 1
					end
					fprintf ([a_state.item.ostream, ce_string ("%N")]).do_nothing
					l_j := l_j + 1
				end
				fprintf ([a_state.item.ostream, ce_string ("%N")]).do_nothing
				fflush (a_state.item.ostream).do_nothing
			end
		end

	print_grid (a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_dim: NATURAL_32)
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_p: INTEGER_32
			js: JUMP_STATE
		do
			l_p := a_state.item.verbose
			if l_p < 1 then
				js.return
			end
			if not js.return_called then
				l_j := {NATURAL_32} 0
				from
				until
					not (not js.loop_done and then (l_j < a_dim))
				loop
					fprintf ([a_state.item.ostream, ce_string ("%N axis %%lu %N"), l_j]).do_nothing
					fprintf ([a_state.item.ostream, ce_string ("      x   %N")]).do_nothing
					l_i := {NATURAL_32} 0
					from
					until
						not (l_i <= (a_state.item.bins).to_natural_32)
					loop
						fprintf ([a_state.item.ostream, ce_string ("%%11.2e"), a_state.item.xi[(l_i * (a_state.item.dim).to_natural_32 + l_j).to_integer_32]]).do_nothing
						if l_i \\ {NATURAL_32} 5 = {NATURAL_32} 4 then
							fprintf ([a_state.item.ostream, ce_string ("%N")]).do_nothing
						end
						l_i := l_i + 1
					end
					fprintf ([a_state.item.ostream, ce_string ("%N")]).do_nothing
					l_j := l_j + 1
				end
				fprintf ([a_state.item.ostream, ce_string ("%N")]).do_nothing
				fflush (a_state.item.ostream).do_nothing
			end
		end

	print_head (a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_num_dim: NATURAL_32; a_calls: NATURAL_32; a_it_num: NATURAL_32; a_bins: NATURAL_32; a_boxes: NATURAL_32)
		do
			fprintf ([a_state.item.ostream, ce_string ("%Nnum_dim=%%lu, calls=%%lu, it_num=%%d, max_it_num=%%d "), a_num_dim, a_calls, a_it_num, a_state.item.iterations]).do_nothing
			fprintf ([a_state.item.ostream, ce_string ("verb=%%d, alph=%%.2f,%Nmode=%%d, bins=%%d, boxes=%%d%N"), a_state.item.verbose, a_state.item.alpha, a_state.item.mode, a_bins, a_boxes]).do_nothing
			fprintf ([a_state.item.ostream, ce_string ("%N       single.......iteration                   ")]).do_nothing
			fprintf ([a_state.item.ostream, ce_string ("accumulated......results   %N")]).do_nothing
			fprintf ([a_state.item.ostream, ce_string ("iteration     integral    sigma             integral   ")]).do_nothing
			fprintf ([a_state.item.ostream, ce_string ("      sigma     chi-sq/it%N%N")]).do_nothing
			fflush (a_state.item.ostream).do_nothing
		end

	print_lim (a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_xl: CE_POINTER [REAL_64]; a_xu: CE_POINTER [REAL_64]; a_dim: NATURAL_32)
		local
			l_j: NATURAL_32
		do
			fprintf ([a_state.item.ostream, ce_string ("The limits of integration are:%N")]).do_nothing
			l_j := {NATURAL_32} 0
			from
			until
				not (l_j < a_dim)
			loop
				fprintf ([a_state.item.ostream, ce_string ("%Nxl[%%lu]=%%f    xu[%%lu]=%%f"), l_j, a_xl[l_j.to_integer_32], l_j, a_xu[l_j.to_integer_32]]).do_nothing
				l_j := l_j + 1
			end
			fprintf ([a_state.item.ostream, ce_string ("%N")]).do_nothing
			fflush (a_state.item.ostream).do_nothing
		end

	print_res (a_state: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_itr: NATURAL_32; a_res: REAL_64; a_err: REAL_64; a_cum_res: REAL_64; a_cum_err: REAL_64; a_chi_sq: REAL_64)
		do
			fprintf ([a_state.item.ostream, ce_string ("%%4d        %%6.4e %%10.2e          %%6.4e      %%8.2e  %%10.2e%N"), a_itr, a_res, a_err, a_cum_res, a_cum_err, a_chi_sq]).do_nothing
			fflush (a_state.item.ostream).do_nothing
		end

	random_point (a_x: CE_POINTER [REAL_64]; a_bin: CE_POINTER [INTEGER_32]; a_bin_vol: CE_POINTER [REAL_64]; a_box: CE_POINTER [INTEGER_32]; a_xl: CE_POINTER [REAL_64]; a_xu: CE_POINTER [REAL_64]; a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_r___3: CE_POINTER [PD_GSL_RNG])
		local
			l_vol: REAL_64
			l_j: NATURAL_32
			l_dim: NATURAL_32
			l_bins: NATURAL_32
			l_boxes: NATURAL_32
			l_tmp: INTEGER_32
			l_z: REAL_64
			l_k: INTEGER_32
			l_y: REAL_64
			l_bin_width: REAL_64
			js: JUMP_STATE
		do
			l_vol := 1.0
			l_dim := a_s.item.dim
			l_bins := a_s.item.bins
			l_boxes := a_s.item.boxes
			from
			until
				js.loop_done
			loop
				if (not a_xu.is_void) then
					l_tmp := 0
				else
					l_tmp := 0
				end
				if not l_tmp.to_boolean then
					js.break
				end
			end
			l_j := (0).to_natural_32
			from
			until
				not (l_j < l_dim)
			loop
				l_z := (((a_box[l_j.to_integer_32]).to_double + (gsl_gsl_rng_inst.gsl_rng_uniform_pos (a_r___3))) / l_boxes.to_real_64) * l_bins.to_real_64
				l_k := l_z.truncated_to_integer
				a_bin[l_j.to_integer_32] := l_k
				if l_k = 0 then
					l_bin_width := a_s.item.xi[(a_s.item.dim + l_j).to_integer_32]
					l_y := l_z * l_bin_width
				else
					l_bin_width := a_s.item.xi[((l_k + 1).to_natural_32 * a_s.item.dim + l_j).to_integer_32] - a_s.item.xi[(l_k.to_natural_32 * a_s.item.dim + l_j).to_integer_32]
					l_y := a_s.item.xi[(l_k.to_natural_32 * a_s.item.dim + l_j).to_integer_32] + (l_z - l_k.to_double) * l_bin_width
				end
				a_x[l_j.to_integer_32] := (a_xl[l_j.to_integer_32] + (l_y * a_s.item.delx[l_j.to_integer_32]))
				l_vol := l_vol * l_bin_width
				l_j := l_j + 1
			end
			a_bin_vol.item := l_vol
		end

	refine_grid (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_dim: NATURAL_32
			l_bins: NATURAL_32
			l_grid_tot_j: REAL_64
			l_tot_weight: REAL_64
			l_weight: CE_POINTER [REAL_64]
			l_oldg: REAL_64
			l_newg: REAL_64
			l_rc: REAL_64
			l_pts_per_bin: REAL_64
			l_xold: REAL_64
			l_xnew: REAL_64
			l_dw: REAL_64
			js: JUMP_STATE
		do
			create l_weight.make

			l_dim := a_s.item.dim
			l_bins := a_s.item.bins
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_j < l_dim))
			loop
				l_weight := a_s.item.weight
				l_oldg := a_s.item.d[l_j.to_integer_32]
				l_newg := a_s.item.d[(a_s.item.dim + l_j).to_integer_32]
				a_s.item.d[l_j.to_integer_32] := (l_oldg + l_newg) / (2).to_double
				l_grid_tot_j := a_s.item.d[l_j.to_integer_32]
				l_i := (1).to_natural_32
				from
				until
					not (l_i < l_bins - {NATURAL_32} 1)
				loop
					l_rc := l_oldg + l_newg
					l_oldg := l_newg
					l_newg := a_s.item.d[((l_i + {NATURAL_32} 1) * a_s.item.dim + l_j).to_integer_32]
					a_s.item.d[(l_i * a_s.item.dim + l_j).to_integer_32] := (l_rc + l_newg) / (3).to_double
					l_grid_tot_j := l_grid_tot_j + (a_s.item.d[(l_i * a_s.item.dim + l_j).to_integer_32])
					l_i := l_i + 1
				end
				a_s.item.d[((l_bins - {NATURAL_32} 1) * a_s.item.dim + l_j).to_integer_32] := (l_newg + l_oldg) / (2).to_double
				l_grid_tot_j := l_grid_tot_j + (a_s.item.d[((l_bins - {NATURAL_32} 1) * a_s.item.dim + l_j).to_integer_32])
				l_tot_weight := (0).to_double
				l_i := (0).to_natural_32
				from
				until
					not (l_i < l_bins)
				loop
					l_weight[l_i.to_integer_32] := (0).to_double
					if a_s.item.d[(l_i * a_s.item.dim + l_j).to_integer_32] > (0).to_double then
						l_oldg := l_grid_tot_j / a_s.item.d[(l_i * a_s.item.dim + l_j).to_integer_32]
						l_weight[l_i.to_integer_32] := pow (((l_oldg - (1).to_double) / l_oldg) / (log (l_oldg)), a_s.item.alpha)
					end
					l_tot_weight := l_tot_weight + (l_weight[l_i.to_integer_32])
					l_i := l_i + 1
				end
				l_pts_per_bin := l_tot_weight / l_bins.to_real_64
				l_xnew := (0).to_double
				l_dw := (0).to_double
				l_i := (1).to_natural_32
				l_k := (0).to_natural_32
				from
				until
					not (not js.loop_done and then (l_k < l_bins))
				loop
					l_dw := l_dw + (l_weight[l_k.to_integer_32])
					l_xold := l_xnew
					l_xnew := a_s.item.xi[((l_k + {NATURAL_32} 1) * a_s.item.dim + l_j).to_integer_32]
					from
					until
						not (l_dw > l_pts_per_bin)
					loop
						l_dw := l_dw - l_pts_per_bin
						a_s.item.xin[l_i.to_integer_32] := l_xnew - ((l_xnew - l_xold) * l_dw) / l_weight[l_k.to_integer_32]
						l_i := l_i + 1
					end
					l_k := l_k + 1
				end
				l_k := (1).to_natural_32
				from
				until
					not (l_k < l_bins)
				loop
					a_s.item.xi[(l_k * a_s.item.dim + l_j).to_integer_32] := a_s.item.xin[l_k.to_integer_32]
					l_k := l_k + 1
				end
				a_s.item.xi[(l_bins * a_s.item.dim + l_j).to_integer_32] := (1).to_double
				l_j := l_j + 1
			end
		end

	reset_grid_values (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE])
		local
			l_i: NATURAL_32
			l_j: NATURAL_32
			l_dim: NATURAL_32
			l_bins: NATURAL_32
			js: JUMP_STATE
		do
			l_dim := a_s.item.dim
			l_bins := a_s.item.bins
			l_i := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_i < l_bins))
			loop
				l_j := (0).to_natural_32
				from
				until
					not (l_j < l_dim)
				loop
					a_s.item.d[(l_i * a_s.item.dim + l_j).to_integer_32] := 0.0
					l_j := l_j + 1
				end
				l_i := l_i + 1
			end
		end

	resize_grid (a_s: CE_POINTER [PD_GSL_MONTE_VEGAS_STATE]; a_bins: NATURAL_32)
		local
			l_j: NATURAL_32
			l_k: NATURAL_32
			l_dim: NATURAL_32
			l_pts_per_bin: REAL_64
			l_xold: REAL_64
			l_xnew: REAL_64
			l_dw: REAL_64
			l_i: INTEGER_32
			js: JUMP_STATE
		do
			l_dim := a_s.item.dim
			l_pts_per_bin := (a_s.item.bins).to_real_64 / a_bins.to_real_64
			l_j := (0).to_natural_32
			from
			until
				not (not js.loop_done and then (l_j < l_dim))
			loop
				l_xnew := (0).to_double
				l_dw := (0).to_double
				l_i := 1
				l_k := (1).to_natural_32
				from
				until
					not (not js.loop_done and then (l_k <= a_s.item.bins))
				loop
					l_dw := l_dw + (1.0)
					l_xold := l_xnew
					l_xnew := a_s.item.xi[(l_k * a_s.item.dim + l_j).to_integer_32]
					from
					until
						not (l_dw > l_pts_per_bin)
					loop
						l_dw := l_dw - l_pts_per_bin
						a_s.item.xin[l_i] := l_xnew - (l_xnew - l_xold) * l_dw
						l_i := l_i + 1
					end
					l_k := l_k + 1
				end
				l_k := (1).to_natural_32
				from
				until
					not (l_k < a_bins)
				loop
					a_s.item.xi[(l_k * a_s.item.dim + l_j).to_integer_32] := a_s.item.xin[l_k.to_integer_32]
					l_k := l_k + 1
				end
				a_s.item.xi[(a_bins * a_s.item.dim + l_j).to_integer_32] := (1).to_double
				l_j := l_j + 1
			end
			a_s.item.bins := a_bins
		end

end
